/*--PSEUDO-CODE--*/

if(locale('ru')){
  var $title="Сессии";
  var $users="Пользователи";
  var $token="Токен";
  var $browser="Браузер";
  var $remember="Запомнить";
  var $type="Тип";
  var $time="Время";
  var $status="Статус";
  var $online="Онлайн";
  var $offline="Офлайн";
  var $perm="Постоянная";
  var $connections="Подключения";
  var $temp="Временная";
  var $close="Закрыть";
  var $close_other="Закрыть все, кроме текущей";
  var $cancel="Отмена";
  var $filter="Фильтр";
  var $selected="Выбрано";
  var $current="Текущая";
  var $close_confirm="Закрыть выбранные сессии?<br>Пользователь будет разлогинен на соответствующих устройствах.";
}else{
  var $title="Sessions";
  var $users="Users";
  var $token="Token";
  var $type="Type";
  var $browser="Browser";
  var $remember="Remember";
  var $time="Time";
  var $status="Status";
  var $online="Online";
  var $offline="Offline";
  var $connections="Connections";
  var $perm="Permanent";
  var $temp="Temporary";
  var $close="Close";
  var $close_other="Close other sessions";
  var $cancel="Cancel";
  var $filter="Filter";
  var $selected="Selected";
  var $current="Current";
  var $close_confirm="Close selected sessions? <br>The user will be logged out on the appropriate devices.";
}


PROTO=widgetInterfaceProtoCommon;

TEMPLATE = createTemplate(
	``,
	`<div id="pult_header">
	<div class="pult_split pult_sep" id="filter_bar">
      <div><a-button id='btn_close_other' hidden="true" caption="${$close_other}"></a-button></div>
      <a-input class="pult_filter" clear_button=1 debounce=300 id='filter' placeholder=${$filter}></a-input>
    </div>
	
    <div class="pult_split pult_sep"  id="buttons_bar">
		<div class="pult_buttons">
			<a-button id='btn_close' caption=${$close}></a-button>
		</div>
		<a-button id='btn_clear' class="pult_cancel"  icon='/images/clear.svg' caption=${$cancel}></a-button>
    </div>
</div>

<div id="pult_content">
	<a-grid class="pult_grid" id="grid"></a-grid>
</div>
`,
);

PROPERTIES=['login','filter'];

ONCREATE=create;
ONDESTROY=destroy;
ONUPDATE=update;

let sub;
let grid;
let sessions=[];
let login;

function create(){

	pult.showHeader(self);
    grid=dom.grid;
		
	dom.btn_clear.onclick=function (){
		grid.unselect();
	};
	dom.btn_close.onclick=showClose;
	dom.btn_close_other.onclick=closeOther;

	dom.filter.onchange=function(e){
		self.filter=e.detail;
	}
}

function destroy(){
	pult.unsubscribe(sub);
}

function update(changes){
	for(let k in changes){
		if(k==='login') update_login();
		if(k==='filter') update_filter();
	}
};

function update_login(){
	login=self.login;
	if(login==='%profile%') login=undefined;
	if(login){
		pult.showPath(self,[
			[$system,'#interface/system'],
			[$users,'#interface/users'],
			[login,'#interface/user?login='+login],
			[$title,'#interface/session?login='+login]
		]);
		pult.showName(self,$title,"/images/sessions.svg");
		pult.selectMenuItem(self,'/system/users');
	}else{
		pult.showPath(self,[
			[$profile,'#interface/profile'],
			[$title,'#interface/sessions'],
		]);
		pult.showName(self,$title,"/images/sessions.svg");
		pult.selectMenuItem(self,'/profile');
	}
	drawTable();
	refreshButtons();
	load(login);
}

function update_filter(){
    let f=self.filter;
    dom.filter.value=f;
    dom.grid.filter = [f];
    refreshButtons();
}

function load(login){

	if(!login) login=pult.user.login;

	pult.request(self,'user_sessions',{login:login},function (req,res,err){
		if(err){
			pult.showModalError(self,err);
		}else{
			sessions=res;

			for(let i=0; i<sessions.length; i++){
				let s=sessions[i];
				s._online=(s.connected && !s.disconnected);
				s._time = s._online ? Time.timestampToDateTime(s.connected) : Time.timestampToDateTime(s.disconnected);
				s._status=!!s._online;
				s._remember=s.remember?`+ (${$perm})`:`— (${$temp})`;
				s._current=s.tok===pult.user.tok;
			}

			grid.records=sessions;
			refreshButtons();

			pult.unsubscribe(sub);
			sub=pult.subscribe('session_update',{
				login:login
			}, function (s,p){
				if(p.data){
					load(login);
				}
			});

		}
	});
}

function drawTable(){

	let columns=[];
	let is_own=!login || login===pult.user.login;

	console.log(login,pult.user.login);

	if(is_own){
		columns.push(
			{
				key:'_current',
				caption:$current,
				tooltip:$current,
				sort_order: 'desc',
				sort_cols:['_current','_online'],
			}
		);
	}

	columns.push(
		{
			key:'_status',
			caption:$status,
			sort_order: 'desc',
			sort_by_format: false,
			format:{
				true:$online,
				false:$offline,
			},
			onrender:function (value,record,column,index,row,cell,refresh) {
				row.classList.toggle("online",!!record._online);
				row.classList.toggle("offline",!record._online);
			},
		},
		{
			key:'_time',
			caption:$time,
		},
		{
			key:'browser',
			caption:$browser,
			
		},
		{
			key:'ip',
			caption:'IP',
		},
		{
			key:'tok',
			caption:$token,
		},
		{
			key:'remember',
			caption:$type,
			format:{
				true:$perm,
				false:$temp,
			}
		},
		{
			key:'connections',
			caption:$connections,
		},
		
	);
		

	grid.checkbox=true;
	grid.store_key="interface_sessions";
	grid.export_title=$title;
	grid.sort_column=1;
	grid.sort_order='desc';
	grid.columns=columns;
	grid.records=[];
	grid.css=`.row.online{color:var(--app-online-color)}.row.online a-icon{--color:var(--app-online-color)}.row.offline{color:var(--app-offline-color)}.row.offline a-icon{--color:var(--app-offline-color)}`;
	
	grid.onselect=function (e) {
		refreshButtons();
	};

	grid.drawColumns();

}


function refreshButtons(){

	dom.btn_close_other.hidden=true;
	
	if(!login || login===pult.user.login){
		for(let i=0; i<sessions.length; i++){
			let s=sessions[i];
			if(!s._current){
				dom.btn_close_other.hidden=false;
				break;
			}
		}
	}

	if(grid.selection.length){
		pult.showCount(self,grid.selection.length,$selected);
		dom.buttons_bar.style.display='';
		dom.filter_bar.style.display='none';
	}else{
		if(dom.filter.value){
        	pult.showCount(self,dom.grid.filter_count,$filter);
		}else{
			pult.showCount(self,dom.grid.filter_count);
		}
		dom.buttons_bar.style.display='none';
		dom.filter_bar.style.display='';
	}
}

function closeOther(){
	let a=[];
	for(let i=0; i<sessions.length; i++){
		let s=sessions[i];
		if(!s._current) a.push(i);
	}
	grid.select(a);
	showClose();
}


function showClose(){
	pult.showConfirm(self,$close_confirm,function (){
		let rr=[];
		for(let i=0; i<grid.selection.length; i++){
			let s=grid._records[grid.selection[i]];
			rr.push(['user_close_session',{
				login:login,
				tok:s.tok,
			}]);
		}

		
		pult.requests(self,rr,function(reqs,ress,errs){
			for(let i=0; i<errs.length; i++){
				let e=errs[i];
				if(e) pult.showError(self,e);
			}
			load(login);
		});

	});
}