/*--PSEUDO-CODE--*/

if(locale("ru")){
  var $title="Настройки";
  var $control_service="Управление сервисом";
  var $error="Ошибка";
  var $status_running="Запущен";
  var $status_stopped="Остановлен";
  var $status_disabled="Отключен";
  var $main_settings="Основные настройки";
  var $advanced_settings="Дополнительные настройки";
  var $save="Сохранить";
  var $cancel="Отменить";
  var $cancel_changes="Отменить изменения?";
  var $reset_to_default="Сбросить на настройки по умолчанию";
  var $reset_settings="Сбросить настройки данного сервиса на настройки по умолчанию?";
  var $reset_in_progress="Выполняется сброс настроек";
  
}else{
  var $title="Settings";
  var $control_service="Control service";
  var $error="Error";
  var $status_running="Running";
  var $status_stopped="Stopped";
  var $status_disabled="Disabled";
  var $main_settings="Main settings";
  var $advanced_settings="Advanced settings";
  var $save="Save";
  var $cancel="Cancel";
  var $cancel_changes="Cancel changes?";
  var $reset_to_default="Reset to default settings";
  var $reset_settings="Reset this service settings to default?";
  var $reset_in_progress="Reset settings in progress";
}

PROTO=widgetInterfaceProtoCommon;

TEMPLATE = createTemplate(
	`#header{padding:20px;display:flex;flex-direction:row;justify-content:space-between;border-bottom:solid 1px var(--app-form-color)}#header>div:first-child{max-width:500px}#save_buttons{display:none}#title{display:flex;flex-direction:row;align-items:center}#title>*{display:block}#title.running #icon{--color:var(--app-status-work-color)}#title.running #name{color:var(--app-status-work-color)}#title.error #icon{--color:var(--app-error-color)}#title.error #name{color:var(--app-error-color)}#icon{margin-right:10px;width:32px;height:32px;--color:var(--app-comment-color)}#name{font-size:24px;color:var(--app-comment-color)}#help{margin-top:10px;color:var(--app-comment-color)}#config{border-top:solid 1px var(--app-form-color);padding:20px}#control{margin-top:10px;white-space:nowrap}#control>a>*{vertical-align:text-top}#status{margin-top:10px}.pult_header>.pult_split{border-left:none!important}#form{padding:0}#form .confhead{padding:30px 20px 10px;font-size:14px;background-color:var(--app-section-header-color);color:var(--app-section-header-text-color);text-transform:uppercase}#form a-form-row{border-bottom:solid 1px var(--app-form-color);padding-top:10px;padding-bottom:10px;--caption-font-weight:bold;--comment-color:var(--app-comment-color);--comment-padding:10px 0 0 0}#form a-label{--text-color:var(--app-comment-color)}`,
	`
<div id="pult_header">
	<div class="pult_split">
		<div  class="pult_buttons">
		  <div id="common_buttons" class="pult_buttons">
			  <a-select class="pult_filter" id='services'></a-select>
			  <a-button id="btn_reset" caption="${$reset_to_default}"></a-button>
		  </div>	
		  <div id="save_buttons" class="pult_buttons">
			<a-button id="btn_save" palette="anode-button-active" caption="${$save}"></a-button>
			<a-button id="btn_cancel" palette="anode-button-empty" caption="${$cancel}"></a-button>
		  </div>
		</div>
	</div>
</div>

<div id="header">
	<div>
	  <div id="title">
		<a-icon id="icon" src="/images/service_disabled.svg"></a-icon>
		<span id="name">...</span>
	  </div>
	  <div id="help"></div>
	  <div id="status">status</div>
	</div>
	<div id="control">
	  <a id="control_link" href="#"><a-icon width="16" src="/images/services.svg"></a-icon> <span>${$control_service}</span></a>
	</div>
  </div>
  <a-form id="form">...</a-form>`,
);

PROPERTIES=[
	{
        key:'service',
        default:'server'
    }
];


ONCREATE=create;
ONUPDATE=update;
ONDESTROY=destroy;

let config;
let schema;
let services;
let service;
let version;
let serverSchema;
let reset;
let reset_interval;

let statuses={
    running: $status_running,
    stopped: $status_stopped,
    disabled:$status_disabled,
};


function create(){

    pult.showPath(self,[
        [$system,'#interface/system'],
        [$title,'#interface/settings'],
    ]);
    pult.showHeader(self,true);
    pult.showName(self,$title,"/images/settings.svg");
    pult.showCaption(self,null);
    pult.selectMenuItem(self,'/system/settings');


    dom.btn_save.onclick=function(){
        dom.form.submit();
    };

    dom.btn_cancel.onclick=function (e){
        pult.showConfirm(self,$cancel_changes,function (e){
            pult.needSave=false;
            pult.reloadWidget();
        });
    }

    dom.services.onchange=function (e){
        self.service=e.detail;
    }

    dom.btn_reset.onclick=function (e){
        pult.showConfirm(self,$reset_settings,function (e){
            console.log(service);
            pult.request(self,"system_reset_config",{
                service:service
            },function(req,res,err){
                if(err){
                    pult.showError(self,err);
                }else{
                    reset=true;
                    pult.showLoading(self,$reset_in_progress);
                }
            });
        });
    }
    
    reset_interval=setInterval(function (){
        if(reset){
            pult.request(self,"system_check_config",{
                service:service
            },function(req,res,err){
                if(err){
                    pult.showError(self,err);
                }else{
                    if(res){
                        pult.reloadWidget();
                    }
                }
            });
        }
    },1000);

}

function destroy(){
    clearInterval(reset_interval);
}

function update(changes){
	//console.log(changes);
	for(let k in changes){
		if(k==='service')		update_service(changes[k]);
	}
}



function update_service(s){
    service=self.__service;
    loadConfig();
}


function loadConfig(){

    pult.showLoading(self);

    Util.promises(function (resolve){
        pult.request(self,'system_services',null,function (req,res,err){
            if(err){
                services=[{name:'server'}];
            }else{
                services=res;
            }
            resolve();                
        });
    },function (resolve){
        pult.request(self,"system_config_schema",{service:'server'},function(req,res,err){ 
            if(err){
                pult.showModalError(self,err);
            }else{
                serverSchema=res;
                resolve();
            }
        });
    },function (resolve){
        pult.request(self,"system_config_schema",{service:service},function(req,res,err){ 
            if(err){
                pult.showModalError(self,err);
            }else{
                schema=res;
                if(schema && schema.CONFIGFACE_APP_INFO) schema.app=schema.CONFIGFACE_APP_INFO;
                if(schema && schema.GATEWAY_SETUP_APP_INFO) schema.app=schema.GATEWAY_SETUP_APP_INFO;
                if(schema && schema.SERVER_APP_INFO) schema.app=schema.SERVER_APP_INFO;
                
                if(schema && schema.CONFIGFACE_APP_INFO) version=2;
                else version=3;
                resolve();
            }
        });
    },function (resolve){
        pult.request(self,"system_config_data",{service:service},function(req,res,err){ 
            if(err){
                pult.showModalError(self,err);
            }else{
                config=res;
                resolve();                
            }
        });
    },function (){
        pult.hideLoading(self);
        showConfig();
    });
}

function ls(str){
    let s;
    try{
        s=JSON.parse(str);
    }catch(e){
        s=str;
    }
    return locale_string(s);
}

function getParamSchema(param){
    let ps;
    ps=schema[param];
    if(!ps){
        for(k in schema){
            if(k.indexOf("*")>=0){
                let r=k.split('*').join('.*');
                if(param.match(r)){
                    ps=schema[k];
                    break;
                }
            }
        }
    }
    if(!ps){
        let schema=serverSchema;
        ps=schema?.[param];
        if(!ps){
            for(k in schema){
                if(k.indexOf("*")>=0){
                    let r=k.split('*').join('.*');
                    if(param.match(r)){
                        ps=schema[k];
                        break;
                    }
                }
            }
        }
    }
    if(!ps) ps={advanced:1};
    ps.advanced=Number(ps.advanced);
    return ps;
}

function getControl(k){

    let sh=getParamSchema(k);
    let v=config[k];
    let c;
    let help=ls(sh.help);
    let at_file=v && v.startsWith('@FILE:');

    
    if(at_file){
        c={
            tag:'a-input',
            width:'100%',
            key:k,
            name:k,
            value:v,
            disabled:1
            //validation:['latinNum','maxLen:64','notEmpty'],
        };
    }else if(sh.type==='serverAlarmSoundFiles'){
        let id='control'+k;
        let btnid='button'+k;
        help=help+'<br>'+window.config.html_dir+'/sound/alarm*.mp3';
        c={
            tag:'div',
            key:k,
            html:[
                {
                    tag:'a-select',
                    sort:'asc',
                    id:id,
                    name:k,
                    value:v,
                },
                {
                    tag:'span',
                    html:' ',
                },
                {
                    tag:'a-button',
                    icon:'/images/sound_on.svg',
                    id:btnid,
                    onclick:function (e){

                        function stop(){
                            if(self.___playingAlarm){
                                dom[btnid].color='';
                                self.___audio.pause();
                                clearInterval(self.___playingAlarm);
                                self.___playingAlarm=null;
                            }
                        }

                        function play(p1){
                            dom[btnid].color='var(--app-alm-alarm-background-color)';
                            self.___audio.play();
                            setTimeout(function (){
                                dom[btnid].color='';
                            },p1*100);
                        }

                        if(self.___playingAlarm){
                            stop();
                        }else{
                            let audio = self.___audio;
                            if(!audio) audio = self.___audio = new Audio();
                            let file=dom[id].value;
                            audio.src=file;
                            let [p1,p0]=pult.getAlarmSoundPhases(file);
                            let n=3;
                            play(p1);
                            self.___playingAlarm=setInterval(function (){
                                play(p1);
                                n--;
                                if(n===0) stop();
                            },(p1+p0)*100);
                        }
                    }
                    // width:16,
                    // style:{
                    //     padding:'10px',
                    // }
                }
            ]
        };

        pult.request(self,'system_html_files',{
            folder:'sound'
        },function (req,res,err){
            if(err){
                pult.showModalError(self,err);
            }else{
                let list=[];
                for(let i=0; i<res.length; i++){
                    let s=res[i];
                    if(s.name.startsWith('alarm')){
                        list.push([`/html/sound/${s.name}`,s.name]);
                    }
                }
                dom[id].list=list;
            }
        });

    }else if(sh.type==='serverNotificationSoundFiles'){
        let id='control'+k;
        let btnid='button'+k;
        help=help+'<br>'+window.config.html_dir+'/sound/notification*.mp3';
        c={
            tag:'div',
            key:k,
            html:[
                {
                    tag:'a-select',
                    sort:'asc',
                    id:id,
                    name:k,
                    value:v,
                },
                {
                    tag:'span',
                    html:' ',
                },
                {
                    tag:'a-button',
                    icon:'/images/sound_on.svg',
                    id:btnid,
                    onclick:function (e){
                        self.___playingNotification=true;
                        let audio = self.___audio;
                        if(!audio) audio = self.___audio = new Audio();
                        let file=dom[id].value;
                        audio.src=file;
                        audio.play();
                    }
                    // width:16,
                    // style:{
                    //     padding:'10px',
                    // }
                }
            ]
        };

        pult.request(self,'system_html_files',{
            folder:'sound'
        },function (req,res,err){
            if(err){
                pult.showModalError(self,err);
            }else{
                let list=[];
                for(let i=0; i<res.length; i++){
                    let s=res[i];
                    if(s.name.startsWith('notification')){
                        list.push([`/html/sound/${s.name}`,s.name]);
                    }
                }
                dom[id].list=list;
            }
        });        

    }else if(sh.control==='select'){
        let lst=[];
        let list=sh.list.split('|');
        for(let i=0; i<list.length; i++){
            let li=list[i];
            let kv=li.split(':');
            if(kv.length===1) kv=[li,li];
            lst.push(kv);
        }
        c={
            tag:'a-select',
            key:k,
            name:k,
            value:v,
            list:lst,
        };
    }else if(sh.control==='checkbox'){
        c={
            tag:'a-label',
            caption: help || ' ',
            key:k,
            html:{
                tag:'a-checkbox',
                name:k,
                value:v,
            }
            //validation:['latinNum','maxLen:64','notEmpty'],
        };
        help=undefined;
    }else if(sh.control==='number'){
        c={
            tag:'a-number',
            key:k,
            name:k,
            value:v,
            width:sh.width,
            //validation:['latinNum','maxLen:64','notEmpty'],
        };
        if(sh.width) c.width=sh.width;
    

    }else{
        c={
            tag:'a-input',
            width:'100%',
            key:k,
            name:k,
            value:v,
            //validation:['latinNum','maxLen:64','notEmpty'],
        };
        if(sh.width) c.width=sh.width;
    }
    c.app_help=help;
    c.sh=sh;
    //console.log(k,sh);

    if(k==='enabled' && service==='server'){
        c.disabled=1;
    }

    return c;
}

function getParamList(){

    let params=[];
    if(version===3){
        for(let k in config){
            if(!schema.hidden){
                params.push(k);
            }
        }
    }else if(version===2){
        for(let k in schema){
            if(config[k]!==undefined){
                params.push(k);
            }
        }
        
        let keys=Object.keys(config);
        for(let i=0; i<keys.length; i++){
            let k=keys[i];
            if(params.indexOf(k)<0){
                params.push(k);
            }
        }
    }

    let pp=[];
    for(let i=0; i<params.length; i++){
        let p=params[i];
        let sh=getParamSchema(p);
        if(!sh.hidden){
            pp.push({name:p,sort:(sh.sort || 0)*1000000 +  (999000-i*1000)  + (999-(sh.index || 0))});
        }
    }
    Util.keySort(pp,'sort',true);
    //console.log(pp);

    params=[];
    for(let i=0; i<pp.length; i++){
        params.push(pp[i]['name']);
    }

    return params;
}


function showConfig(){
    let ss=[];
    for(let i=0; i<services.length; i++){
        ss.push(services[i].name);
    }
    dom.services.list=ss;
    dom.services.value=service;

    dom.name.innerHTML=service;
    dom.control_link.href=`#interface/system?service=${service}`;
    dom.help.innerHTML=schema.app && schema.app.help ? ls(schema.app.help) : '';

    let s=services.find(function (s){if(s.name===service) return s});

    if(s){
        if(s.status==='running') dom.icon.src='/images/service_running.svg';
        if(s.status==='stopped') dom.icon.src='/images/service_stopped.svg';
        if(s.status==='disabled') dom.icon.src='/images/service_disabled.svg';

        if(s.errors){
            let e=pult.errstr({
                code:s.last_error_code,
                message:s.last_error_message,
            });
            dom.title.className='error';
            dom.status.innerHTML=`<error>${$error}: ${e}</error>`;
        }else{
            dom.title.className=s.status;
            dom.status.innerHTML=statuses[s.status];
        }




        dom.form.innerHTML='';
        let fs=[];
        let params=getParamList(s);
        let groups=[];
    
        // let main=0;
        for(let i=0; i<params.length; i++){
            let k=params[i];
            let c=getControl(k);
            let gn;
            try{
                gn=JSON.parse(c.sh.group);
            }catch(e){}
            let g=locale_string(gn);
            if(!g){
                if(c.sh.advanced){
                    g=$advanced_settings;
                }else{
                    g=$main_settings;
                }
            }

            let group=groups.find(function (group){
                return group.name===g;
            });

            if(!group){
                group={
                    name:g,
                    controls:[],
                };
                groups.push(group);
                if(group.name===$main_settings) group.sort=0;
                else if(group.name===$advanced_settings) group.sort=1000;
                else group.sort=groups.length;
            }

            group.controls.push(c);

            // if(!c.sh.advanced && k!=='enabled'){
            //     main++;
            // }
        }

        groups.sort(function (a,b){
            return a.sort-b.sort;
        });

        for(let i=0; i<groups.length; i++){
            let g=groups[i];
            fs.push({
                tag:'div',
                class:'confhead',
                html:g.name,
            });
            for(let i=0; i<g.controls.length; i++){
                let c=g.controls[i];
                fs.push({
                    tag:'a-form-row',
                    caption:c.key,
                    comment:c.app_help,
                    visibleif:c.sh.visibleif,
                    html:c
                });
            }
        }

    
        Util.create(dom.form,fs);
    
        dom.form.onchange=function(e){
            pult.needSave=true;
            dom.save_buttons.style.display='inherit';
            dom.common_buttons.style.display='none';
        }
    
        dom.form.onsubmit=function(e){
            let cfg=e.detail;
            for(let k in cfg){
                if(cfg[k]===true) cfg[k]=1;
                if(cfg[k]===false) cfg[k]=0;
            }

            let params={};
            params.config={};
            params.config[service]=cfg;
            

            dom.btn_save.submit_start();
    
            pult.request(self,"system_set_config",params,function(req,res,err){
                setTimeout(function(){dom.btn_save.submit_reset()},300);
                if(err){
                    dom.form.reset();
                    showModalError(err);
                }else{
                    dom.form.complete();
                    pult.needSave=false;

                    setTimeout(function(){
                        dom.save_buttons.style.display='none';
                        dom.common_buttons.style.display='';
                    },300);
                }
            });
    
        }
    }

}
