/*--PSEUDO-CODE--*/

if(locale("ru")){
  var $title="Время";
  var $comment="Данное время применяется ко всем сервисам на SCADA-сервере и подключенных шлюзах, используется в работе расписаний, скриптов, логов и в журнале событий.";
  var $manual_comment="Изменение времени применяется только к сервисам данного SCADA-сервера в виде постоянной поправки к системному времени и&nbsp;не&nbsp;изменяет настройки времени в операционной системе.";
  var $sync_comment="При установке времени SCADA-сервера по времени ОС постоянная поправка будет сброшена и время SCADA-сервера будет точно соответствовать времени ОС.";
  var $cur_time="Текущее время SCADA-сервера";
  var $new_time_manual="Произвольное время";
  var $new_time_sync="Время ОС";
  var $apply="Применить";
  var $time_changed="Время изменено";
  var $link_manual="Установить произвольное время";
  var $link_sync="Установить время по времени операционной системы";
}else{
  var $title="Time";
  var $comment="This time applies to all services on the SCADA server and connected gateways, and is used in the operation of schedules, scripts, logs, and the event journal.";
  var $manual_comment="The time change is applied only to the services of this SCADA server as a permanent correction to the system time and does not change the time settings in the operating system.";
  var $sync_comment="When setting the SCADA server time to the OS time, the permanent correction will be reset and the SCADA server time will exactly match the OS time.";
  var $new_time_manual="Custom time";
  var $new_time_sync="OS Time";
  var $apply="Apply";
  var $time_changed="Time changed";
  var $link_manual="Set custom time";
  var $link_sync="Set time to operating system time";
}


PROTO=widgetInterfaceProtoCommon;

TEMPLATE = createTemplate(
	`#container{max-width:500px}#container h3{margin:0}#container #manual_time,#container #sync_time{display:none}#container a{cursor:pointer;display:inline-block;text-decoration:underline dashed;margin-bottom:10px}#container a-select{font-size:14px;--select-height:30px;vertical-align:middle;--disabled-text-color:var(--app-text-color);--disabled-border-color:var(--app-control-color)}#container a-time-picker{font-size:14px;height:30px;vertical-align:middle}#container .comment{margin-top:20px}`,
	`<div id="container">
    <h3>${$cur_time}</h1>
    <br>
    <a-time-picker id="cur_time" disabled="1" show_clock_button="0"></a-time-picker>&nbsp;<a-select id="cur_zone" disabled="1"></a-select>
    <br>
    <div class="comment">${$comment}</div>
    <br>
    <a-hr></a-hr>
    <br>
    <a href="javascript:void(0)" id="link_manual">${$link_manual}</a>
    <br>
    <a href="javascript:void(0)" id="link_sync">${$link_sync}</a>
    <br>
    <div id="manual_time">
        <a-hr></a-hr>
        <br>
        <h3>${$new_time_manual}</h1>
        <br>
        <a-time-picker id="new_time_manual" show_clock_button="0"></a-time-picker>&nbsp;<a-select id="new_zone_manual"></a-select>
        <br>
        <br>
        <a-button id="btn_set_manual" caption="${$apply}" palette='anode-button-active' ></a-button>
        <br>
        <div class="comment">${$manual_comment}</div>
        <br>
        <a-hr></a-hr>
    </div>

    <div id="sync_time">
        <a-hr></a-hr>
        <br>
        <h3>${$new_time_sync}</h1>
        <br>
        <a-time-picker id="new_time_sync" disabled="1" show_clock_button="0"></a-time-picker>&nbsp;<a-select disabled="1" id="new_zone_sync"></a-select>
        <br>
        <br>
        <a-button id="btn_set_sync" caption="${$apply}" palette='anode-button-active' ></a-button>
        <br>
        <div class="comment">${$sync_comment}</div>
        <br>
        <a-hr></a-hr>
    </div>

</div>`,
);

ONCREATE=create;
ONDESTROY=destroy;

let int;
let dt=0;
let dt_os=0;
let timestamp_manual=0;
let timestamp_sync=0;


function create(){

    dom._interface_content.classList.add('pult_small_form');

    pult.showPath(self,[
        [$system,'#interface/system'],
        [$title,'#interface/time'],
    ]);
    pult.showHeader(self,true);
    pult.showName(self,$title,"/images/time.svg");
    pult.showCaption(self,null);
    pult.selectMenuItem(self,'/system/time');


    requestTime();

    dom.cur_zone.list=Time.utcCityZones;
    dom.new_zone_manual.list=Time.utcCityZones;
    dom.new_zone_sync.list=Time.utcCityZones;

    dom.link_manual.onclick=function (){
        dom.sync_time.style.display='';
        if(dom.manual_time.style.display){
            dom.manual_time.style.display='';
        }else{
            dom.manual_time.style.display='block';
        }
    }

    dom.link_sync.onclick=function (){
        dom.manual_time.style.display='';
        if(dom.sync_time.style.display){
            dom.sync_time.style.display='';
        }else{
            dom.sync_time.style.display='block';
        }
    }

    dom.btn_set_manual.onclick=function(){

        let nz=dom.new_zone_manual.value*1;
        let cz=dom.new_time_manual.zone*1;
        let ct=dom.new_time_manual.value;
        let nt=ct + (cz-nz)*3600;

        pult.request(self,"system_set_time",{
            time:nt,
            zone:nz*60,
        },function(req,res,err){
            if(err){
                pult.showModalError(self,err);
            }else{
                window.Time.syncRequest();
                Time.read();
                requestTime();
                pult.showTooltip(self,$time_changed);
            }
        });
    }

    dom.btn_set_sync.onclick=function(){

        pult.request(self,"system_reset_time",null,function(req,res,err){
            if(err){
                pult.showModalError(self,err);
            }else{
                window.Time.syncRequest(); 
                Time.read();
                requestTime();
                pult.showTooltip(self,$time_changed);
            }
        });
    }


    int=setInterval(function(){
        {
            let ts=Time.timestamp-dt;
            if(ts!==timestamp_manual){
                timestamp_manual=ts;
                dom.cur_time.value=timestamp_manual;
            }
        }
        
        {
            let ts=Time.timestamp-dt_os;
            if(ts!==timestamp_sync){
                timestamp_sync=ts;
                dom.new_time_sync.value=timestamp_sync;
            }
        }
    },100);

}


function destroy(){
    clearInterval(int);
}


function requestTime(){
    pult.request(self,"system_time",null,function(req,res,err){
        if(err){
            pult.showError(self,err);
        }else{

            let t=res.time/1000;
            let z=res.zone/60;

            let t_os=res.os_time/1000;
            let z_os=res.os_zone/60;
            dt=Time.timestamp-t;
            dt_os=Time.timestamp-t_os;

            dom.cur_zone.value=z;
            rem_zone=res.zone*60;
           
            dom.cur_time.value=Time.timestamp;
            dom.cur_time.zone=z;

            dom.new_time_manual.value=Time.timestamp;
            dom.new_time_manual.zone=z;
            dom.new_zone_manual.value=z;
            
            dom.new_time_sync.value=Time.timestamp-dt_os;
            dom.new_time_sync.zone=z_os;
            dom.new_zone_sync.value=z_os;

        }
    });
}