// ATTENTION!
// 'undefined' object key not allowed!
// 'default' object key not allowed!
// ';' required after all 'let a={...};'

let schema;

(function (){

	
	function change(o,p){
	let x={};
	for(let k in o){
		x[k]=o[k];
	}
	for(let k in p){
		x[k]=p[k];
	}
	return x;
}
	let commonParams={
    tags:{
        key:'tags',
        caption:{en:'Tags',ru:'Теги'},
        control:'edit',
        wordwrap: true,
        validation:[
            'tags'
        ],
        help:{
            ru:`
            Имена тегов, через запятую, без пробелов. Теги ограничивают список доступных пользователю переменных и тревог и используются в функциях агрегации значений (тревоги и виртуальные переменные).

            Каждый пользователь получает доступ только к тем переменных, чей результирующий набор тегов (к переменным применяются теги прототипа, узла, папок и шлюза) будет соответствовать фильтру тегов хотя бы одной из групп, в которых состоит пользователь.

            Например:
            <em>ventilation,floor3,fire_alarm</em>
            `,
            en:`
            Tag names, separated by commas, without spaces. Tags limit the list of variables and alarms available to the user and are used in value aggregation functions (alarms and virtual variables).

            Each user gets access only to those variables whose resulting set of tags (prototype, node, folder, and gateway tags are applied to the variables) matches the tag filter of at least one of the groups to which the user belongs.

            For example:
            <em>ventilation,floor3,fire_alarm</em>
            `
        }
    },

    inactive:{
        key: "inactive",
        tooltip: {
            ru: "Активный",
            en: "Active",
        },
        control: "checkbox",
        invert: true,
        reload: true,
        table: {
            caption: {
                ru: "Акт.",
                en: "Act.",
            },
            tooltip: {
                ru: "Активный",
                en: "Active",
            },
        },
        props: {
            caption: {
                ru: "Активный",
                en: "Active",
            },
        },
        help:{
            ru:`
            Активный объект участвует в сборке (для объекта генерируются переменные) и отображается в интерфейсе при включенном параметре 'Видимый'.
            Неактивный объект не участвует в сборке и не отображается в интерфейсе.
            `,
            en:`
            An active object participates in the building (variables are generated for the object) and is displayed in the interface when the 'Visible' option is enabled. 
            An inactive object does not participate in the building and is not displayed in the interface.
            `
        }
    },

    invisible:{
        key: "invisible",
        tooltip: {
            ru: "Видимый",
            en: "Visible",
        },
        control: "checkbox",
        invert: true,
        reload: true,
        visibleIf: '!inactive',
        table: {
            caption: {
                ru: "Вид.",
                en: "Vis.",
            },
            tooltip: {
                ru: "Видимый",
                en: "Visible",
            },
        },
        props: {
            caption: {
                ru: "Видимый",
                en: "Visible",
            },
        },
        help:{
            ru:`
            Объект отображается в интерфейсе
            `,
            en:`
            The object is displayed in the interface
            `
        }
    },
    
    hide_alarm_panel:{
        key: "hide_alarm_panel",
        caption:{ru:"Панель тревог",en:"Alarm panel"},
        control:"select",
        list:[
            [false,{ru:'Показать',en:'Show'}],
            [true,{ru:'Скрыть',en:'Hide'}],
        ],
        def:false,
        undef:true,
    },

    show_header:{
        key: "show_header",
        group:{ru:"Разное",en:"Miscellaneous"},
        caption:{ru:"Заголовок окна",en:"Window header"},
        control:"select",
        list:[
            [true,{ru:'Показать',en:'Show'}],
            [false,{ru:'Скрыть',en:'Hide'}],
        ],
        def:true,
        undef:true,
    },

    unit:{
        key:'unit',
        caption:{en:'Unit',ru:'Ед. изм.'},
        control:'edit',
        list:['%', '°', '°C', 'K', 'бар', 'Па', 'А', 'В', 'кВ', 'Вт', 'Гц', 'кВт', 'кВт⋅ч', 'л', 'м', 'м/с', 'м²', 'м³', 'мин', 'рад', 'с', 'ч'],
        help:{
            ru:`
            Единица измерения
            `,
            en:`
            Variable unit
            `
        }
    },

    cron:{
        key:'cron',
        caption:'Cron',
        control:'edit',
        def:"0 0 12 * * *",
        formfunc:'prototypeShowCron',
        help:{
            ru:`
			Опрос в соответствии с расписанием в формате Cron. Доступны два варианта:

			- 5 разрядов (от минут до дней недели)
			- 6 разрядов (от секунд до дней недели)

			Например:
			0 * * * * * - в нулевую секунду каждой минуты
			0 * * * * - в нулевую минуту каждого часа
			5 4 * * sun - в 4:05 в воскресенье
			0 0 1,15 * * - в 00:00 первого и пятнадцатого числа
			`,
            en:`
			Polling according to a schedule in Cron format.	Two options are available:

			- 5 digits (from minutes to days of the week)
			- 6 digits (from seconds to days of the week)

			For example:
			0 * * * * * - at the zero second of every minute
			0 * * * * - at zero minute of every hour
			5 4 * * sun - at 4:05 on Sunday
			0 0 1.15 * * - at 00:00 on the first and fifteenth
			`,
        }
    },

    prototype:{
        key: "prototype",
        caption: {
            ru: "Прототип",
            en: "Prototype",
        },
        extra_properties: "prototypes",
        dictionary_link: true,
        dictionary: "prototypes",
        dictionary_link: '#editor/prototype?path=${project}/prototypes/${value}',
        control: "select",
        help:{
            ru:`
            Узел получит набор переменных, параметров, отчетов и скриптов, определенных в прототипе`,
            en:`
            The node will receive a set of variables, parameters, reports and scripts defined in the prototype
            `,
        }
    },

    status_var:{
        key: "status_var",
        caption: {
            ru: "Переменная статуса",
            en: "Status var",
        },
        validation: [
            "latinNum",
        ],
        help:{
            ru:`Имя переменной, без знака $, значение которой будет использоваться для индикации статуса узла в интерфейсе. Значения переменной интерпритируются следующим образом:

            0 - офлайн (серый)
            1 - онлайн (голубой)
            2 - работа (зеленый)
            3 - предупреждение (оранжевый)
            4 - авария (красный)
            5 - сервис (филетовый)
            
            В качестве переменной используйте виртуальную переменную с выражением вида:

            status($online,$work,$warning,$alarm,$service);

            и параметром 'Недостоверно'='Ноль'.

            Здесь аргументами функции status() являются пять переменных, ненулевые значения которых будут интерпретированы функцией status() в соответствующий код статуса (0-5). Имена переменных могут быть произвольными, важен их порядок в списке аргументов. Если какой-то статус неприменим к узлу, соответствующую ему переменную можно заменить нулем, напрмер:

            status($online);
            status($online,0,$warning);
            status($online,0,0,$alarm);

            Значение параметра 'Недостоверно' у статусной переменной необходимо установить в значение 'Ноль'. Это обеспечит корректную обработку выражения, без возникновения ошибки DRV_ERROR_UNRELIABLE_VAR при ошибке в одной или нескольких наблюдаемых переменных. 

            `,
            en:`
            The name of the variable, without the $ sign, whose value will be used to indicate the node status in the interface. The variable values ​​are interpreted as follows:

            0 - offline (gray)
            1 - online (blue)
            2 - working (green)
            3 - warning (orange)
            4 - alarm (red)
            5 - service (violet)

            As a variable, use a virtual variable with an expression of the form:

            status($online,$work,$warning,$alarm,$service);

            and the parameter 'Unreliable'='Zero'.

            Here, the arguments of the status() function are five variables, non-zero values ​​of which will be interpreted by the status() function into the corresponding status code (0-4). The names of the variables can be arbitrary, their order in the argument list is important. If a status is not applicable to a node, the variable corresponding to it can be replaced with zero, for example:

            status($online);
            status($online,0,$warning);
            status($online,0,0,$alarm);

            The 'Unreliable' parameter value for the status variable must be set to 'Zero'. This will ensure that the expression is processed correctly, without causing a DRV_ERROR_UNRELIABLE_VAR error if there is an error in one or more monitored variables.
            `
        }
    },

    icon:{
        key: "icon",
        caption: {
            ru: "Иконка",
            en: "Icon",
        },
        bottom:true,
        dictionary: "icons",
        noCache: true,
        filesubst: true,
        reload: true,
        control: "edit",
        help: {
            ru: "Путь к объекту-иконке, svg-код, URL или Data:URI изображения\n\nДля путей допускаются подстановки:\n<ul>\n<li>%files% - раздел Файлы текущего проекта</li>\n<li>%resources% - папка Ресурсы текущего объекта</li>\n<li>%images% - папка /images сервера</li>\n</ul>\nПримеры:\n<ul>\n<li>%files%/home.svg</li>\n<li>%resources%/icons/plc.png</li>\n<li>%images%/user.svg</li>\n<li>/projects/Project1/files/cooler.gif</li>\n<li>&lt;svg viewBox=\\\"0 0 48 48\\\"&gt;&lt;path d=\\\"m18 18 10 13 10-13h-4l-6 8-6-8z\\\"/&gt;&lt;/svg&gt; \n</li>\n<li>http://myserver.com/someimage.png</li>\n</ul>",
            en: "Path to the icon object, svg code, URL or Data:URI of the image\n\nSubstitutions are allowed for paths:\n<ul>\n<li>%files% - section Files of the current project</li>\n<li>%resources% - folder Resources of the current object</li>\n<li>%images% - server /images folder</li>\n</ul>\nExamples:\n<ul>\n<li>%files%/home.svg</li>\n<li>%resources%/icons/plc.png</li>\n<li>%images%/user.svg</li>\n<li>/projects/Project1/files/cooler.gif</li>\n<li>&lt;svg viewBox=\\\"0 0 48 48\\\"&gt;&lt;path d=\\\"m18 18 10 13 10-13h-4l-6 8-6-8z\\\"/&gt;&lt;/svg&gt;\n</li>\n<li>http://myserver.com/someimage.png</li>\n</ul>",
        },
        table: {
            hide: true,
        },
    },

    round:{
        key:'round',
        caption:{en:'Rounding',ru:'Округление'},
        control:'edit',
		undef:true,
		list:[
			0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16
		],
        help:{
            ru:`
            Округление значения до указанного количества знаков после запятой
            `,
            en:`
            Rounding the value to the specified number of decimal places
            `
        }
    },

    comment:{
        key: "comment",
        caption:{ru:'Комментарий',en:'Comment'},
        bottom: true,
        control: "edit",
        spellcheck: true,
        multiline:true,
        wordwrap:true
    },

};
	let commonHelp={
    alarmTemplate:{
        ru:`
            В шаблоне допускается подстановка следующих параметров:

            %project_name% - имя проекта
            %node_path% - путь узла
            %node_name% - имя узла
            %alarm_description% - описание тревоги
            %alarm_class% - класс тревоги
            %alarm_user% - исполнитель

            Если шаблон не задан, будет использоваться шаблон по умолчанию (заданный в настройках проекта или в настройках группы)

        `,
        en:`
            The template allows the following parameters to be substituted:
            
            %project_name% - project name
            %node_path% - node path
            %node_name% - node name
            %alarm_description% - description of the alarm
            %alarm_class% - alarm class
            %alarm_user% - executor

            If no template is specified, the default template (which set in the project settings or group settings) will be used.
        `
    },

    expression:{
        ru:`<b>Математическое выражение с подстановкой переменных узла. Например:</b>

            sqrt(pow($var1,2)+pow($var2,2))

            Имена переменных необходимо предварять символом $.

            При ошибке одной из переменных в выражении, значение выражения будет равно ошибке DRV_ERROR_UNRELIABLE_VAR (недостоверное значение).

            Для подстановки не значения переменной, а номера ошибки используйте суффикс __error (два подчеркивания), например:

            iif($sensor__error,$min_power,$max_power)

            Так же доступны суффиксы:
            __time - Timestamp (в миллисекундах) последнего изменения статуса переменной
            __value - значение (позволяет всегда получать последнее достоверное значение переменной и игнорировать ошибку).

            Доступные функции:

            abs(X)
            acos(X)
            acosh(X)
            asin(X)
            asinh(X)
            atan(X)
            atanh(X)
            atan2(X,Y)
            bit(X,Y)    - возвращает 1 если в X установлен бит Y
            bool(X)     - возвращает 0 если X==0, иначе 1
            cbrt(X)
            ceil(X)
            cos(X)
            cosh(X)
            double(A,B,C,D,E,F,G,H) - конвертирует 8 байт (A,B,C,D,E,F,G,H) в число формата IEEE-754 Floating Point 64-bit (Double precision)
            exp(X)
            float(A,B,C,D) - конвертирует 4 байта (A,B,C,D) в число формата IEEE-754 Floating Point 32-bit
            floor(X)
            hi16(X) - возвращает старшие 8 бит 16-битного числа
            hi32(X) - возвращает старшие 16 бит 32-битного числа
            hypot(X,Y)
            iif(X,Y,Z)  - возвращает Y если X, иначе Z
            lo16(X) - возвращает младшие 8 бит 16-битного числа
            lo32(X) - возвращает младшие 16 бит 32-битного числа
            log(X)
            log10(X)
            pow(X,Y)
            round(X)    - округляет X до целого
            round2(X,Y) - округляет X до Y знаков после запятой
            sin(X)
            sinh(X)
            sqrt(X)
            tan(X)
            tanh(X)

            Операторы:

            -X      - унарный минус
            !X      - унарное логическое НЕ
            ^X      - унарное побитовое НЕ
            X ** Y  - возведение X в степень Y
            X * Y   - умножение X на Y
            X / Y   - деление X на Y
            X % Y   - остаток от деления X на Y
            X + Y   - сложение X и Y
            X - Y   - вычитание Y из X
            X << Y  - побитовый сдвиг X на Y бит влево
            X >> Y  - побитовый сдвиг X на Y бит вправо
            X < Y   - 1, если X меньше Y, иначе - 0
            X <= Y  - 1, если X меньше или равно Y, иначе - 0
            X > Y   - 1, если X больше Y, иначе - 0
            X >= Y  - 1, если X больше или равно Y, иначе - 0
            X == Y  - 1, если X ревен Y, иначе - 0
            X != Y  - 1, если X не равен Y, иначе - 0
            X & Y   - побитовое И
            X | Y   - побитовое ИЛИ
            X ^ Y   - побитовое исключающее ИЛИ
            X && Y  - логическое И
            X || Y  - логическое ИЛИ

        `,
        en:`<b>Mathematical expression with node variable substitution. For example:</b>

            sqrt(pow($var1,2)+pow($var2,2))

            Variable names must be preceded by the $ symbol.
            
            If there is an error in one of the variables in the expression, the value of the expression will be equal to the error DRV_ERROR_UNRELIABLE_VAR.

            To substitute not the variable value, but the error number, use the __error suffix (two underscores), for example:

            iif($sensor__error,$min_power,$max_power)

            Suffixes are also available:
            __time - Timestamp (in milliseconds) of the last variable status change
            __value - value (allows you to always get the last valid value of the variable and ignore the error).

            Available functions:

            abs(X)
            acos(X)
            acosh(X)
            asin(X)
            asinh(X)
            atan(X)
            atanh(X)
            atan2(X,Y)
            bit(X,Y)    - returns 1 if bit Y in number X is set
            bool(X)     - returns 0 if X==0, otherwise 1
            cbrt(X)
            ceil(X)
            cos(X)
            cosh(X)
            double(A,B,C,D,E,F,G,H) - converts 8 bytes (A,B,C,D,E,F,G,H) into a number in IEEE-754 Floating Point 64-bit format (Double precision)
            exp(X)
            float(A,B,C,D) - converts 4 bytes (A,B,C,D) to an IEEE-754 Floating Point 32-bit number
            floor(X)
            hi16(X) - returns the higher 8 bits of a 16-bit number
            hi32(X) - returns the higher 16 bits of a 32-bit number
            hypot(X,Y)
            iif(X,Y,Z)  - returns Y if X, otherwise Z
            lo16(X) - returns the lower 8 bits of a 16-bit number
            lo32(X) - returns the lower 16 bits of a 32-bit number
            log(X)
            log10(X)
            pow(X,Y)
            round(X)    - rounds X to an integer
            round2(X,Y) - rounds X to Y decimal places
            sin(X)
            sinh(X)
            sqrt(X)
            tan(X)
            tanh(X)

            Operators:

            -X      - unary minus
            !X      - unary logical NOT
            ^X      - unary bitwise NOT
            X ** Y  - raising X to the power Y
            X * Y   - multiplying X by Y
            X / Y   - division of X by Y
            X % Y   - remainder of division of X by Y
            X + Y   - addition of X and Y
            X - Y   - subtract Y from X
            X << Y  - bitwise shift X by Y bits to the left
            X >> Y  - bitwise shift X by Y bits to the right
            X < Y   - 1 if X is less than Y, otherwise - 0
            X <= Y  - 1 if X is less than or equal to Y, otherwise - 0
            X > Y   - 1 if X is greater than Y, otherwise - 0
            X >= Y  - 1 if X is greater than or equal to Y, otherwise - 0
            X == Y  - 1 if X is equal to Y, otherwise - 0
            X != Y  - 1 if X is not equal to Y, otherwise - 0
            X & Y   - bitwise AND
            X | Y   - bitwise OR
            X ^ Y   - bitwise exclusive OR
            X && Y  - logical AND
            X || Y  - logical OR
        `
    }
};
	let mimic_menu={
	typename: {
		ru: "Элемент меню",
		en: "Menu item",
	},
	dontApplyCommonProps: true,
	properties: [
		{
			key: "type",
			caption: {ru: "Тип",en: "Type"},
			control:'select',
			list:[
				['custom',{ru:'Произвольный элемент',en:'Custom element'}],
				['settings',{ru:'Настройки',en:'Settings'}],
				['nodes',{ru:'Узлы',en:'Nodes'}],
				['vars',{ru:'Переменные',en:'Vars'}],
				['submimic',{ru:'Связанная мнемосхема',en:'Linked mimic'}],
				['group',{ru:'Группа элементов',en:'Items group'}],
				['separator',{ru:'Разделитель',en:'Separator'}],
			],
			def:'custom'
		},
		{
			key: "format",
			caption: {ru: "Формат",en: "Format"},
			control:'select',
			list:[
				['icon_text',{ru:'Иконка + Надпись',en:'Icon + Caption'}],
				['icon',{ru:'Иконка',en:'Icon'}],
				['text',{ru:'Надпись',en:'Caption'}],
			],
			def:'icon_text',
			visibleIf:'type!=separator'
		},
		change(commonParams.icon,{
			reload:false,
			bottom:false,
			visibleIf:'type!=separator;format!=text',
			table:{
				hide:false,
				control:'input'
			},
			help:{
				ru:`
				Путь к объекту-иконке,
				Допускается использование выражений и подстановка узлов из параметров мнемосхемы.
				Например:

				$node1.icon
				`,
				en:`
				Path to the icon object,
				Expressions and substitution of nodes from the mimic parameters are allowed.
				For example:

				$node1.icon
				`,
			}
		}),
		{
			key: "text",
			caption: {ru: "Надпись",en: "Caption"},
			visibleIf:'type!=separator;format!=icon',
			help:{
				ru:`
				Надпись в элементе меню.
				Допускается использование выражений и подстановка узлов из параметров мнемосхемы.
				Примеры: 

				$node1.title
				Узел $node1.prefix
				Щит $node1.params.boxname №$node.params.boxnumber
				`,
				en:`
				Caption in the menu item.
				Substitution of nodes from the mimic parameters is allowed.
				Examples:

				$node1.title
				Node $node1.prefix
				Panel $node1.params.boxname #$node.params.boxnumber
				`,
			}
		},
		{
			key: "group",
			caption: {ru: "В группе",en: "In group"},
			visibleIf:'type!=group',
			control:'checkbox',
			help:{
				ru:`
				Если опция включена, элемент меню будет размещен в раскрывающемся списке группы элементов.
				
				Группа элементов должна быть предварительно создана и находиться в списке над элементами группы.
				`,
				en:`
				If the option is enabled, the menu item will be placed in the drop-down list of the group of items.

				The group of items must be previously created and located in the list above the group items.
				`,
			}
		},

		{
			key: "submimic",
			caption: {ru: "Св. мнемосхема",en: "Linked mimic"},
			visibleIf:'type=submimic',
			dictionary: "mimics",
			dictionary_link: '#editor/mimic?path=${project}/mimics/${value}',
			control: "select",
		},
		

		{
			key: "tooltip",
			caption: {ru: "Подсказка",en: "Tooltip"},
			visibleIf:'type!=separator',
			help:{
				ru:`
				Всплывающая подсказка (появляется при наведении курсора на элемент меню).
				Допускается использование выражений и подстановка узлов из параметров мнемосхемы.
				Например:

				Переход к экрану узла $node.title
				`,
				en:`
				Tooltip (appears when you hover your cursor over a menu item).
				Substitution of nodes from the mimic parameters is allowed.
				For example:

				Go to node screen $node.title
				`,
			}
		},
		{
			key: "visible",
			caption: {ru: "Видимый",en: "Visible"},
			control:'input',
			def:1,
			help:{
				ru:`
				1 - элемент меню отображается, 0 - скрыт.
				Допускается использование выражений и подстановка узлов из параметров мнемосхемы.
				Например:

				$node1.params.show_node1_link
				`,
				en:`
				1 - menu item is displayed, 0 - hidden.
				Expressions and substitution of nodes from the mimic parameters are allowed.
				For example:

				$node1.params.show_node1_link
				`,
			}
		},
		{
			key: "enabled",
			caption: {ru: "Активный",en: "Active"},
			control:'input',
			visibleIf:'type!=separator',
			def:1,
			help:{
				ru:`
				1 - элемент меню активный, 0 - неактивный (отображается неактивным и не реагирует на действия пользователя).
				Допускается использование выражений и подстановка узлов из параметров мнемосхемы.
				Например:

				{{!$node1.params.enable_node1_link}}
				`,
				en:`
				1 - menu item is displayed, 0 - hidden.
				Expressions and substitution of nodes from the mimic parameters are allowed.
				For example:

				$node1.params.show_node1_link
				`,
			}
		},

		{
			key: "href",
			caption: {ru: "Ссылка",en: "Link"},
			visibleIf:'type=custom',
		},
		
		
		
		commonParams.comment,
		
	],
};

	let mimic_param={
	typename: {
		ru: "Параметр мнемосхемы",
		en: "Mimic parameter",
	},
	dontApplyCommonProps: true,
	properties: [
		{
			key:'name',
			caption:{en:'Name',ru:'Имя'},
			control:'input',
			validation: [
				"attrName",
			],
		},
		{
			key:'type',
			caption:{en:'Type',ru:'Тип'},
			control:'select',
			def:'string',
			list:[
				['string',{en:'String',ru:'Строка'}],
				['number',{en:'Number',ru:'Число'}],
				['boolean',{en:'Boolean',ru:'Булево'}],
				['object',{en:'Node',ru:'Узел'}],
			],
		},
		{
			key:'proto',
			caption:{en:'Prototype',ru:'Прототип'},
			control:'select',
			dictionary:'prototypes',
			visibleIf:'type=object',
			hide:true,
		},
		{
			key:'default',
			caption:{en:'Default',ru:'По умолчанию'},
			control:'input',
			visibleIf:'type!=object',
			help: {en:'Default Parameter value',ru:'Значение параметра по умолчанию'}
		},
	],
};
	let prototype_param={
	typename: {
		ru: "Параметр узла",
		en: "Node parameter",
	},
	dontApplyCommonProps: true,
	properties: [
		{
			key:'group',
			caption:{ru:'Группа свойств',en:'Property group'},
			control:'input',
			def:{ru:'Параметры узла',en:'Node parameters'},
			help: {
				ru:'Заголовок группы свойств, отображаемый в свойствах узла',
				en:'Property group title displayed in node properties'
			}
		},
		{
			key: "name",
			caption: {
				ru: "Имя",
				en: "Name",
			},
			unique: true,
			validation: [
				"notEmpty",
				"latinNum",
			],
			help: {
				ru: `
					Определенные в прототипе параметры будут наследованы узлом, связанным с данным прототипом.
					
					Значения данных параметров, указанные в свойствах узла, передаются в соответствующие свойства переменных через подстановку строки вида %param_name%.
					
					Например:
					1) в параметрах прототипа заводится параметр host
					2) у переменных драйвера MBTCP в свойстве Хост указывается %host%
					3) создается узел на основе данного прототипа
					4) узел имеет наследует свойство host от прототипа
					5) задаем в свойствах узла значение host=192.168.0.210
					6) Modbus-переменные узла получают в свойство Хост значение 192.168.0.210
				`,
				en: `
					The parameters defined in the prototype will be inherited by the node associated with this prototype.
					
					The values of these parameters specified in the properties of the node are transferred to the corresponding properties of the variables through the substitution of a string of the form %param_name%.
					
					For example:
					1) the host parameter is set in the prototype parameters
					2) MBTCP driver variables have %host% in the Host property
					3) a node is created based on this prototype
					4) the node inherits the host property from the prototype
					5) set the host properties to host=192.168.0.210
					6) Modbus node variables receive the value 192.168.0.210 in the Host property
				`,
			},
		},

		{
			key: "caption",
			caption: {
				ru: "Подпись",
				en: "Caption",
			},
			help: {
				ru: `
					Отображаемое имя параметра
				`,
				en: `
					Parameter display name
				`,
			},
		},

		{
			key:'hidden',
			caption:{ru:'Скрытый',en:'Hidden'},
			control:'checkbox',
			help: {
				ru:'Скрытый параметр. Если опция включена, параметр не будет отображаться в интерфейсе.',
				en:'Hidden parameter. If the option is enabled, the option will not appear in the interface.'
			}
		},
		{
			key: "type",
			caption: {
				ru: "Тип",
				en: "Type",
			},
			control: "select",
			def:'string',
			list: [
				[
					"string",
					{
						en: "String",
						ru: "Строка",
					},
				],
				[
					"number",
					{
						en: "Number",
						ru: "Число",
					},
				],
				[
					"checkbox",
					{
						en: "Checkbox",
						ru: "Чекбокс",
					},
				],
				[
					"list",
					{
						en: "List",
						ru: "Список",
					},
				],
				[
					"dictionary",
					{
						en: "Dictionary",
						ru: "Справочник",
					},
				],
			],
		},
		{
			key:'invert',
			caption:{ru:'Инвертировать',en:'Invert'},
			control:'checkbox',
			visibleIf:'type=checkbox',
			table:{
				caption:{ru:'Инв.',en:'Inv.'},
			},
			help: {
				ru:'Инвертирует значение чекбокса',
				en:'Invert checkbox value'
			}
		},
		{
			key: "dictionary",
			caption: {
				ru: "Справочник",
				en: "Dictionary",
			},
			visibleIf: "type=dictionary",
			control: "select",
			dictionary: "dictionaries",
			dictionary_link: '#editor/dictionary?path=${project}/dictionaries/${value}',
		},
		{
			key: "list",
			caption: {
				ru: "Список",
				en: "List",
			},
			hidden: true,
			visibleIf: "type=list",
			control: 'edit',
			minheight: 50,
			maxheight: 200,
			multiline: true,
			help: {
				en: `
				List of valid parameter values, one on each line.

				Formats:
				1) list of pairs Meaning: Caption
				2) list of Values

				Values and labels can be numbers or strings.

				Format example 1:

				0:Disabled
				1:Enabled
				2:Auto

				Format example 2:

				192.168.0.101
				192.168.0.102
				192.168.0.103
				`,
				ru: `
				Список допустимых значений параметра, по одному на каждой строке.

				Форматы: 
				1) список пар Значение:Надпись
				2) список Значений

				Значения и надписи могут быть числами и строками.
				
				Пример формата 1:
				
				0:Отключено
				1:Включено
				2:Авто

				Пример формата 2:
				
				192.168.0.101
				192.168.0.102
				192.168.0.103
				`,
			},
		},
		{
			key:'default',
			caption:{ru:'По умолчанию',en:'Default'},
			control:'input',
			help: {en:'Default Parameter value',ru:'Значение параметра по умолчанию'}
		},
		{
			key:'application',
			group:{ru:'Применение',en:'Application'},
			control:'edit',
			minheight: 50,
			maxheight: 200,
			multiline: true,
			formfunc:'prototypeParamSelectApplication',
			//placeholder:'—',
			groupHelp: {
				ru:`
					Свойства переменных, к которым применим данный параметр. Если значение задано, при создании новых переменных в указанные свойства будет подставлен данный параметр. Также в свойствах будет доступен список или справочник параметра.

					Формат: список свойств переменных в виде строки "drv_name.prop_name", где drv_name - имя драйвера, prop_name - имя свойства переменной. Имя драйвера может быть заменено на символ звездочки (*), тогда параметр будет применим к указанному свойству во всех драйверах. Иначе - только к свойству переменной указанного драйвера. 

					Пример 1: 
					drv_mbtcp.drv_node_host
					drv_mbrtu.drv_node_host

					Пример 2: 
					*.round
				`,
				en:`
					Properties of variables to which this parameter applies. If the value is specified, when creating new variables, this parameter will be substituted into the specified properties. A list or directory of the parameter will also be available in the properties.

					Format: list of variable properties in the form of the string "drv_name.prop_name", where drv_name is the driver name, prop_name is the variable property name. The driver name can be replaced with an asterisk (*), which will apply the setting to the specified property in all drivers. Otherwise - only to the variable property of the specified driver.
					
					Example 1:
					drv_mbtcp.drv_node_host
					drv_mbrtu.drv_node_host
					
					Example 2:
					*.round
				`,
			}
		},
		{
			key:'validation',
			caption:{ru:'Валидация',en:'Validation'},
			control:'edit',
			list_append:'\n',
			multiline:true,
			list:[
				'maxVal:?',
				'minVal:?',
				'maxLen:?',
				'minLen:?',
				'len:?',
				'port',
				'host',
				'ip',
				'hex',
				'mac',
				'path',
			],
			help: {en:'Parameter validation',ru:'Валидация параметра'}
		},
		
		{
			key:'help',
			group:{en:'Help',ru:'Подсказка'},
			control:'edit',
			minheight: 50,
			maxheight: 200,
			multiline: true,
			wordwrap: true,
		},
	],
};
	let prototype_report={
	typename: {
		ru: "Отчет",
		en: "Report",
	},
	dontApplyCommonProps: true,
	properties: [
		
		{
			key: "type",
			caption: {
				ru: "Объект",
				en: "Object",
			},
			disabled: true,
			props:{
				hide:true,
			}
		},

		
		{
			key: "name",
			caption: {
				ru: "Имя",
				en: "Name",
			},
			unique: true,
		},

		commonParams.inactive,

		{
			key: "period",
			caption: {
				ru: "Период",
				en: "Period",
			},
			control:'select',
			list:[
				['today','Сегодня'],
				['yesterday','Вчера'],
				['thisweek','Текущая неделя'],
				['thismonth','Текущий месяц'],
				['lastweek','Прошлая неделя'],
				['lastmonth','Прошлый месяц'],
				['last60','Последние 60 минут'],
				['last24','Последние 24 часа'],
				['last7','Последние 7 дней'],
				['last30','Последние 30 дней'],
				['last90','Последние 90 дней'],
				['last365','Последние 365 дней'],
			],
			help: {
				ru: `
					Период отчета по умолчанию
				`,
				en: `
					Default report period
				`,
			},
		},

		{
			key: "diff_hide",
			group: {ru: "Разница",en: "Difference"},
			caption: {
				ru: "Показывать",
				en: "Show",
			},
			control:'checkbox',
			invert:true,
		},

		{
			key: "diff_caption",
			group: {ru: "Разница",en: "Difference"},
			caption: {
				ru: "Надпись",
				en: "Caption",
			},
			def:{
				ru: "Разница",
				en: "Difference",
			},
			visibleIf:'!diff_hide',
		},

		{
			key: "diff_hide_unit",
			group: {ru: "Разница",en: "Difference"},
			caption: {
				ru: "Ед. изм.",
				en: "Unit",
			},
			control:'checkbox',
			invert:true,
			visibleIf:'!diff_hide',
		},

		{
			key: "diff_hide_formula",
			group: {ru: "Разница",en: "Difference"},
			caption: {
				ru: "Формула",
				en: "Formula",
			},
			control:'checkbox',
			invert:true,
			visibleIf:'!diff_hide',
		},

		{
			key: "volume_hide",
			group: {ru: "Стоимость",en: "Volume"},
			caption: {
				ru: "Показывать",
				en: "Show",
			},
			control:'checkbox',
			invert:true,
		},

		{
			key: "volume_caption",
			group: {ru: "Стоимость",en: "Volume"},
			caption: {
				ru: "Надпись",
				en: "Caption",
			},
			def:{
				ru: "Стоимость",
				en: "Volume",
			},
			visibleIf:'!volume_hide',
		},
		
		change(commonParams.round,{key:'volume_round',group:{ru: "Стоимость",en: "Volume"},visibleIf:'!volume_hide'}),

		{
			key: "tariff_hide",
			group: {ru: "Тариф",en: "Tariff"},
			caption: {
				ru: "Показывать",
				en: "Show",
			},
			control:'checkbox',
			invert:true,
		},

		{
			key: "tariff_caption",
			group: {ru: "Тариф",en: "Tariff"},
			caption: {
				ru: "Надпись",
				en: "Caption",
			},
			def:{
				ru: "Тариф",
				en: "Tariff",
			},
			visibleIf:'!tariff_hide',
		},

		{
			key: "total_hide",
			group: {ru: "Сумма",en: "Total"},
			caption: {
				ru: "Показывать",
				en: "Show",
			},
			control:'checkbox',
			invert:true,
		},

		{
			key: "total_caption",
			group: {ru: "Сумма",en: "Total"},
			caption: {
				ru: "Надпись",
				en: "Caption",
			},
			def:{
				ru: "Сумма",
				en: "Total",
			},
			visibleIf:'!total_hide',
		},

		change(commonParams.round,{key:'total_round',group:{ru: "Сумма",en: "Total"},visibleIf:'!total_hide'}),



		commonParams.comment,
		
	],
};

let prototype_report_indicator={
	typename: {
		ru: "Показатель",
		en: "Indicator",
	},
	dontApplyCommonProps: true,
	properties: [
		{
			key: "name",
			caption: {
				ru: "Имя",
				en: "Name",
			},
			unique: true,
		},

		{
			key:'expr',
			caption:{en:'Formula',ru:'Формула'},
			control:'edit',
			multiline:true,
			help:{ 
				ru:`
				Математическое выражение с одной или несколькими переменными узла. 

				На основе данного выражения рассчитывается значение показателя на начало и конец периода, заданного пользователем. У переменных должна быть включена запись в тренд.

				В простейшем случае выражение может содержать имя одной переменной, значение которой и будет являться значением показателя, например:

				$var1
				
				<hr>
				`+commonHelp.expression.ru,

				en:`
				A mathematical expression with one or more node variables.

				Based on this expression, the value of the indicator is calculated at the beginning and end of the period specified by the user. Variables must have trend recording enabled.

				In the simplest case, the expression may contain the name of one variable, the value of which will be the value of the indicator, for example:

				$var1
				
				<hr>
				`+commonHelp.expression.en,
			},
		},
		
		commonParams.round,
		commonParams.unit,

		{
			key: "volume_tariff",
			group: {ru: "Стоимость",en: "Volume"},
			caption: {ru: "Тариф",en: "tariff"},
			control:'number',
			def:1,
			help:{ 
				ru:`
				СТОИМОСТЬ = РАЗНИЦА х ТАРИФ
				`,
				en:`
				VOLUME = DIFFERENCE x TARIFF
				`,
			},
		},

		{
			key: "volume_unit",
			group: {ru: "Стоимость",en: "Volume"},
			caption: {ru: "Ед. изм.",en: "Unit"},
			def: {ru: "руб",en: "rub"},
		},
		
		commonParams.comment,
		
	],
};
	let prototype_script={
	typename: {
		ru: "Скрипт",
		en: "Script",
	},
	dontApplyCommonProps: true,
	properties: [

		{
			key: "type",
			caption: {
				ru: "Объект",
				en: "Object",
			},
			disabled: true,
			props:{
				hide:true,
			}
		},

		{
			key: "name",
			caption: {
				ru: "Имя",
				en: "Name",
			},
			unique: true,
		},

		commonParams.inactive,

		{
			key:'start',
			caption:{en:'Start',ru:'Запуск'},
			control: 'select',
			def:'period',
			undef:true,
			list:[
				['period',{en:'Period',ru:'Период'}],
				['cron','Cron'],
				['trigger',{en:'Trigger',ru:'Триггер'}],
			],
		},

		{
			key: "period",
			caption: {
				ru: "Период",
				en: "Period",
			},
			control:'number',
			def:1000,
			help:{
				ru:`Период выполнения скрипта, мс`,
				en:`Script execution period, ms`,
			},
			visibleIf:"start=period",
		},

		change(commonParams.cron,{
			visibleIf:"start=cron",
		}),

		{
			key:'trigger',
			caption:{en:'Trigger',ru:'Триггер'},
			control:'input',
			visibleIf:"start=trigger",
			help:{
				ru:`
				Имена переменных (с символом $, через запятую), изменение которых будет приводить к выполнению скрипта
				
				Напрмер: $var1,$var2,$var3
				`,
				en:`
				Names of variables (with $ sign, separated by commas), changing which will lead to script execution

				Example: $var1,$var2,$var3
				`,
			}
		},

		{
			key:'timeout',
			caption:{en:'Timeout',ru:'Таймаут'},
			control:'number',
			def:100,
			help:{
				ru:`Таймаут времени выполнения скрипта, мс`,
				en:`Script execution timeout, ms`,
			}
		},

		commonParams.comment,
		
	],
};

	let dictionary_record={
	typename: {
		ru: "Запись словаря",
		en: "Dictionary record",
	},
	dontApplyCommonProps: true,
	properties: [
		{
			key: "value",
			caption: {
				ru: "Значение",
				en: "Value",
			},
		},
		{
			key: "text",
			caption: {
				ru: "Текст",
				en: "Text",
			},
		},

	],
};
	let commonDriverParams={
	name:{
		key:'name',
		caption: {ru: "Имя",en: "Name"},
		control:'input',
		unique: true,
		validation: [
			"notEmpty",
			"latinNum",
		],
		help:{
            ru:`
                Уникальное имя переменной (в контексте всего прототипа, включая другие драйвера). Допустимые символы: [a-z,A-Z,0-9,_].
            `,
            en:`
                The unique name of the variable (in the context of the entire prototype, including other drivers). Valid characters: [a-z,A-Z,0-9,_].
            `
        }
	},

	extra_server_active:{
        key:'extra_server_active',
        caption:{en:'Active',ru:'Активная'},
        control:'checkbox',
        def:true,
		table:{
			caption:{en:'Act.',ru:'Акт.'},
		},
		help:{
            ru:`
            Активная переменная участвует в сборке, обрабатывается соответствующим драйвером и доступна в списке переменных сервера. 

			Неактивная переменная игнорируется сборщиком, не обрабатывается драйвером и не доступна в списке переменных сервера.

			Неактивная переменная также не учитываются при подсчете лицензируемых переменных. 
            `,
            en:`
            The active variable takes part in the assembly, is processed by the corresponding driver and is available in the list of server variables.

			An inactive variable is ignored by the collector, is not processed by the driver, and is not available in the list of server variables.

			An inactive variable is also not counted towards licensed variables.
            `
        }
    },

    extra_server_notify:{
        key:'extra_server_notify',
        group:{en:'Notification',ru:'Уведомление'},
        control:'edit',
        multiline: true,
        minheight: 100,
    },

	trend:{
        key:'trend',
        caption:{en:'Trend',ru:'Тренд'},
        control:'edit',
		listholder:true,
		undef:true,
        checkbox:true,
        def:0,
        drvdef:0,
		list:[
			[0,{ru:'0 - откл',en:'0 - off'}],
			[1,{ru:'1 - вкл',en:'1 - on'}],
		],
        help:{
            ru:`
            Записывать изменения переменной в файл тренда. 1 запись = 16 байт.
            `,
            en:`
            Write the changes of the variable to the trend file. 1 record = 16 bytes.
            `
        }
    },

	buffer:{
        key:'buffer',
        caption:{en:'Buffering',ru:'Буферизация'},
        control:'edit',
		listholder:true,
		undef:true,
        checkbox:true,
        def:0,
        drvdef:0,
		list:[
			[0,{ru:'0 - откл',en:'0 - off'}],
			[1,{ru:'1 - вкл',en:'1 - on'}],
		],
        help:{
            ru:`
            Буферизация данных в кольцевом буфере шлюза для отправки после разрыва/восстановления связи.
            Используйте данную опцию с тревогами и трендами для минимизации потерь исторических данных.
            `,
            en:`
            Buffer data in a gateway ring buffer for sending after a link break/restore.
            Use this option with alarms and trends to minimize loss of historical data.
            `
        }
    },

    drv_expr_unrel:{
        key:'drv_expr_unrel',
        caption:{en:'Unreliable',ru:'Недостоверно'},
        control:'select',
        undef:true,
        def:0,
        drvdef:0,
        list:[
            [0,{ru:'Ошибка',en:'Error'}],
            [1,{ru:'Ноль',en:'Zero'}],
            [2,{ru:'Значение',en:'Value'}],
        ],
        
        help:{
            ru:`
            Параметр определяет метод обработки переменных в выражении, значение которых недостоверно (переменная в ошибке).

            Ошибка - при наличии в выражении хотя бы одной недостоверной переменной, результатом расчета выражения будет ошибка DRV_ERROR_UNRELIABLE_VAR.

            Ноль - при наличии в выражении недостоверных переменных, их значение берется равным нулю, выражение рассчитывается без генерации ошибки.

            Значение - при наличии в выражении недостоверных переменных, их значение берется равным последнему достоверному, выражение рассчитывается без генерации ошибки.

            При любой установке данного параметра доступно использование прямых значений полей переменной через соответствующий суффикс, например:

            $var1__time - время последнего изменения статуса переменной
            $var1__value - последнее достоверное значение
            $var1__error - код ошибки переменной 
            `,
            en:`
            The parameter defines the method for processing variables in an expression whose value is invalid (variable in error).

            Error - if there is at least one invalid variable in the expression, the result of calculating the expression will be the error DRV_ERROR_UNRELIABLE_VAR.

            Zero - if there are invalid variables in the expression, their value is taken equal to zero, the expression is calculated without generating an error.

            Value - if there are invalid variables in the expression, their value is taken equal to the last valid one, the expression is calculated without generating an error.

            With any setting of this parameter, it is possible to use direct values ​​of the variable fields through the corresponding suffix, for example:

            $var1__time - time of the last change in the variable status
            $var1__value - last valid value
            $var1__error - variable error code
            `
        }
    },

	readonly:{
        key:'readonly',
		group:{en:'Miscellaneous',ru:'Разное'},
        caption:{en:'Read-only',ru:'Только чтение'},
        bottom: true,
        table:{
            caption:'RO',
            tooltip:{en:'Read-only',ru:'Только чтение'},
        },
        control:'edit',
		undef:true,
        def:0,
        drvdef:0,
        checkbox:true,
		listholder:true,
		list:[
            [0,{ru:'0 - откл',en:'0 - off'}],
			[1,{ru:'1 - вкл',en:'1 - on'}],
		],
		help:{
            ru:`
            Запрет на изменение значения переменной со стороны пользователя.
            `,
            en:`
            Prohibition on changing the value of a variable by the user.
            `
        }
    },

	gateway:{
        key:'gateway',
		group:{en:'Miscellaneous',ru:'Разное'},
        caption:{en:'Location',ru:'Размещение'},
        bottom: true,
        control:'select',
        def:'%gateway%',
        drvdef:'%gateway%',
        list:[
            ['%gateway%',{en:'Auto',ru:'Авто'}],
            ['[LOCAL]',{en:'Local',ru:'Локально'}],
        ],
        help:{
            ru:`
                Размещение переменной:
                
                Авто - если узел находится за шлюзом или является шлюзом, будет создана переменная для соответствующего драйвера на удаленном шлюзе и локальная переменная для драйвера шлюза со ссылкой на удаленную переменную. Если узел локальный - будет создана только локальная переменная для соответствующего драйвера.

                Локально - будет создана только локальная переменная для соответствующего драйвера.
                Это может потребоваться для случаев, когда узел находится за шлюзом или является шлюзом, но в него необходимо добавить переменную для локального драйвера. Так, например, в узел можно добавить локальную виртуальную переменную "статус техобслуживания", значение которой будет всегда доступно на чтение и запись, независимо от того, подключен шлюз или нет.

            `,
            en:`
				Variable placement:
					
				Auto - If the node is behind a gateway or is a gateway, a variable will be created for the corresponding driver on the remote gateway and a local variable for the gateway driver with a reference to the remote variable. If the node is local, only a local variable will be created for the corresponding driver.

				Local - only a local variable will be created for the corresponding driver.
				This may be required for cases where the node is behind a gateway or is a gateway, but needs to add a local driver variable to it. For example, you can add a local virtual variable “maintenance status” to a node, the value of which will always be readable and writable, regardless of whether the gateway is connected or not.
            `
        }
    },

    extra_polling:{
        key:'extra_polling',
        group:{en:'Polling',ru:'Опрос'},
        caption:{en:'Polling mode',ru:'Режим опроса'},
        control:'select',
		def:'period',
        undef:true,
        list:[
            ['period',{en:'Period',ru:'Период'}],
            ['cron','Cron'],
        ],
        help:{
            ru:`
                Режим опроса переменных:
                
                <b>Период</b>: опрос выполняется с интервалом, указанным в параметре "Период"

                <b>Cron</b>: опрос выполняется по расписанию в формате Cron
            `,
            en:`
                Variable polling mode:
                    
                <b>Period</b>: polling is performed at the interval specified in the "Period" parameter

                <b>Cron</b>: polling is performed according to the schedule in Cron format
            `
        }
    },

	drv_period:{
        key:'drv_period',
        group:{en:'Polling',ru:'Опрос'},
        caption:{en:'Period',ru:'Период'},
        control:'input',
        visibleIf:"extra_polling=period",
        help:{
            ru:`Период опроса переменной, мс`,
            en:`Variable polling period, ms`,
        }
    },

    drv_cron:change(commonParams.cron,{
        key:'drv_cron',
        group:{en:'Polling',ru:'Опрос'},
        visibleIf:"extra_polling=cron",
    }),
    
    round:change(commonParams.round,{group:{en:'Processing',ru:'Обработка'},}),

	threshold:{
        key:'threshold',
        group:{en:'Processing',ru:'Обработка'},
        caption:{en:'Value threshold',ru:'Порог по значению'},
        control:'input',
        help:{
            ru:`
                Переменная не будет изменять свое значение, пока изменение значения сигнала не превысит по модулю заданную в данном параметре величину.
            `,
            en:`
                The variable will not change its value until the change in the signal value modulo does not exceed the value set in this parameter.
            `
        }
    },

	threstime:{
        key:'threstime',
        group:{en:'Processing',ru:'Обработка'},
        caption:{en:'Time threshold',ru:'Порог по времени'},
        control:'input',
        help:{
            ru:`
                Переменная не будет изменять свое значение чаще, чем указанно в данном параметре, мс.
            `,
            en:`
                The variable will not change its value more often than specified in this parameter, ms.
            `
        }
    },

    min:{
        key:'min',
        group:{en:'Processing',ru:'Обработка'},
        caption:{en:'Minimum',ru:'Минимум'},
        control:'input',
        help:{
            ru:`
                Минимально допустимое значение переменной, которое может быть записано со стороны пользователя.
                Данный параметр не применяется к значениям со стороны наблюдаемых сигналов.
            `,
            en:`
                The minimum allowable value of a variable that can be written by the user.
                This parameter does not apply to the values on the observed signal side.
            `
        }
    },

    max:{
        key:'max',
        group:{en:'Processing',ru:'Обработка'},
        caption:{en:'Maximum',ru:'Максимум'},
        control:'input',
        help:{
            ru:`
                Максимально допустимое значение переменной, которое может быть записано со стороны пользователя.
                Данный параметр не применяется к значениям со стороны наблюдаемых сигналов.
            `,
            en:`
                The maximum allowable value of a variable that can be written by the user.
                This parameter does not apply to the values on the observed signal side.
            `
        }
    },

    coef:{
        key:'coef',
        group:{en:'Processing',ru:'Обработка'},
        caption:{en:'Coef',ru:'Множитель'},
        control:'input',
        def:1,
        drvdef:1,
        help:{
            ru:`
                VarValue = SignalValue * Множитель + Смещение
                SignalValue = (VarValue - Смещение) / Множитель
            `,
            en:`
                VarValue = SignalValue * Coef + Offset
                SignalValue = (VarValue - Offset) / Coef
            `
        }
    },

    offset:{
        key:'offset',
        group:{en:'Processing',ru:'Обработка'},
        caption:{en:'Offset',ru:'Смещение'},
        control:'input',
        def:0,
        drvdef:0,
        help:{
            ru:`
                VarValue = SignalValue * Множитель + Смещение
                SignalValue = (VarValue - Смещение) / Множитель
            `,
            en:`
                VarValue = SignalValue * Coef + Offset
                SignalValue = (VarValue - Offset) / Coef
            `
        }
    },

    scl_hi:{
        key:'scl_hi',
        group:{en:'Normalization',ru:'Нормализация'},
        caption:'Scale High',
        control:'input',
        groupHelp:{
            ru:`
                Нормализация значения переменной:    
                VarValue = ((SignalValue - RawLow) / (RawHigh - RawLow)) * (ScaleHigh - ScaleLow) + ScaleLow
            `,
            en:`
                Normalization of variable value:    
                VarValue = ((SignalValue - RawLow) / (RawHigh - RawLow)) * (ScaleHigh - ScaleLow) + ScaleLow
            `
        }
    },

    scl_lo:{
        key:'scl_lo',
        group:{en:'Normalization',ru:'Нормализация'},
        caption:'Scale Low',
        control:'input',
    },

    raw_hi:{
        key:'raw_hi',
        group:{en:'Normalization',ru:'Нормализация'},
        caption:'Raw High',
        control:'input',
    },

    raw_lo:{
        key:'raw_lo',
        group:{en:'Normalization',ru:'Нормализация'},
        caption:'Raw Low',
        control:'input',
    },


    reset_interval:{
        key:'reset_interval',
        group:{en:'Reset value',ru:'Сброс значения'},
        caption:{en:'Reset Time',ru:'Время сброса'},
        control:'input',
        help:{
            ru:`
                Если параметр не равен нулю, значение переменной будет сброшено на 'Значение сброса' по прошествии указанного времени, мс
            `,
            en:`
                If the parameter is not zero, the value of the variable will be reset to 'Reset value' after the specified time has elapsed, ms
            `
        }
    },

    reset_value:{
        key:'reset_value',
        group:{en:'Reset value',ru:'Сброс значения'},
        caption:{en:'Reset value',ru:'Значение сброса'},
        control:'input',
        def:0,
        drvdef:0,
        help:{
            ru:`
                Если задано 'Время сброса', значение переменной будет сброшено на указанное значение по прошествии времени сброса
            `,
            en:`
                If 'Reset Time' is set, the value of the variable will be reset to the specified value after the reset time has elapsed
            `
        }
    },

    group_vars:{
        key:'group_vars',
        group:{en:'Group actions',ru:'Групповые операции'},
        caption:{en:'Direct vars',ru:'Прямые переменные'},
        control:'input',
        help:{
            ru:`
            Имена прямых управляемых переменных со знаком $, через запятую.
            
            При изменении значения управляющей (текущей) переменной всем прямым управляемым переменным будет отправлена команда установить значение, равное значению управляющей переменной.

            Например: 
            а) управляемые переменные в текущем узле: $va1,$var2,$var3
            б) управляемые переменные в других узлах: $P1_N1_var1,$P1_N2_var1,$P1_N3_var1
            `,
            en:`
            Names of direct controlled variables with a $ sign, separated by commas.
            
            When the value of the current variable changes, a command will be sent to all direct controlled variables to set the value equal to the value of the current variable.

            For example: 
            a) controlled variables in the current node: $va1,$var2,$var3
            b) controlled variables in other nodes: $P1_N1_var1,$P1_N2_var1,$P1_N3_var1
            `
        }
    },

    group_inv:{
        key:'group_inv',
        group:{en:'Group actions',ru:'Групповые операции'},
        caption:{en:'Inv. vars',ru:'Инв. переменные'},
        control:'input',
        help:{
            ru:`
            Имена инвертируемых управляемых переменных со знаком $, через запятую.
            
            При изменении значения управляющей (текущей) переменной всем инвертируемым управляемым переменным будет отправлена команда установить значение, равное логически инвертированному значению управляющей переменной (0, если значение УП != 0; 1 если значение УП == 0).
            `,
            en:`
            Names of inverted controlled variables with a $ sign, separated by commas.
            
            When the value of the current variable changes, a command will be sent to all inverted controlled variables to set the value equal to logical inversion of the value of the current variable.
            `
        }
    },

    group_ignore:{
        key:'group_ignore',
        group:{en:'Group actions',ru:'Групповые операции'},
        caption:{en:'Ign. value',ru:'Игн. значение'},
        control:'input',
        help:{
            ru:`
            Игнорируемое значение, при переходе на которое не происходит запись в управляемые переменные.
            Данный параметр можно использовать в сочетании со "Сбросом значения", когда значение сброса указывается как игнорируемое
            `,
            en:`
            An ignored value, upon transition to which no write to the controlled variables occurs.
            This parameter can be used in combination with "Reset value", when the reset value is specified as ignored
            `
        }
    },

    group_block:{
        key:'group_block',
        group:{en:'Group actions',ru:'Групповые операции'},
        caption:{en:'Wait confirm',ru:'Ожидать подтверждения'},
        control:'edit',
        undef:true,
        def:0,
        drvdef:0,
        listholder:true,
        checkbox:true,
        list:[
            [0,{ru:'0 - откл',en:'0 - off'}],
            [1,{ru:'1 - вкл',en:'1 - on'}],
        ],
        help:{
            ru:`
                Ожидать подтверждения записи в управляемые переменные. 
                    
                Если опция включена, последующая запись в групповые переменные будет возможна только после подтверждения предыдущей записи или после достижения времени таймаута. Опция позволяет избежать сваливания в цикл при "закольцованных" групповых переменных (когда var1 является групповой переменной в var2, а var2 - групповой переменной в var1).
            `,
            en:`
                Wait for confirmation of writing to managed variables. 
                    
                If this option is enabled, subsequent writing to group variables will be possible only after confirming the previous entry or after reaching the timeout time. This option allows you to avoid falling into a loop with "looped" group variables (when var1 is a group variable in var2, and var2 is a group variable in var1).
            `
        }
    },

    group_sync:{
        key:'group_sync',
        group:{en:'Group actions',ru:'Групповые операции'},
        caption:{en:'Forced sync',ru:'Принуд. синхронизация'},
        control:'edit',
        undef:true,
        def:0,
        drvdef:0,
        listholder:true,
        checkbox:true,
        list:[
            [0,{ru:'0 - откл',en:'0 - off'}],
            [1,{ru:'1 - вкл',en:'1 - on'}],
        ],
        help:{
            ru:`
                Если опция включена, ядро будет отслеживать значения управляемых переменных. Если значения управляемых переменных будут отличаться от значения управляющей переменной, ядро выполнит принудительную синхронизацию значений.

                Если опция отключена, изменение управляемых переменных будет выполняться только при изменении управляющей переменной.
            `,
            en:`
                If the option is enabled, the kernel will monitor the values ​​of the controlled variables. If the values ​​of the controlled variables differ from the value of the control variable, the kernel will force the values ​​to be synchronized.

                If the option is disabled, the controlled variables will only be changed when the control variable is changed.
            `
        }
    },

    group_timeout:{
        key:'group_timeout',
        group:{en:'Group actions',ru:'Групповые операции'},
        caption:{en:'Timeout',ru:'Таймаут'},
        control:'input',
        def:1000,
        drvdef:1000,
        help:{
            ru:`
                Таймаут на установку значения управляемой переменной, мс.    

                Данный параметр учитывается при ожидании подтверждения и принудительной синхронизации. Синхронизация управляемых переменных блокируется на указанное время или до успешной синхронизации (когда значения управляемых переменных становятся равны значению управляющей переменной).

                Применение таймаута позволяет избежать повторных запросов на изменение управляемых переменных, когда они уже были установлены, но еще не успели обновить свое значение.
            `,
            en:`
                Timeout for setting the value of the controlled variable, ms.

                This parameter is taken into account when waiting for confirmation and forced synchronization. Synchronization of controlled variables is blocked for the specified time or until successful synchronization (when the values ​​of the controlled variables become equal to the value of the control variable).

                Using a timeout allows you to avoid repeated requests to change controlled variables when they have already been set, but have not yet updated their value.
            `
        }
    },

    group_tolerance:{
        key:'group_tolerance',
        group:{en:'Group actions',ru:'Групповые операции'},
        caption:{en:'Tolerance',ru:'Допуск'},
        control:'input',
        def:0.001,
        drvdef:0.001,
        help:{
            ru:`
                Данный параметр учитывается при ожидании подтверждения и принудительной синхронизации при сравнении значений управляемых переменных со значением управляющей переменной.
                
                Если значения отличаются более чем на величину допуска, значения считаются разными, иначе - равными.
            `,
            en:`
                This parameter is taken into account when waiting for confirmation and forced synchronization when comparing the values ​​of controlled variables with the value of the control variable.

                If the values ​​differ by more than the tolerance value, the values ​​are considered different, otherwise - equal.
            `
        }
    },


    retry_count:{
        key:'retry_count',
        group:{en:'Retry set',ru:'Попытки записи'},
        caption:{en:'Retry count',ru:'Кол-во попыток'},
        control:'input',
        def:1,
        drvdef:1,
        visibleIf:'!readonly',
        help:{
            ru:`
                Количество попыток записи нового значения переменной
            `,
            en:`
                Number of attempts to write a new variable value
            `
        }
    },

    retry_interval:{
        group:{en:'Retry set',ru:'Попытки записи'},
        key:'retry_interval',
        caption:{en:'Retry interval',ru:'Интервал попыток'},
        control:'input',
        def:1000,
        drvdef:1000,
        visibleIf:'!readonly',
        help:{
            ru:`
                Интервал между попытками, мс
            `,
            en:`
                Interval between attempts, ms
            `
        }
    },

    sim_random:{
        key:'sim_random',
        group:{en:'Simulation',ru:'Симуляция'},
        caption:{en:'Random values',ru:'Случайные значения'},
        control:'edit',
		undef:true,
        def:0,
        drvdef:0,
        checkbox:true,
		listholder:true,
		list:[
            [0,{ru:'0 - откл',en:'0 - off'}],
			[1,{ru:'1 - вкл',en:'1 - on'}],
		],
        help:{
            ru:`
                Если опция включена, значения переменной будут генерироваться случайным образом. Реальные значения от драйвера будут игнорироваться. Текущее случайное значение может быть изменено оператором. 
                
                Данный режим может быть использован для отладки проектов и может быть отключен в любое время. 
            `,
            en:`
                If the option is enabled, the variable values ​​will be generated randomly. Real values ​​from the driver will be ignored. The current random value can be changed by the operator.

                This mode can be used for debugging projects and can be disabled at any time. 
            `
        }
    },

    sim_random_delta:{
        key:'sim_random_delta',
        group:{en:'Simulation',ru:'Симуляция'},
        caption:{en:'Delta',ru:'Дельта'},
        control:'input',
        def:1,
        drvdef:1,
        visibleIf:'sim_random',
        help:{
            ru:`
                Если параметр задан, значения переменной при симуляции будут изменяться  на указанную величину в случайном направлении (увеличение или уменьшение).

                Если параметр не задан, значение переменной будет случайным числом из диапазона, заданного параметрами Минимум и Максимум.
            `,
            en:`
                If the parameter is specified, the values of the variable during the simulation will change by the specified amount in a random direction (increase or decrease).

                If the parameter is not specified, the value of the variable will be a random number from the range specified by the Minimum and Maximum parameters.
            `
        }
    },

    sim_random_min:{
        key:'sim_random_min',
        group:{en:'Simulation',ru:'Симуляция'},
        caption:{en:'Minimum',ru:'Минимум'},
        control:'input',
        def:0,
        drvdef:0,
        visibleIf:'sim_random',
        help:{
            ru:`
                Если параметр задан, случайные значения переменной при симуляции не будут опускаться ниже данного значения.
            `,
            en:`
                If the parameter is specified, the random values of the variable during the simulation will not fall below this value.
            `
        }
    },

    sim_random_max:{
        key:'sim_random_max',
        group:{en:'Simulation',ru:'Симуляция'},
        caption:{en:'Maximum',ru:'Максимум'},
        control:'input',
        def:100,
        drvdef:100,
        visibleIf:'sim_random',
        help:{
            ru:`
                Если параметр задан, случайные значения переменной при симуляции не будут подниматься выше данного значения.
            `,
            en:`
                If the parameter is specified, the random values of the variable during the simulation will not rise above this value.
            `
        }
    },

    sim_random_period:{
        key:'sim_random_period',
        group:{en:'Simulation',ru:'Симуляция'},
        caption:{en:'Period',ru:'Период'},
        control:'input',
        def:1000,
        drvdef:1000,
        visibleIf:'sim_random',
        help:{
            ru:`
                Период генерации значений. Если период равен нулю, новые значения не генерируются.
            `,
            en:`
                The period for generating values. If the period is zero, no new values ​​are generated.
            `
        }
    },

    sim_random_init:{
        key:'sim_random_init',
        group:{en:'Simulation',ru:'Симуляция'},
        caption:{en:'Initial value',ru:'Начальное значение'},
        control:'input',
        def:0,
        drvdef:0,
        visibleIf:'sim_random',
        help:{
            ru:`
                Данное начальное значение будет установлено переменной, если она еще не была инициализирована.
            `,
            en:`
                This initial value will be set to the variable if it has not already been initialized.
            `
        }
    },

    extra_service_remote_map_mbtcp__enabled:{
        key:'extra_service_remote_map_mbtcp__enabled',
        group:{en:'Gateway Modbus-map',ru:'Modbus-карта шлюза'},
        caption:{en:'Enable',ru:'Включить'},
        control:'edit',
		undef:true,
        def:0,
        drvdef:0,
        checkbox:true,
		listholder:true,
		list:[
            [0,{ru:'0 - откл',en:'0 - off'}],
			[1,{ru:'1 - вкл',en:'1 - on'}],
		],
        help:{
            ru:`
                Включить трансляцию данной переменной в Modbus-карту шлюза
            `,
            en:`
                Enable translation of this variable to Gateway Modbus-map 
            `
        }
    },

    service_remote_map_mbtcp__port:{
        key:'service_remote_map_mbtcp__port',
        group:{en:'Gateway Modbus-map',ru:'Modbus-карта шлюза'},
        caption:{en:'Port',ru:'Порт'},
        control:'input',
        def:5002,
        visibleIf:"extra_service_remote_map_mbtcp__enabled",
        help:{
            ru:`
                Номер Modbus-порта карты
            `,
            en:`
                Map modbus-port number
            `
        }
    },

    service_remote_map_mbtcp__slave:{
        key:'service_remote_map_mbtcp__slave',
        group:{en:'Gateway Modbus-map',ru:'Modbus-карта шлюза'},
        caption:{en:'Slave',ru:'Slave'},
        control:'input',
        def:1,
        drvdef:1,
        visibleIf:"extra_service_remote_map_mbtcp__enabled",
        help:{
            ru:`
                Modbus Slave ID
            `,
            en:`
                Modbus Slave ID
            `
        }
    },

    service_remote_map_mbtcp__reg_type:{
        key:'service_remote_map_mbtcp__reg_type',
        group:{en:'Gateway Modbus-map',ru:'Modbus-карта шлюза'},
        caption:{en:'Register type',ru:'Тип регистра'},
        control:'edit',
        undef:true,
        list:['CL','DI','HR','IR'],
        def:'HR',
        drvdef:'HR',
        visibleIf:"extra_service_remote_map_mbtcp__enabled",
        help:`
                CL: Coil
                DI: Digital Input
                HR: Holding Register
                IR: Input Register
            `
    },

    service_remote_map_mbtcp__reg_addr:{
        key:'service_remote_map_mbtcp__reg_addr',
        group:{en:'Gateway Modbus-map',ru:'Modbus-карта шлюза'},
        caption:{en:'Register address',ru:'Адрес регистра'},
        control:'input',
        visibleIf:"extra_service_remote_map_mbtcp__enabled",
        help:{
            ru:`
                Адрес регистра, число 0 - 65535.
                Физический адрес регистра = Адрес + Смещение
            `,
            en:`
                Register Address, number 0 - 65535.
                Register Physical Address = Address + Offset
            `
        }
    },

    service_remote_map_mbtcp__reg_off:{
        key:'service_remote_map_mbtcp__reg_off',
        group:{en:'Gateway Modbus-map',ru:'Modbus-карта шлюза'},
        caption:{en:'Register offset',ru:'Смещение регистра'},
        control:'input',
        def:0,
        drvdef:0,
        visibleIf:"extra_service_remote_map_mbtcp__enabled",
        help:{
            ru:`
                Смещение адреса регистра.
                Физический адрес регистра = Адрес + Смещение
            `,
            en:`
                Offset of register address.
                Register Physical Address = Address + Offset
            `
        }
    },

    service_remote_map_mbtcp__format:{
        key:'service_remote_map_mbtcp__format',
        group:{en:'Gateway Modbus-map',ru:'Modbus-карта шлюза'},
        caption:{en:'Register format',ru:'Формат регистра'},
        visibleIf:"extra_service_remote_map_mbtcp__enabled",
        control:'edit',
        undef:true,
        def:'INT16',
        drvdef:'INT16',
        list:[
            'INT16',
            'UINT16',
            'FLOAT_ABCD',
            'FLOAT_CDAB',
            'FLOAT_BADC',
            'FLOAT_DCBA',
            'INT32_ABCD',
            'INT32_CDAB',
            'INT32_BADC',
            'INT32_DCBA',
            'UINT32_ABCD',
            'UINT32_CDAB',
            'UINT32_BADC',
            'UINT32_DCBA',
            'BIT_0',
            'BIT_1',
            'BIT_2',
            'BIT_3',
            'BIT_4',
            'BIT_5',
            'BIT_6',
            'BIT_7',
            'BIT_8',
            'BIT_9',
            'BIT_10',
            'BIT_11',
            'BIT_12',
            'BIT_13',
            'BIT_14',
            'BIT_15',
        ],
    },

    extra_service_local_map_mbtcp__enabled:{
        key:'extra_service_local_map_mbtcp__enabled',
        group:{en:'Server Modbus-map',ru:'Modbus-карта сервера'},
        caption:{en:'Enable',ru:'Включить'},
        control:'edit',
		undef:true,
        def:0,
        drvdef:0,
        checkbox:true,
		listholder:true,
		list:[
            [0,{ru:'0 - откл',en:'0 - off'}],
			[1,{ru:'1 - вкл',en:'1 - on'}],
		],
        help:{
            ru:`
                Включить трансляцию данной переменной в Modbus-карту сервера
            `,
            en:`
                Enable translation of this variable to Server Modbus-map 
            `
        }
    },

    service_local_map_mbtcp__port:{
        key:'service_local_map_mbtcp__port',
        group:{en:'Server Modbus-map',ru:'Modbus-карта сервера'},
        caption:{en:'Port',ru:'Порт'},
        control:'input',
        def:5002,
        visibleIf:"extra_service_local_map_mbtcp__enabled",
        help:{
            ru:`
                Номер Modbus-порта карты
            `,
            en:`
                Map modbus-port number
            `
        }
    },

    service_local_map_mbtcp__slave:{
        key:'service_local_map_mbtcp__slave',
        group:{en:'Server Modbus-map',ru:'Modbus-карта сервера'},
        caption:{en:'Slave',ru:'Slave'},
        control:'input',
        def:1,
        drvdef:1,
        visibleIf:"extra_service_local_map_mbtcp__enabled",
        help:{
            ru:`
                Modbus Slave ID
            `,
            en:`
                Modbus Slave ID
            `
        }
    },

    service_local_map_mbtcp__reg_type:{
        key:'service_local_map_mbtcp__reg_type',
        group:{en:'Server Modbus-map',ru:'Modbus-карта сервера'},
        caption:{en:'Register type',ru:'Тип регистра'},
        control:'edit',
        undef:true,
        list:['CL','DI','HR','IR'],
        def:'HR',
        drvdef:'HR',
        visibleIf:"extra_service_local_map_mbtcp__enabled",
        help:`
                CL: Coil
                DI: Digital Input
                HR: Holding Register
                IR: Input Register
            `
    },

    service_local_map_mbtcp__reg_addr:{
        key:'service_local_map_mbtcp__reg_addr',
        group:{en:'Server Modbus-map',ru:'Modbus-карта сервера'},
        caption:{en:'Register address',ru:'Адрес регистра'},
        control:'input',
        visibleIf:"extra_service_local_map_mbtcp__enabled",
        help:{
            ru:`
                Адрес регистра, число 0 - 65535.
                Физический адрес регистра = Адрес + Смещение
            `,
            en:`
                Register Address, number 0 - 65535.
                Register Physical Address = Address + Offset
            `
        }
    },

    service_local_map_mbtcp__reg_off:{
        key:'service_local_map_mbtcp__reg_off',
        group:{en:'Server Modbus-map',ru:'Modbus-карта сервера'},
        caption:{en:'Register offset',ru:'Смещение регистра'},
        control:'input',
        def:0,
        drvdef:0,
        visibleIf:"extra_service_local_map_mbtcp__enabled",
        help:{
            ru:`
                Смещение адреса регистра.
                Физический адрес регистра = Адрес + Смещение
            `,
            en:`
                Offset of register address.
                Register Physical Address = Address + Offset
            `
        }
    },

    service_local_map_mbtcp__format:{
        key:'service_local_map_mbtcp__format',
        group:{en:'Server Modbus-map',ru:'Modbus-карта сервера'},
        caption:{en:'Register format',ru:'Формат регистра'},
        visibleIf:"extra_service_local_map_mbtcp__enabled",
        control:'edit',
        undef:true,
        def:'INT16',
        drvdef:'INT16',
        list:[
            'INT16',
            'UINT16',
            'FLOAT_ABCD',
            'FLOAT_CDAB',
            'FLOAT_BADC',
            'FLOAT_DCBA',
            'INT32_ABCD',
            'INT32_CDAB',
            'INT32_BADC',
            'INT32_DCBA',
            'UINT32_ABCD',
            'UINT32_CDAB',
            'UINT32_BADC',
            'UINT32_DCBA',
            'BIT_0',
            'BIT_1',
            'BIT_2',
            'BIT_3',
            'BIT_4',
            'BIT_5',
            'BIT_6',
            'BIT_7',
            'BIT_8',
            'BIT_9',
            'BIT_10',
            'BIT_11',
            'BIT_12',
            'BIT_13',
            'BIT_14',
            'BIT_15',
        ],
    },

    extra_service_local_exec__enabled:{
        key:'extra_service_local_exec__enabled',
        group:{en:'Server Command Line',ru:'Командная строка сервера'},
        caption:{en:'Enable',ru:'Включить'},
        control:'edit',
		undef:true,
        def:0,
        drvdef:0,
        checkbox:true,
		listholder:true,
		list:[
            [0,{ru:'0 - откл',en:'0 - off'}],
			[1,{ru:'1 - вкл',en:'1 - on'}],
		],
        help:{
            ru:`
                Включить трансляцию данной переменной в командную строку сервера.

                Если данный параметр включен, при изменении статуса переменной на сервере будет выполнена команда командной строки, указанная в параметре "Команда", с подстановкой имени, времени, значения, ошибки и пейлоада переменной.
            `,
            en:`
                Enable broadcasting of this variable to the server command line.

                If this parameter is enabled, when the status of a variable changes on the server, the command line command specified in the "Command" parameter will be executed, substituting the name, time, value, error and payload of the variable.
            `
        }
    },

    service_local_exec__init:{
        key:'service_local_exec__init',
        group:{en:'Server Command Line',ru:'Командная строка сервера'},
        caption:{en:'Initialization',ru:'Инициализация'},
        visibleIf:"extra_service_local_exec__enabled",
        control:'edit',
		undef:true,
        def:0,
        drvdef:0,
        checkbox:true,
		listholder:true,
		list:[
            [0,{ru:'0 - откл',en:'0 - off'}],
			[1,{ru:'1 - вкл',en:'1 - on'}],
		],
        help:{
            ru:`
                Выполнять команду при инициализации переменной.

                При перезапуске сервиса или при переподключении к ядру выполняется инициализация переменной полученным от ядра статусом. 
                
                Если данный параметр включен, то инициализация считается изменением и команда будет выполнена. Это может приводить к дублированию записей, выполняемых по команде, т.к. данный статус переменной уже мог фигурировать в предыдущих изменениях.

                Если данный параметр выключен, команды начнут выполняться только при поступлении свежих изменений статуса переменной. Получение статуса при инициализации будет проигнорировано.
            `,
            en:`
                Execute a command when a variable is initialized.
                
                 When the service is restarted or when reconnecting to the kernel, the variable is initialized with the status received from the kernel.
                
                 If this parameter is enabled, then the initialization is considered a change and the command will be executed. This may lead to duplication of records executed by command, because this variable status may have already appeared in previous changes.

                 If this option is disabled, commands will begin to be executed only when new changes to the variable status arrive. Receiving status during initialization will be ignored.
            `
        }
    },

    service_local_exec__timeout:{
        key:'service_local_exec__timeout',
        group:{en:'Server Command Line',ru:'Командная строка сервера'},
        caption:{en:'Timeout',ru:'Таймаут'},
        control:'input',
        def:1000,
        drvdef:1000,
        visibleIf:"extra_service_local_exec__enabled",
        help:{
            ru:`
                Максимальное время ожидания выполнения команды.
                Если команда не успела выполнится за указанное время, поток, в которм выполняется команда, будет закрыт с кодом ETIMEOUT (RC 116).
            `,
            en:`
                The maximum time to wait for a command to complete.
                If the command does not complete within the specified time, the thread in which the command is executed will be closed with the code ETIMEOUT (RC 116).
            `
        }
    },

    service_local_exec__command:{
        key:'service_local_exec__command',
        group:{en:'Server Command Line',ru:'Командная строка сервера'},
        caption:{en:'Command',ru:'Команда'},
        control:'edit',
        minheight: 50,
        maxheight: 200,
        multiline: true,
        visibleIf:"extra_service_local_exec__enabled",
        help:{
            ru:`
                Команда командной строки, с возможностью подстановки:

                %var_name% - глобальное имя переменной
                %var_value% - значение переменной
                %var_time% - timestamp времени изменения статуса
                %var_error% - код ошибки
                %var_payload% - поле payload

                <b>Примеры</b>

                Вывод изменений в файл:

                echo %var_time%,%var_name%,%var_value%,%var_error% >> temp.csv

                Вывод изменений в БД MySQL:

                mysql -e 'REPLACE INTO table1 (name, value) VALUES ("%var_name%", %var_value%);' testdb
            `,
            en:`
                Command line command, with substitution option:

                %var_name% - global variable name
                %var_value% - variable value
                %var_time% - timestamp of status change time
                %var_error% - error code
                %var_payload% - payload field

                <b>Examples</b>
                
                Outputting changes to a file:

                echo %var_time%,%var_name%,%var_value%,%var_error% >> temp.csv

                Output to MySQL DB:

                mysql -e 'REPLACE INTO table1 (name, value) VALUES ("%var_name%", %var_value%);' testdb
            `
        }
    },


    extra_server_unit: change(commonParams.unit,{key:'extra_server_unit'}),

    //change(commonDriverParams.buffer,{def:1}),
    extra_server_tags:change(commonParams.tags,{key:'extra_server_tags'}),

    extra_server_description:{
        key:'extra_server_description',
        caption:{en:'Description',ru:'Описание'},
        control:'edit',
        help:{
            ru:`
            Краткое описание переменной. Описание регистрируется в журнале при изменении переменной пользователем. Для тревог описание подставляется в текст тревожного сообщения. В тексте допускается подстановка %параметров_узла%.
            
            Пример описания для переменных общего назначения: 
            <em>Включение вентилятора</em>, <em>Уставка температуры в %room%</em>, <em>Сброс аварий</em>
            
            Пример описания для тревожных переменных: 
            <em>Отказ циркуляционного насоса</em>, <em>Превышение давления теплоносителя</em>, <em>Разгерметизация шлюза № %block_number%</em>
            `,
            en:`
            Brief description of the variable. The description is logged in the journal when the user changes the variable. For alarms, the description is the alarm message. Substitution of %node_parameters% is allowed in the text.
            
             Example description for general purpose variables:
             <em>Fan on</em>, <em>Temperature setpoint in %room%</em>, <em>Alarm reset</em>
            
             Example description for emergency variables:
             <em>Circulation pump failure</em>, <em>Excessive coolant pressure</em>, <em>Depressurization of gateway No. %block_number%</em>
            `
        }
    },  

	extra_server_comment:{
        key:'extra_server_comment',
        caption:{en:'Comment',ru:'Комментарий'},
        group:{en:'Miscellaneous',ru:'Разное'},
        control:'edit',
        multiline: true,
        wordwrap: true,
        bottom: true,
    },

};
	let drv_alarm={
	typename: {
		ru: "Тревожная переменная",
		en: "Alarm var",
	},
	drvdesc: {
		ru: "Тревожные переменные",
		en: "Alarm vars",
	},
	dontApplyCommonProps: true,
	defaultVarName:'alarm',
	properties:[
		commonDriverParams.name,
		commonDriverParams.extra_server_active,
		commonDriverParams.trend,
		change(commonDriverParams.buffer,{def:1}),

		commonDriverParams.gateway,
		commonDriverParams.extra_server_tags,
		commonDriverParams.extra_server_description,
		commonDriverParams.drv_expr_unrel,

		{
			key:'drv_node_name',
			control:'input',
			def:'%name%',
			hide:true,
		},

		{
			key:'drv_type',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Type',ru:'Тип'},
			control:'select',
			def:0,
			drvdef:0,
			undef:true,
			list:[
				[0,{en:'Regular alarm',ru:'Обычная тревога'}],
				[1,{en:'Summary alarm',ru:'Сводная тревога'}],
			],
			help:{
				ru:`
					Обычная тревога принимает в качестве сигнала значение одной или нескольких указанных переменных и обрабатывает его в соответствии с заданной уставкой и другими параметрами.

					Сводная тревога принимает в качестве сигнала все тревоги узла, удовлетворяющие фильтрам. Перечисление переменных не требуется. Сводная тревога активируется при активации хотя бы одной из наблюдаемых тревог и нормализуется только при нормализации всех наблюдаемых тревог.

					В качестве фильтров для отбора наблюдаемых тревог из всех тревог узла могут выступать теги и классы тревог.

					ВНИМАНИЕ! Сводная тревога не отображается в списке тревог, но может быть использована как переменная на менмосхеме, в скриптах или в выражениях виртуальных переменных.
				`,
				en:`
					A regular alarm receives the value of one or more specified variables as a signal and processes it in accordance with the specified setpoint and other parameters.

					The summary alarm takes as a signal all the node alarms that meet are filters. No listing of variables is required. The summary alarm is activated when at least one of the observed alarms is activated and is normalized only when all observed alarms are normalized.
					
					Tags and alarm classes can be used as filters for selecting monitored alarms from all alarms of a node.

					ATTENTION! The summary alarm is not displayed in the alarm list, but can be used as a variable on the schematic diagram, in scripts or in virtual variable expressions.
				`
			}
		},

		{
			key:'extra_polling',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Polling mode',ru:'Режим опроса'},
			control:'select',
			def:'subs',
			undef:true,
			list:[
				['subs',{en:'Subscription',ru:'Подписка'}],
				['period',{en:'Period',ru:'Период'}],
				['cron','Cron'],
			],
			help:{
				ru:`
					Режим опроса переменных:
					
					<b>Подписка</b>: для обычной тревоги опрос выполняется при каждом изменении хотя бы одной из наблюдаемых переменных. Для сводной тревоги опрос выполняется при первой возможности после изменения одной из наблюдаемых переменных.

					<b>Период</b>: опрос выполняется с интервалом, указанным в параметре "Период"
	
					<b>Cron</b>: опрос выполняется по расписанию в формате Cron
				`,
				en:`
					Variable polling mode:
						
					<b>Period</b>: polling is performed at the interval specified in the "Period" parameter
	
					<b>Cron</b>: polling is performed according to the schedule in Cron format
				`
			}
		},

		change(commonDriverParams.drv_period,{group:{en:'Mode',ru:'Режим'},def:1000}),
		change(commonDriverParams.drv_cron,{group:{en:'Mode',ru:'Режим'}}),

		{
			key:'drv_summary_priority',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Priority',ru:'Приоритет'},
			control:'select',
			def:"unack",
			drvdef:"unack",
			undef:true,
			visibleIf:'drv_type=1',
			list:[
				["unack",{en:'Unacknowledged alarms',ru:'Неквитированные тревоги'}],
				["act",{en:'Active alarms',ru:'Активные тревоги'}],
			],
			help:{
				ru:`
					Значением сводной тревоги является значение максимальной тревоги, которая определяется функцией агрегации. Данная опция определяет, какие тревоги будут иметь высший приритет для функции агрегации.

					<b>Неквитированные тревоги</b>: неквитированные тревоги имеют высший приоритет перед активными тревогами (по умолчанию)
					<b>Активные тревоги</b>: активные тревоги имеют высший приоритет перед неквитированными тревогами
				`,
				en:`
					The aggregate alarm value is the maximum alarm value, which is determined by the aggregation function. This option determines which alarms will have the highest priority for the aggregation function.

					<b>Unacknowledged alarms</b>: Unacknowledged alarms have the highest priority over active alarms (default)
					<b>Active alarms</b>: Active alarms have the highest priority over unacknowledged alarms
				`
			}
		},

		{
			key:'drv_summary_filter_classes',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Class filter',ru:'Фильтр по классам'},
			control:'select',
			def:0,
			drvdef:0,
			undef:true,
			visibleIf:'drv_type=1',
			list:[
				[0,{en:'All classes',ru:'Все классы'}],
				[1,{en:'Selected classes',ru:'Выбранные классы'}],
			],
		},

		{
			key:'drv_summary_filter_class_alarm',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Class "Alarm"',ru:'Класс "Авария"'},
			control:'checkbox',
			def:0,
			drvdef:0,
			undef:true,
			visibleIf:'drv_type=1;drv_summary_filter_classes=1',
		},
		{
			key:'drv_summary_filter_class_fault',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Class "Fault"',ru:'Класс "Неисправность"'},
			control:'checkbox',
			def:0,
			drvdef:0,
			undef:true,
			visibleIf:'drv_type=1;drv_summary_filter_classes=1',
		},
		{
			key:'drv_summary_filter_class_warning',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Class "Warning"',ru:'Класс "Предупреждение"'},
			control:'checkbox',
			def:0,
			drvdef:0,
			undef:true,
			visibleIf:'drv_type=1;drv_summary_filter_classes=1',
		},
		{
			key:'drv_summary_filter_class_notice',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Class "Notice"',ru:'Класс "Уведомление"'},
			control:'checkbox',
			def:0,
			drvdef:0,
			undef:true,
			visibleIf:'drv_type=1;drv_summary_filter_classes=1',
		},
		{
			key:'drv_summary_filter_tags',
			group:{en:'Node tag filter',ru:'Фильтр по тегам узла'},
			caption:'',
			control:'edit',
			wordwrap: true,
			multiline:true,
			minheight:50,
			visibleIf:'drv_type=1',
			groupHelp: {
				ru:`
				Логическое выражение с именами тегов. Если выражение задано, сводная тревога будет наблюдать только те тревоги узла, которые по набору тегов удовлетворяют данному выражению.

				Примеры:

				fire_alarm
				ventilation & !fire_alarm
				ventilation | heating | cooling

				`,
				en:`
				A logical expression with tag names. If the expression is specified, the aggregate alarm will only observe those node alarms that satisfy the given expression by the set of tags.

				Examples:

				fire_alarm
				ventilation & !fire_alarm
				ventilation | heating | cooling
				`
			}
		},

		{
			key:'drv_signal',
			group:{en:'Alarm condition',ru:'Условие тревоги'},
			caption:{en:'Signal',ru:'Сигнал'},
			control:'input',
			visibleIf:'drv_type=0',
			help:{
				ru:`
				В качестве сигнала используется математическое выражение с подстановкой значений одной или нескольких переменных. В простейшем случае, это может быть имя наблюдаемой переменной:

				$var1

				<hr>
				`+commonHelp.expression.ru,

				en:`
				A mathematical expression with substitution of the values of one or more variables is used as a signal. In the simplest case, this could be the name of the observed variable:

				$var1

				<hr>
				`+commonHelp.expression.en
			}
		},

		{
			key:'drv_condition',
			group:{en:'Alarm condition',ru:'Условие тревоги'},
			caption:{en:'Condition',ru:'Условие'},
			control:'select',
			def:'nz',
			drvdef:'nz',
			undef:true,
			visibleIf:'drv_type=0',
			list:[
				['nz','!=0'],
				['z','==0'],
				'==',
				'!=',
				'>',
				'<',
				'>=',
				'<=',
				['dev',{en:'Deviation',ru:'Отклонение'}],
				['roc',{en:'Rate of Change',ru:'Скорость изменения'}],
			],
			help:{
				ru:`
				Условие активации тревоги:

				!=0 - Сигнал не равен нулю
				==0 - Сигнал равен нулю
				==  - Сигнал равен Уставке
				!=  - Сигнал не равен Уставке
				>  - Сигнал больше Уставки
				<  - Сигнал меньше Уставки
				>=  - Сигнал больше или равен Уставке
				<=  - Сигнал меньше или равен Уставке
				Отклонение - Сигнал больше или меньше Уставки на величину, заданную в параметре Отклонение
				Скорость изменения - ROC (Rate of Change), величина изменения Сигнала за указанный период
				`,
				en:`
				Alarm activation condition:
				
				 !=0 - Signal is not zero
				 ==0 - Signal is zero
				 == - Signal equal to Setpoint
				 != - Signal is not equal to Setpoint
				 > - Signal greater than Setpoint
				 < - Signal less than Setpoint
				 >= - Signal greater than or equal to Setpoint
				 <= - Signal less than or equal to Setpoint
				 Deviation - Signal is greater or less than the Setpoint by the amount specified in the Deviation parameter
				 Rate of change - (ROC) the amount of change in the Signal over a specified period
				`
			}
		},

		{
			key:'drv_deviation',
			group:{en:'Alarm condition',ru:'Условие тревоги'},
			caption:{en:'Deviation',ru:'Отклонение'},
			control:'input',
			visibleIf:'drv_condition=dev;drv_type=0',
			help:{
				ru:`
				Величина допустимого отклонения Сигнала от Уставки. Активация тревоги будет происходить при отклонении Сигнала от Уставки (в большую или меньшую сторону) на величину, заданную в данном параметре.

				В качестве значения может использоваться математическое выражение с подстановкой одной или нескольких переменных.

				<hr>
				`+commonHelp.expression.ru,

				en:`
				The value of the permissible deviation of the Signal from the Setpoint. An alarm will be activated when the Signal deviates from the Setpoint (up or down) by the amount specified in this parameter.

				 The value can be a mathematical expression with substitution of one or more variables.

				 <hr>
				 `+commonHelp.expression.en
			}
		},

		{
			key:'drv_roc_period',
			group:{en:'Alarm condition',ru:'Условие тревоги'},
			caption:{en:'Period',ru:'Период'},
			control:'input',
			visibleIf:'drv_condition=roc;drv_type=0',
			help:{
				ru:`
				Период измерения скорости изменения Сигнала, миллисекунды.

				Изменение Сигнала за указанный период будет сравниваться с Уставкой. Если изменение больше или равно уставке (с учетом параметра Направление) - произойдет активация тревоги.
				`,
				en:`
				Period of measuring the rate of change of the Signal, milliseconds.
				
				 The Signal change over the specified period will be compared with the Setpoint. If the change is greater than or equal to the setpoint (taking into account the Direction parameter), an alarm will be activated.
				`
			}
		},

		{
			key:'drv_roc_dir',
			group:{en:'Alarm condition',ru:'Условие тревоги'},
			caption:{en:'Direction',ru:'Направление'},
			visibleIf:'drv_condition=roc;drv_type=0',
			control:'select',
			def:'both',
			drvdef:'both',
			list:[
				['both',{en:'Rise and Fall',ru:'Увеличение и уменьшение'}],
				['rise',{en:'Rise',ru:'Увеличение'}],
				['fall',{en:'Fall',ru:'Уменьшение'}],
			],
			help:{
				ru:`
				Направление изменения Сигнала, при котором возможна активация тревоги.
				`,
				en:`
				The direction of the Signal change that may trigger an alarm.
				`
			}
		},

		{
			key:'drv_setpoint',
			group:{en:'Alarm condition',ru:'Условие тревоги'},
			caption:{en:'Setpoint',ru:'Уставка'},
			control:'input',
			visibleIf:'drv_condition!=nz;drv_condition!=z;drv_type=0',
			help:{
				ru:`
				В качестве значения уставки может использоваться математическое выражение с подстановкой одной или нескольких переменных.

				<hr>
				`+commonHelp.expression.ru,

				en:`
				The value of the Setpoint can be a mathematical expression with substitution of one or more variables.

				<hr>
				`+commonHelp.expression.en
			}
		},


		{
			key:'drv_delay',
			group:{en:'Activation params',ru:'Параметры активации'},
			caption:{en:'Delay',ru:'Задержка'},
			control:'a-input',
			edit:'text',
			visibleIf:'drv_type=0',
			help:{
				ru:`
				Задержка перед активацией тревоги, секунды.
				Может использоваться для защиты от "дребезга" - кратковременных активаций/нормализаций тревоги.

				Если условие активации тревоги выполняется и задана задержка, тревога не будет активирована до истечения времени задержки. Если во время задержки условие активации было нарушено, следующая активация перезапускает цикл задержки. 
				
				Задержка не применяется к нормализации тревоги (тревога всегда переходит в состояние "Норма" без задержки).

				В качестве значения может использоваться математическое выражение с подстановкой одной или нескольких переменных.

				<hr>
				`+commonHelp.expression.ru
				,
				en:`
				Delay before alarm activation, s.
				The delay protects against “bouncing” - short-term alarm activations/resets.

				If the alarm activation condition is met and a delay is specified, the alarm will not be activated until the delay time has expired. If the activation condition is violated during the delay, the next activation restarts the delay cycle.
				
				The delay does not apply to trouble recovery (the alarm is always restored to Normal without delay).

				The value can be a mathematical expression with substitution of one or more variables.

				<hr>
				`+commonHelp.expression.en
			}
		},

		{
			key:'drv_deadband',
			group:{en:'Activation params',ru:'Параметры активации'},
			caption:{en:'Deadband',ru:'Мертвая зона'},
			control:'input',
			visibleIf:'drv_type=0',
			help:{
				ru:`
				Если задана мертвая зона (гистерезис), нормализация тревоги будет происходить только если разница между значением Сигнала и Уставкой сравняется или превысит по модулю значение мертвой зоны.

				Например, если условие активации тревоги <i>Сигнал >= 20</i>, при значении Сигнала, равным 20, произойдет активация тревоги. Если при этом задана мертвая зона, равная, например 3, то нормализация тревоги произойдет только при уменьшении значения Сигнала до 17.

				В качестве значения параметра может использоваться математическое выражение с подстановкой одной или нескольких переменных.

				<hr>
				`+commonHelp.expression.ru,

				en:`
				If a dead zone (hysteresis) is set, normalization of the alarm will occur only if the difference between the Signal value and the Setpoint equals or exceeds the absolute value of the dead zone.

				For example, if the alarm activation condition is <i>Signal >= 20</i>, if the Signal value is equal to 20, the alarm will be activated. If a dead zone is set equal to, for example, 3, then normalization of the accident will occur only when the Signal value is reduced to 17.

				The parameter value can be a mathematical expression with the substitution of one or more variables.

				<hr>
				`+commonHelp.expression.en
			}
		},
		
		{
			key:'drv_class',
			group:{en:'Alarm params',ru:'Параметры тревоги'},
			caption:{en:'Class',ru:'Класс'},
			control:'select',
			def:3,
			drvdef:0,
			visibleIf:'drv_type=0',
			list:[
				[3,{en:'Alarm',ru:'Авария'}],
				[2,{en:'Failure',ru:'Неисправность'}],
				[1,{en:'Warning',ru:'Предупреждение'}],
				[0,{en:'Notice',ru:'Оповещение'}],
			],
			help:{
				ru:`
				Класс тревоги определяет ее цвет и ранжирование в окне тревог.
				`,
				en:`
				The class of an alarm determines its color and ranking in the alarm window.
				`
			}
		},

		{
			key:'drv_acknowledge',
			group:{en:'Alarm params',ru:'Параметры тревоги'},
			caption:{en:'Acknowledge',ru:'Квитирование'},
			control:'checkbox',
			def:1,
			drvdef:0,
			undef:true,
			visibleIf:'drv_type=0',
			help:{
				ru:`
				Если параметр включен, тревога будет заведена как квитируемая.

				Квитируемые тревоги остаются в списке актуальных тревог даже после нормализации, до подтверждения (квитирования) оператором.

				Неквитируемые тревоги автоматически убираются из списка актуальных тревог после нормализации и не требуют подтверждения.
				`,
				en:`
				If the parameter is enabled, the alarm will be acknowledgeable.

				Acknowledged alarms remain in the list of actual alarms even after normalization, until confirmation (acknowledgment) by the operator.

				Unacknowledgeable alarms are automatically removed from the list of actual alarms after normalization and do not require confirmation.
				`
			}
		},

		{
			key:'drv_priority',
			group:{en:'Alarm params',ru:'Параметры тревоги'},
			caption:{en:'Priority',ru:'Приоритет'},
			control:'input',
			def:0,
			drvdef:0,
			visibleIf:'drv_type=0',
			help:{
				ru:`
				Тревоги с более высоким приоритетом будут находится выше в списке (в рамках тревог одного одного класса)
				`,
				en:`
				Alarms with a higher priority will be higher in the list (within the same class of alarms)
				`
			}
		},

		{
			key:'extra_server_sound',
			caption:{en:'Sound',ru:'Звук'},
			group:{en:'Alarm params',ru:'Параметры тревоги'},
			control:'select',
			undef:true,
			def:{en:'By project settings',ru:'По настройкам проекта'},
			visibleIf:'drv_type=0',
			list:[
				[0,{en:'Off',ru:'Откл'}],
				[1,{en:'On',ru:'Вкл'}],
			],
		},


		commonDriverParams.group_vars,
commonDriverParams.group_inv,
commonDriverParams.group_ignore,
commonDriverParams.group_block,
commonDriverParams.group_sync,
commonDriverParams.group_timeout,
commonDriverParams.group_tolerance,
		commonDriverParams.extra_service_remote_map_mbtcp__enabled,
commonDriverParams.service_remote_map_mbtcp__port,
commonDriverParams.service_remote_map_mbtcp__slave,
commonDriverParams.service_remote_map_mbtcp__reg_type,
commonDriverParams.service_remote_map_mbtcp__reg_addr,
commonDriverParams.service_remote_map_mbtcp__reg_off,
commonDriverParams.service_remote_map_mbtcp__format,

commonDriverParams.extra_service_local_map_mbtcp__enabled,
commonDriverParams.service_local_map_mbtcp__port,
commonDriverParams.service_local_map_mbtcp__slave,
commonDriverParams.service_local_map_mbtcp__reg_type,
commonDriverParams.service_local_map_mbtcp__reg_addr,
commonDriverParams.service_local_map_mbtcp__reg_off,
commonDriverParams.service_local_map_mbtcp__format,
		commonDriverParams.extra_service_local_exec__enabled,
commonDriverParams.service_local_exec__init,
commonDriverParams.service_local_exec__timeout,
commonDriverParams.service_local_exec__command,

		commonDriverParams.extra_server_notify,
		commonDriverParams.extra_server_comment,
	]
};
	let drv_bacnet={
	typename: {
		ru: "Переменная BACnet",
		en: "BACnet var",
	},
	drvdesc: {
		ru: "Переменные BACnet",
		en: "BACnet vars",
	},
	dontApplyCommonProps: true,
	defaultVarName:'var',
	properties:[
		commonDriverParams.name,
		commonDriverParams.extra_server_active,
		commonDriverParams.trend,
		change(commonDriverParams.buffer,{def:1}),

		commonDriverParams.gateway,
		commonDriverParams.extra_server_tags,
		commonDriverParams.extra_server_description,

		{
			key:'extra_polling',
			group:{en:'Poll',ru:'Опрос'},
			caption:{en:'Polling mode',ru:'Режим опроса'},
			control:'select',
			def:'period',
			undef:true,
			list:[
				['period',{en:'Period',ru:'Период'}],
				['cron','Cron'],
			],
			help:{
				ru:`
					Режим опроса переменных:
					
					<b>Период</b>: опрос выполняется с интервалом, указанным в параметре "Период"
	
					<b>Cron</b>: опрос выполняется по расписанию в формате Cron
				`,
				en:`
					Variable polling mode:
						
					<b>Period</b>: polling is performed at the interval specified in the "Period" parameter
	
					<b>Cron</b>: polling is performed according to the schedule in Cron format
				`
			}
		},

		change(commonDriverParams.drv_period,{group:{en:'Poll',ru:'Опрос'},def:1000}),
		change(commonDriverParams.drv_cron,{group:{en:'Poll',ru:'Опрос'}}),

		

		{
			key:'extra_data',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Data',ru:'Данные'},
			control:'select',
			def:'object',
			undef:true,
			list:[
				['object',{en:'Object',ru:'Объект'}],
				['metric',{en:'Metric',ru:'Метрика'}],
			],
			help:{
				ru:`<b>Объект</b>: в переменную будет считываться значение свойства указанного объекта
				
				<b>Метрика</b>: в переменную будет считываться статистика работы устройства
				`,
				en:`<b>Object</b>: the variable will read the value of the specified object
				<b>Metric</b>: the variable will read the statistics of the device
				`
			}
		},
		
		{
			key:'drv_metric',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Metric',ru:'Метрика'},
			control:'select',
			visibleIf:"extra_data=metric",
			list:[
				['online','Online'],
				['offline','Offline'],
			],
			help:{
				ru:`
				Работа переменной в режиме "Метрика" позволяет получать статистику о работе устройства. Все данные считаются за период опроса и для одного устройства.
				<b>Online</b>: Узел отвечает
				<b>Offline</b>: Узел не отвечает
				`,
				en:`
				The variable's operation in the "Metric" mode allows you to obtain statistics about the device's operation. All data is calculated for the polling period and for one device.
				<b>Online</b>: The node is responding
				<b>Offline</b>: The node is not responding
				`
			}
		},



		{
			key:'drv_object_type',
			group:{en:'Object',ru:'Объект'},
			caption:{en:'Object Type',ru:'Object Type'},
			control:'edit',
			list:[
				"analog-output","analog-value","binary-input","binary-output","binary-value","calendar","command","device","event-enrollment","file","group","loop","multi-state-input","multi-state-output","notification-class","program","schedule","averaging","multi-state-value","trend-log","life-safety-point","life-safety-zone","accumulator","pulse-converter","event-log","global-group","trend-log-multiple","load-control","structured-view","access-door","access-credential","access-point","access-rights","access-user","access-zone","credential-data-input","network-security","bitstring-value","characterstring-value","date-pattern-value","date-value","datetime-pattern-value","datetime-value","integer-value","large-analog-value","octetstring-value","positive-integer-value","time-pattern-value","time-value","notification-forwarder","alert-enrollment","channel","lighting-output","binary-lighting-output","network-port","elevator-group","escalator","lift","staging","audit-log","audit-reporter","color","color-temperature",
			],
			sort:true,
			undef:true,
			visibleIf:"extra_data=object",
		},
		{
			key:'drv_object_instance',
			group:{en:'Object',ru:'Объект'},
			caption:{en:'Object Instance',ru:'Object Instance'},
			control:'input',
			def:0,
			drvdef:0,
			visibleIf:"extra_data=object",
		},
		{
			key:'drv_object_property',
			group:{en:'Object',ru:'Объект'},
			caption:{en:'Object Property',ru:'Object Property'},
			def:'present-value',
			control:'edit',
			sort:true,
			undef:true,
			visibleIf:"extra_data=object",
			list:["acked-transitions","ack-required","action","action-text","active-text","active-vt-sessions","alarm-value","alarm-values","all","all-writes-successful","apdu-segment-timeout","apdu-timeout","application-software-version","archive","bias","change-of-state-count","change-of-state-time","notification-class","controlled-variable-reference","controlled-variable-units","controlled-variable-value","cov-increment","datelist","daylight-savings-status","deadband","derivative-constant","derivative-constant-units","description","description-of-halt","device-address-binding","device-type","effective-period","elapsed-active-time","error-limit","event-enable","event-state","event-type","exception-schedule","fault-values","feedback-value","file-access-method","file-size","file-type","firmware-revision","high-limit","inactive-text","in-process","instance-of","integral-constant","integral-constant-units","issue-confirmednotifications","limit-enable","list-of-group-members","list-of-object-property-references","list-of-session-keys","local-date","local-time","location","low-limit","manipulated-variable-reference","maximum-output","max-apdu-length-accepted","max-info-frames","max-master","max-pres-value","minimum-off-time","minimum-on-time","minimum-output","min-pres-value","model-name","modification-date","notify-type","number-of-APDU-retries","number-of-states","object-identifier","object-list","object-name","object-property-reference","object-type","optional","out-of-service","output-units","event-parameters","polarity","present-value","priority","priority-array","priority-for-writing","process-identifier","program-change","program-location","program-state","proportional-constant","proportional-constant-units","protocol-conformance-class","protocol-object-types-supported","protocol-services-supported","protocol-version","read-only","reason-for-halt","recipient","recipient-list","reliability","relinquish-default","required","resolution","segmentation-supported","setpoint","setpoint-reference","state-text","status-flags","system-status","time-delay","time-of-active-time-reset","time-of-state-count-reset","time-synchronization-recipients","units","update-interval","utc-offset","vendor-identifier","vendor-name","vt-classes-supported","weekly-schedule","attempted-samples","average-value","buffer-size","client-cov-increment","cov-resubscription-interval","current-notify-time","event-time-stamps","log-buffer","log-device-object-property","enable","log-interval","maximum-value","minimum-value","notification-threshold","previous-notify-time","protocol-revision","records-since-notification","record-count","start-time","stop-time","stop-when-full","total-record-count","valid-samples","window-interval","window-samples","maximum-value-timestamp","minimum-value-timestamp","variance-value","active-cov-subscriptions","backup-failure-timeout","configuration-files","database-revision","direct-reading","last-restore-time","maintenance-required","member-of","mode","operation-expected","setting","silenced","tracking-value","zone-members","life-safety-alarm-values","max-segments-accepted","profile-name","auto-slave-discovery","manual-slave-address-binding","slave-address-binding","slave-proxy-enable","last-notify-record","schedule-default","accepted-modes","adjust-value","count","count-before-change","count-change-time","COV-period","input-reference","limit-monitoring-interval","logging-object","logging-record","prescale","pulse-rate","scale","scale-factor","update-time","value-before-change","value-set","value-change-time","align-intervals","interval-offset","last-restart-reason","logging-type","time-of-device-restart","time-synchronization-interval","trigger","utc-time-synchronization-recipients","node-subtype","node-type","structured-object-list","subordinate-annotations","subordinate-list","actual-shed-level","duty-window","expected-shed-level","full-duty-baseline","requested-shed-level","shed-duration","shed-level-descriptions","shed-levels","state-description","door-alarm-state","door-extended-pulse-time","door-members","door-open-too-long-time","door-pulse-time","door-status","door-unlock-delay-time","lock-status","masked-alarm-values","secured-status","absentee-limit","access-alarm-events","access-doors","access-event","access-event-authentication-factor","access-event-credential","access-event-time","access-transaction-events","accompaniment","accompaniment-time","activation-time","active-authentication-policy","assigned-access-rights","authentication-factors","authentication-policy-list","authentication-policy-names","authentication-status","authorization-mode","belongs-to","credential-disable","credential-status","credentials","credentials-in-zone","days-remaining","entry-points","exit-points","expiration-time","extended-time-enable","failed-attempt-events","failed-attempts","failed-attempts-time","last-access-event","last-access-point","last-credential-added","last-credential-added-time","last-credential-removed","last-credential-removed-time","last-use-time","lockout","lockout-relinquish-time","master-exemption","max-failed-attempts","members","muster-point","negative-access-rules","number-of-authentication-policies","occupancy-count","occupancy-count-adjust","occupancy-count-enable","occupancy-exemption","occupancy-lower-limit","occupancy-lower-limit-enforced","occupancy-state","occupancy-upper-limit","occupancy-upper-limit-enforced","passback-exemption","passback-mode","passback-timeout","positive-access-rules","reason-for-disable","supported-formats","supported-format-classes","threat-authority","threat-level","trace-flag","transaction-notification-class","user-external-identifier","user-information-reference","user-information-reference","user-name","user-type","uses-remaining","zone-from","zone-to","verification-time","base-device-security-policy","distribution-key-revision","do-not-hide","key-sets","last-key-server","network-access-security-policies","packet-reorder-time","security-pdu-timeout","security-time-window","supported-security-algorithm","update-key-set-timeout","backup-and-restore-state","backup-preparation-time","restore-completion-time","restore-preparation-time","bit-mask","bit-text","is-utc","group-members","group-member-names","member-status-flags","requested-update-interval","covu-period","covu-recipients","event-message-texts","event-message-texts-config","event-detection-enable","event-algorithm-inhibit","event-algorithm-inhibit-ref","time-delay-normal","reliability-evaluation-inhibit","fault-parameters","fault-type","local-forwarding-only","process-identifier-filter","subscribed-recipients","port-filter","authorization-exemptions","allow-group-delay-inhibit","channel-number","control-groups","execution-delay","last-priority","write-status","property-list","serial-number","blink-warn-enable","default-fade-time","default-ramp-rate","default-step-increment","egress-time","in-progress","instantaneous-power","lighting-command","lighting-command-default-priority","max-actual-value","min-actual-value","power","transition","egress-active","inteface-value","fault-high-limit","fault-low-limit","low-diff-limit","strike-count","strike-count","default-timeout","initial-timeout","last-state-change","state-change-values","timer-running","timer-state","apdu-length","ip-address","ip-default-gateway","ip-dhcp-enable","ip-dhcp-lease-time","ip-dhcp-lease-time-remaining","ip-dhcp-server","ip-dns-server","bacnet-ip-global-address","bacnet-ip-mode","bacnet-ip-multicast-address","bacnet-ip-nat-traversal","ip-subnet-mask","bacnet-ip-udp-port","bbmd-accept-fd-registrations","bbmd-broadcast-distribution-table","bbmd-foreign-device-table","changes-pending","command","fd-bbmd-address","fd-subscription-lifetime","link-speed","link-speeds","link-speed-autonegotiate","mac-address","network-interface-name","network-number","network-number-quality","network-type","routing-table","virtual-mac-address-table","command-time-array","current-command-priority","last-command-time","value-source","value-source-array","bacnet-ipv6-mode","ipv6-address","ipv6-prefix-length","bacnet-ipv6-udp-port","ipv6-default-gateway","bacnet-ipv6-multicast-address","ipv6-dns-server","ipv6-auto-addressing-enable","ipv6-dhcp-lease-time","ipv6-dhcp-lease-time-remaining","ipv6-dhcp-server","ipv6-zone-index","assigned-landing-calls","car-assigned-direction","car-door-command","car-door-status","car-door-text","car-door-zone","car-drive-status","car-load","car-load-units","car-mode","car-moving-direction","car-position","elevator-group","energy-meter","energy-meter-ref","escalator-mode","fault-signals","floor-text","group-id","group-mode","higher-deck","installation-id","landing-calls","landing-call-control","landing-door-status","lower-deck","machine-room-id","making-car-call","next-stopping-floor","operation-direction","passenger-alarm","power-mode","registered-car-call","active-cov-multiple-subscriptions","protocol-level","reference-port","deployed-profile-location","profile-location","tags","subordinate-node-types","subordinate-tags","subordinate-relationships","default-subordinate-relationship","represents","default-present-value","present-stage","stages","stage-names","target-references","audit-source-level","audit-level","audit-notification-recipient","audit-priority-filter","auditable-operations","delete-on-forward","maximum-send-delay","monitored-objects","send-now","floor-number","device-uuid","additional-reference-ports","certificate-signing-request-file","command-validation-result","issuer-certificate-files","max-bvlc-length-accepted","max-npdu-length-accepted","operational-certificate-file","current-health","sc-connect-wait-timeout","sc-direct-connect-accept-enable","sc-direct-connect-accept-uris","sc-direct-connect-binding","sc-direct-connect-connection-status","sc-direct-connect-initiate-enable","sc-disconnect-wait-timeout","sc-failed-connection-requests","sc-failover-hub-connection-status","sc-failover-hub-uri","sc-hub-connector-state","sc-hub-function-accept-uris","sc-hub-function-binding","sc-hub-function-connection-status","sc-hub-function-enable","sc-heartbeat-timeout","sc-primary-hub-connection-status","sc-primary-hub-uri","sc-maximum-reconnect-time","sc-minimum-reconnect-time","color-override","color-reference","default-color","default-color-temperature","override-color-reference","color-command","high-end-trim","low-end-trim","trim-fade-time",]
		},



		{
			key:'drv_node_device_id',
			group:{en:'Device',ru:'Устройство'},
			caption:{en:'Device ID',ru:'Device ID'},
			control:'input',
		},
		{
			key:'drv_node_udp_port',
			group:{en:'Device',ru:'Устройство'},
			caption:{en:'UDP port',ru:'UDP-порт'},
			def:47808,
			drvdef:47808,
			control:'input',
		},
		{
			key:'drv_node_timeout',
			group:{en:'Device',ru:'Устройство'},
			caption:{en:'Timeout',ru:'Таймаут'},
			def:100,
			drvdef:100,
			control:'input',
		},
		{
			key:'drv_node_broadcast_ip',
			group:{en:'Device',ru:'Устройство'},
			caption:{en:'Broadcast IP',ru:'Broadcast IP'},
			control:'edit',
			list:[
				"192.168.0.255",
				"192.168.1.255",
			],
			undef:true,
			def:'auto',
			drvdef:'auto'
		},



		commonDriverParams.round,
commonDriverParams.threshold,
commonDriverParams.threstime,
commonDriverParams.min,
commonDriverParams.max,
commonDriverParams.coef,
commonDriverParams.offset,
commonDriverParams.scl_hi,
commonDriverParams.scl_lo,            
commonDriverParams.raw_hi,
commonDriverParams.raw_lo,
commonDriverParams.reset_interval,
commonDriverParams.reset_value,
commonDriverParams.group_vars,
commonDriverParams.group_inv,
commonDriverParams.group_ignore,
commonDriverParams.group_block,
commonDriverParams.group_sync,
commonDriverParams.group_timeout,
commonDriverParams.group_tolerance,
		commonDriverParams.group_vars,
commonDriverParams.group_inv,
commonDriverParams.group_ignore,
commonDriverParams.group_block,
commonDriverParams.group_sync,
commonDriverParams.group_timeout,
commonDriverParams.group_tolerance,
		commonDriverParams.extra_service_remote_map_mbtcp__enabled,
commonDriverParams.service_remote_map_mbtcp__port,
commonDriverParams.service_remote_map_mbtcp__slave,
commonDriverParams.service_remote_map_mbtcp__reg_type,
commonDriverParams.service_remote_map_mbtcp__reg_addr,
commonDriverParams.service_remote_map_mbtcp__reg_off,
commonDriverParams.service_remote_map_mbtcp__format,

commonDriverParams.extra_service_local_map_mbtcp__enabled,
commonDriverParams.service_local_map_mbtcp__port,
commonDriverParams.service_local_map_mbtcp__slave,
commonDriverParams.service_local_map_mbtcp__reg_type,
commonDriverParams.service_local_map_mbtcp__reg_addr,
commonDriverParams.service_local_map_mbtcp__reg_off,
commonDriverParams.service_local_map_mbtcp__format,
		commonDriverParams.extra_service_local_exec__enabled,
commonDriverParams.service_local_exec__init,
commonDriverParams.service_local_exec__timeout,
commonDriverParams.service_local_exec__command,

		commonDriverParams.extra_server_comment,
	]
};
	let drv_virt={
	typename: {
		ru: "Виртуальная переменная",
		en: "Virtual variable",
	},
	drvdesc: {
		ru: "Виртуальные переменные",
		en: "Virtual vars",
	},
	dontApplyCommonProps: true,
	properties:[
		
		commonDriverParams.name,
		commonDriverParams.extra_server_active,
		commonDriverParams.extra_server_unit,
		commonDriverParams.trend,
		commonDriverParams.buffer,
		commonDriverParams.gateway,
		commonDriverParams.readonly,
		commonDriverParams.extra_server_tags,
		commonDriverParams.extra_server_description,
		commonDriverParams.extra_polling,
		commonDriverParams.drv_period,
		commonDriverParams.drv_cron,

		{
			key:'extra_data',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Data',ru:'Данные'},
			control:'select',
			def:'calc',
			list:[
				['calc',{en:'Stored value',ru:'Хранимое значение'}],
				['expr',{en:'Expression',ru:'Выражение'}],
				['ping',{en:'Ping',ru:'Пинг'}],
				['time',{en:'Time',ru:'Время'}],
				['split',{en:'Split variable',ru:'Раздельная переменная'}],
				['aggr',{en:'Aggregation (Server only)',ru:'Агрегация (только Сервер)'}],
			],
			help:{
				ru:`
				Хранимое значение - переменная хранит собственное значение, которое может быть модифицировано модификатором (например, инкрементироваться). Данный режим может быть использован для хранения вспомогательных значений или генерации случайных чисел. Значение переменной сохраняется при перезагрузке драйвера (если в конфиге драйвера не указано другое поведение).

				Выражение - значением переменной является результат вычисления математического выражения, которое может включать в себя другие переменные. Данный режим может быть использован для получения расчетных значений.

				Пинг - значением переменной является время ответа на ICMP-пинг указанного хоста (мс). При успешном пинге значение всегда будет больше или равно 1.

				Время - значением переменной является одна из метрик текущей даты/времени (секунды, день недели, timestamp и тп).

				Раздельная переменная - значение раздельной переменной читается из читаемой переменной, а записывается в записываемую переменную. При этом, возможна принудительная синхронизация значения записываемой переменной, чтобы оно всегда соответствовало значению читаемой переменной. Раздельная переменная может быть использована при работе с Modbus-устройствами, где чтение сигнала выполняется из регистра IR, а запись - в регистр HR. 

				Агрегация - значением переменной является результат агрегационной функции, для которой массивом данных служат значения всех переменных сервера, удовлетворяющих фильтру по тегам. <b>Данный режим работает только на стороне сервера (переменные на шлюзах и в узлах за шлюзами не будут работать в данном режиме)</b>.
				`,
				en:`
				Stored value - the variable stores its own value, which can be modified by the modifier (for example, incremented). This mode can be used to store auxiliary values ​​or generate random numbers. The variable value is saved when the driver is rebooted (unless another behavior is specified in the driver config).

				Expression - the variable value is the result of calculating a mathematical expression, which can include other variables. This mode can be used to obtain calculated values.

				Ping - the variable value is the response time to the ICMP ping of the specified host (ms). If the ping is successful, the value will always be greater than or equal to 1.

				Time - the variable value is one of the current date/time metrics (seconds, day of the week, timestamp, etc.).

				Aggregation - the variable value is the result of an aggregation function, for which the data array is the values ​​of all server variables that satisfy the filter by tags. <b>This mode works only on the server side (variables on gateways and in nodes behind gateways will not work in this mode)</b>.
				`,
			}
		},

		

		{
			key:'drv_calc',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Modificator',ru:'Модификатор'},
			control:'select',
			undef:true,
			def:undefined,
			list:[
				['incr',{en:'Increment',ru:'Инкремент'}],
				['decr',{en:'Decrement',ru:'Декремент'}],
				['swap',{en:'Swap',ru:'Переключение'}],
				['rand',{en:'Random values',ru:'Случайные значения'}],
			],
			help:{
				ru:"Значение будет изменяться при каждом опросе выбраным методом на величину, заданную в параметре  'Дельта'",
				en:"The value will change at each polling with the selected method by the value set in the 'Delta' parameter"
			},
			visibleIf:'extra_data=calc'
		},

		{
			key:'drv_delta',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Delta',ru:'Дельта'},
			control:'input',
			def:1,
			drvdef:1,
			visibleIf:'drv_calc;extra_data=calc'
		},

		{
			key:'drv_initval',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Initial value',ru:'Начальное значение'},
			control:'input',
			def:0,
			drvdef:0,
			visibleIf:'extra_data=calc'
		},

		{
			key:'drv_min',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Minimal value',ru:'Минимальное значение'},
			control:'input',
			visibleIf:'extra_data=calc',
		},

		{
			key:'drv_max',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Maximal value',ru:'Максимальное значение'},
			control:'input',
			visibleIf:'extra_data=calc'
		}, 

		change(commonDriverParams.drv_expr_unrel,{
			group:{en:'Mode',ru:'Режим'},
			visibleIf:'extra_data=expr',
		}),

		{
			key:'drv_counter',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Counter',ru:'Счетчик'},
			control:'edit',
			undef:true,
			def:0,
			drvdef:0,
			listholder:true,
			checkbox:true,
			list:[
				[0,{ru:'0 - откл',en:'0 - off'}],
				[1,{ru:'1 - вкл',en:'1 - on'}],
			],
			visibleIf:'extra_data=expr',
			help:{
				ru:`
				Прибавлять рассчитанное значение выражения к текущему значению переменной. Если задан период или cron (Опрос), прибавление выполняется по указанному времени. Иначе - при каждом изменении значения выражения.
				`,
				en:`
				Add the calculated value of an expression to the current value of a variable. If a period or cron is specified (Polling), the addition is performed at the specified time. Otherwise - every time the value of the expression changes.
				`
			}
		},

		{
			key:'drv_nosave',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'No save',ru:'Не сохранять'},
			control:'edit',
			undef:true,
			def:0,
			drvdef:0,
			listholder:true,
			checkbox:true,
			list:[
				[0,{ru:'0 - откл',en:'0 - off'}],
				[1,{ru:'1 - вкл',en:'1 - on'}],
			],
			visibleIf:'extra_data=calc,expr',
			help:{
				ru:`
				Не сохранять значение переменной на диске. Если опция включена, значение переменной не будет сохраняться в постоянной памяти устройства и при перезагрузке будет потеряно (сброшено на начальное значение).

				Включение данной опции позволяет существенно повысить производительность при работе с часто обновляемым большим количеством виртуальных переменных (при общем количестве изменений в секунду от сотен тысяч).
				`,
				en:`
				Do not save the variable's value to disk. If the option is enabled, the value of the variable will not be saved in the device's permanent memory and will be lost (reset to the initial value) upon reboot.

  				Enabling this option can significantly improve performance when working with a large number of virtual variables that are frequently updated (with a total number of changes per second of hundreds of thousands).
				`
			}
		},
		

		{
			key:'drv_time',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Time',ru:'Время'},
			control:'select',
			def:'timestamp',
			list:[
				["timestamp",{en:'Timestamp, s',ru:'Timestamp, с'}],
				["timestampms",{en:'Timestamp, ms',ru:'Timestamp, мс'}],
				["mday",{en:'Month day',ru:'День месяца'}],
				["wday",{en:'Week day',ru:'День недели'}],
				["mon",{en:'Month',ru:'Месяц'}],
				["year",{en:'Year',ru:'Год'}],
				["hour",{en:'Hour',ru:'Час'}],
				["min",{en:'Minute',ru:'Минута'}],
				["sec",{en:'Second',ru:'Секунда'}],
				["zone",{en:'Time Zone',ru:'Временная зона'}],
			],
			visibleIf:'extra_data=time'
		},

		{
			key:'drv_split',
			def:1,
			drvdef:0,
			hide:1,
			visibleIf:'extra_data=split'
		},

		{
			key:'drv_split_read_var',
			group:{en:'Split variables',ru:'Раздельная переменная'},
			caption:{en:'Read variable',ru:'Читаемая переменная'},
			control:'input',
			visibleIf:'extra_data=split',
			help:{
				ru:`
				Имя переменной с символом $, которая будет использована для чтения значения. 
				Например: $setpoint_ir
				`,
				en:`
				The name of the variable with the $ symbol that will be used to read the value.
				For example: $setpoint_ir
				`
			},
		},
		
		{
			key:'drv_split_write_var',
			group:{en:'Split variables',ru:'Раздельная переменная'},
			caption:{en:'Write variable',ru:'Записываемая переменная'},
			control:'input',
			visibleIf:'extra_data=split',
			help:{
				ru:`
				Имя переменной с символом $, которая будет использована для записи значения. 
				Например: $setpoint_hr
				`,
				en:`
				The name of the variable with the $ symbol that will be used to write the value.
				For example: $setpoint_hr
				`
			},
		},

		{
			key:'drv_split_sync',
			group:{en:'Split variables',ru:'Раздельная переменная'},
			caption:{en:'Syncronization',ru:'Синхронизация'},
			control:'edit',
			undef:true,
			def:1,
			drvdef:1,
			listholder:true,
			checkbox:true,
			list:[
				[0,{ru:'0 - откл',en:'0 - off'}],
				[1,{ru:'1 - вкл',en:'1 - on'}],
			],
			visibleIf:'extra_data=split',
			help:{
				ru:`
				Синхронизация значения записываемой переменной по значению читаемой перемнной. Если опция включена, драйвер будет отслеживать изменения значения записываемой переменной. При несоответствии этого значения значению читаемой переменной в записываемую переменную будет отправлено значение читаемой переменной.
				`,
				en:`
				Synchronization of the value of the variable being written by the value of the variable being read. If the option is enabled, the driver will track changes in the value of the variable being written. If this value does not match the value of the variable being read, the value of the variable being read will be sent to the variable being written.
				`
			}
		},

		{
			key:'drv_split_sync_delay',
			group:{en:'Split variables',ru:'Раздельная переменная'},
			caption:{en:'Sync delay',ru:'Задержка синхронизации'},
			control:'input',
			visibleIf:'extra_data=split;drv_split_sync',
			def:10000,
			drvdef:10000,
			help:{
				ru:`
				Задержка, мс, между обнаружением разницы в значениях читаемой и записываемой переменной и выполнением запроса синхронизации. Задержка требуется для того, чтобы дать возможность узлу применить значение, полученное из записываемой переменной.
				`,
				en:`
				The delay, in ms, between detecting a difference in the values ​​of the variable being read and written and executing the synchronization request. The delay is required to allow the node to apply the value obtained from the variable being written.
				`
			},
		},

		{
			key:'drv_split_unrel',
			group:{en:'Split variables',ru:'Раздельная переменная'},
			caption:{en:'Unreliable',ru:'Недостоверно'},
			control:'select',
			drvdef:0,
			def:0,
			undef:true,
			list:[
				[0,{ru:'Ошибка',en:'Error'}],
				[1,{ru:'Ноль',en:'Zero'}],
				[2,{ru:'Значение',en:'Value'}],
			],
			visibleIf:'extra_data=split',
			help:{
				ru:`
				Параметр определяет метод обработки читаемой и записываемой переменных, значение которых недостоверно (переменная в ошибке).
	
				Ошибка - раздельная переменная будет установлена в ошибку DRV_ERROR_UNRELIABLE_VAR.
	
				Ноль - раздельной переменной будет установлено значение 0.
	
				Значение - раздельной переменной будет установлено последне достоверное значение читаемой перемнной.
				`,
				en:`
				The parameter defines the method of processing read and written variables whose value is unreliable (variable in error).

				Error - the split variable will be set to the error DRV_ERROR_UNRELIABLE_VAR.

				Zero - the split variable will be set to 0.

				Value - the split variable will be set to the last reliable value of the read variable.
				`
			}
		},


		{
			key:'drv_aggr_function',
			group:{en:'Aggregation',ru:'Агрегация'},
			caption:{en:'Function',ru:'Функция'},
			control:'select',
			drvdef:'',
			def:'summ',
			undef:true,
			list:[
				["summ",{en:'Summ',ru:'Сумма'}],
				["avg",{en:'Average',ru:'Среднее'}],
				["min",{en:'Minimum',ru:'Минимум'}],
				["max",{en:'Maximum',ru:'Максимум'}],
				["errors",{en:'Errors',ru:'Ошибки'}],
				["count",{en:'Count',ru:'Количество'}],
			],
			visibleIf:'extra_data=aggr'
		},

		{
			key:'drv_aggr_scope',
			group:{en:'Aggregation',ru:'Агрегация'},
			caption:{en:'Scope',ru:'Область видимости'},
			control:'select',
			drvdef:'node',
			def:'node',
			undef:true,
			list:[
				["node",{en:'Current node',ru:'Текущий узел'}],
				["project",{en:'Current project',ru:'Текущий проект'}],
				["server",{en:'Entire server',ru:'Весь сервер'}],
			],
			help:{
				ru:`
				Агрегационная функция будет обрабатывать переменные только из выбранной области (с учетом фильтров по тегам и значениям).
				`,
				en:`
				The aggregation function will process variables only from the selected scope (taking into account filters by tags and values).
				`
			},
			visibleIf:'extra_data=aggr'
		},

		{
			key:'drv_aggr',
			group:{en:'Aggregation',ru:'Агрегация'},
			caption:{en:'Tag filter',ru:'Фильтр по тегам'},
			control:'input',
			help:{
				ru:`
				Логическое выражение с именами тегов. Все перемнные сервера, удовлетворяющие фильтру, будут источником данных для функции агрегации.

				Примеры:

				fire_alarm
				ventilation & !fire_alarm
				ventilation | heating | cooling

				`,
				en:`
				A logical expression with tag names. All server variables that satisfy the filter will be the source of data for the aggregation function.

				Examples:

				fire_alarm
				ventilation & !fire_alarm
				ventilation | heating | cooling
				`
			},
			visibleIf:'extra_data=aggr'
		},

		{
			key:'drv_aggr_val_filter',
			group:{en:'Aggregation',ru:'Агрегация'},
			caption:{en:'Value filter',ru:'Фильтр по значению'},
			control:'input',
			help:{
				ru:`
				Математическое выражение, принимающее в "x" значение агрегируемой переменной. Если выражение задано, то функция агрегации обработает только переменные, выражение с которыми вернет значение отличное от нуля.

				Пример 1
				Требуется посчитать количество датчиков температуры, значение которых больше 20 и меньше или равно 25. Решение:
				
				1) Выбираем функцию агрегации 'Количество'
				2) Задаем в данном поле выражение '(x > 20) && (x <= 25)'

				Пример 2
				Требуется найти сигнал с минимальным значением среди сигналов с установленным битом 15. Решение:

				1) Выбираем функцию агрегации 'Минимум'
				2) Задаем в данном поле выражение 'bit(x,15)'
				`,
				en:`
				A mathematical expression that takes the value of the variable being aggregated in "x". If the expression is specified, the aggregation function will only process variables whose expression returns a value different from zero.

				Example 1
				It is necessary to count the number of temperature sensors whose value is greater than 20 and less than or equal to 25. Solution:

				1) Select the 'Quantity' aggregation function
				2) Specify the expression '(x > 20) && (x <= 25)' in this field

				Example 2
				It is necessary to find the signal with the minimum value among signals with bit 15 set. Solution:

				1) Select the 'Minimum' aggregation function
				2) Specify the expression 'bit(x,15)' in this field
				`
			},
			visibleIf:'extra_data=aggr'
		},

		{
			key:'drv_aggr_val_expr',
			group:{en:'Aggregation',ru:'Агрегация'},
			caption:{en:'Value prepare',ru:'Подготовка значения'},
			control:'input',
			help:{
				ru:`
				Математическое выражение, принимающее в "x" значение агрегируемой переменной. Если выражение задано, то функция агрегации примет в качестве значения агрегируемой переменной результат данного выражения.

				Например, требуется получить среднюю температуру с уличных датчиков, значения которых представлены с коэффициентом х10 (т.е. 217, 223, 189 и тп). Тогда для преобразования каждого значения можно задать выражение

				x / 10

				которое будет передавать функции агрегации уже преобразованные значения (21.7, 22.3, 18.9)
				`,
				en:`
				A mathematical expression that takes the value of the aggregated variable in "x". If the expression is specified, the aggregation function will take the result of this expression as the value of the aggregated variable.

				For example, you need to get the average temperature from outdoor sensors, the values ​​of which are presented with a coefficient of x10 (i.e. 217, 223, 189, etc.). Then, to transform each value, you can specify the expression

				x / 10

				which will pass the already transformed values ​​(21.7, 22.3, 18.9) to the aggregation function.
				`
			},
			visibleIf:'extra_data=aggr'
		},

		change(commonDriverParams.drv_expr_unrel,{
			key:'drv_aggr_unrel',
			group:{en:'Aggregation',ru:'Агрегация'},
			drvdef:0,
			def:2,
			visibleIf:'extra_data=aggr',
			help:{
				ru:`
				Параметр определяет метод обработки переменных функцией агрегации, значение которых недостоверно (переменная в ошибке).
	
				Ошибка - при наличии хотя бы одной недостоверной переменной, результатом расчета функции будет ошибка DRV_ERROR_UNRELIABLE_VAR.
	
				Ноль - при наличии недостоверных переменных, их значение берется равным нулю, функция рассчитывается без генерации ошибки.
	
				Значение - при наличии недостоверных переменных, их значение берется равным последнему достоверному, функция рассчитывается без генерации ошибки.
				`,
				en:`
				The parameter defines the method of processing variables by the aggregation function, the value of which is unreliable (variable in error).

				Error - if there is at least one unreliable variable, the result of calculating the function will be the error DRV_ERROR_UNRELIABLE_VAR.

				Zero - if there are unreliable variables, their value is taken equal to zero, the function is calculated without generating an error.

				Value - if there are unreliable variables, their value is taken equal to the last reliable one, the function is calculated without generating an error.
				`
			}
		}),

		{
			key:'drv_ping',
			group:{en:'Ping',ru:'Пинг'},
			caption:{en:'Host',ru:'Хост'},
			control:'input',
			def:'192.168.0.1',
			visibleIf:'extra_data=ping'
		},

		{
			key:'drv_timeout',
			group:{en:'Ping',ru:'Пинг'},
			caption:{en:'Timeout',ru:'Таймаут'},
			control:'input',
			def:5000,
			drvdef:5000,
			visibleIf:'extra_data=ping'
		},

		{
			key:'drv_retries',
			group:{en:'Ping',ru:'Пинг'},
			caption:{en:'Retries',ru:'Попытки'},
			control:'input',
			def:1,
			drvdef:1,
			visibleIf:'extra_data=ping'
		},

		{
			key:'drv_ping_noerror',
			group:{en:'Ping',ru:'Пинг'},
			caption:{en:'No errors',ru:'Без ошибок'},
			control:'checkbox',
			visibleIf:'extra_data=ping',
			help:{
				ru:`
				Если опция включена, переменная не будет переходить в статус ошибки при отсутствии ответов на ICMP-запросы. Значение переменной при этом будет установлено равным нулю.
				`,
				en:`
				If the option is enabled, the variable will not go into error status if there are no responses to ICMP requests. The value of the variable will be set equal to zero.
				`
			}
		},

		{
			key:'drv_expr',
			group:{en:'Expression',ru:'Выражение'},
			visibleIf:'extra_data=expr',
			control:'edit',
			multiline:true,
			minheight:50,
			maxheight:200,
			groupHelp:{ 
				ru:`
				Значению переменной будет присвоен результат математического выражения.

				В случае, если указан период опроса или Cron, выражение будет вычисляться с заданным интервалом. Иначе - по факту изменения любой переменной, входящей в состав выражения.
				
				<hr>
				`+commonHelp.expression.ru,

				en:`
				The value of the variable will be assigned the result of the given mathematical expression.

				If a polling period or Cron is specified, the expression will be evaluated at the specified interval. Otherwise - upon the fact of changing any variable included in the expression.
				
				<hr>
				`+commonHelp.expression.en,
			},
		},

		commonDriverParams.round,
commonDriverParams.threshold,
commonDriverParams.threstime,
commonDriverParams.min,
commonDriverParams.max,
commonDriverParams.coef,
commonDriverParams.offset,
commonDriverParams.scl_hi,
commonDriverParams.scl_lo,            
commonDriverParams.raw_hi,
commonDriverParams.raw_lo,
commonDriverParams.reset_interval,
commonDriverParams.reset_value,
commonDriverParams.group_vars,
commonDriverParams.group_inv,
commonDriverParams.group_ignore,
commonDriverParams.group_block,
commonDriverParams.group_sync,
commonDriverParams.group_timeout,
commonDriverParams.group_tolerance,
		commonDriverParams.extra_service_remote_map_mbtcp__enabled,
commonDriverParams.service_remote_map_mbtcp__port,
commonDriverParams.service_remote_map_mbtcp__slave,
commonDriverParams.service_remote_map_mbtcp__reg_type,
commonDriverParams.service_remote_map_mbtcp__reg_addr,
commonDriverParams.service_remote_map_mbtcp__reg_off,
commonDriverParams.service_remote_map_mbtcp__format,

commonDriverParams.extra_service_local_map_mbtcp__enabled,
commonDriverParams.service_local_map_mbtcp__port,
commonDriverParams.service_local_map_mbtcp__slave,
commonDriverParams.service_local_map_mbtcp__reg_type,
commonDriverParams.service_local_map_mbtcp__reg_addr,
commonDriverParams.service_local_map_mbtcp__reg_off,
commonDriverParams.service_local_map_mbtcp__format,
		commonDriverParams.extra_service_local_exec__enabled,
commonDriverParams.service_local_exec__init,
commonDriverParams.service_local_exec__timeout,
commonDriverParams.service_local_exec__command,
		commonDriverParams.sim_random,
commonDriverParams.sim_random_period,
commonDriverParams.sim_random_delta,
commonDriverParams.sim_random_init,
commonDriverParams.sim_random_min,
commonDriverParams.sim_random_max,

		commonDriverParams.extra_server_comment,
	]
};
	let drv_cam={
	typename: {
		ru: "Переменная WebCam",
		en: "WebCam var",
	},
	drvdesc: {
		ru: "Переменные драйвера WebCam",
		en: "WebCam driver vars",
	},
	dontApplyCommonProps: true,
	properties:[
		
		commonDriverParams.name,
		commonDriverParams.extra_server_active,
		commonDriverParams.extra_server_unit,
		commonDriverParams.trend,
		commonDriverParams.buffer,
		commonDriverParams.gateway,
		commonDriverParams.extra_server_tags,
		commonDriverParams.extra_server_description,

		{
			key:'drv_url',
			group:{en:'Data',ru:'Данные'},
			caption:{en:'Videostream URL',ru:'URL видеопотока'},
			control:'input',
			help:{
				ru:`
					URL видеопотока в формате MJPEG или JPEG
					Например: http://192.168.0.123/mjpg/video.mjpg

					При наличии видеопотока в значение переменной будет записываться время получения кадра, сам кадр в формате Data URL (Base64) будет сохранен в поле payload.image.

					На мнемосхеме видеопоток можно отобразить, связав поле payload с атрибутом src контрола Изображение или html-элемента IMG. Например:

					&lt;img src="$node1.vars.var1.payload.image"&gt;
				`,
				en:`
				Video stream URL in MJPEG or JPEG format
				For example: http://192.168.0.123/mjpg/video.mjpg

				If there is a video stream, the time of receiving the frame will be recorded in the value of the variable; the frame itself in Data URL (Base64) format will be saved in the payload.image field.

				A video stream can be displayed on a mnemonic diagram by associating the payload field with the src attribute of control Image or of the IMG html element. For example:

				&lt;img src="$node1.vars.P1_N1_var1.payload.image"&gt;
				`
			}
		}, 

		{
			key:'drv_checkcert',
			group:{en:'Data',ru:'Данные'},
			caption:{en:'Check certificate',ru:'Проверка сертификата'},
			control:'edit',
			undef:true,
			def:0,
			drvdef:0,
			listholder:true,
			checkbox:true,
			list:[
				[0,{ru:'0 - откл',en:'0 - off'}],
				[1,{ru:'1 - вкл',en:'1 - on'}],
			],
			help:{
				ru:`
					Включить проверку SSL-сертификата
				`,
				en:`
					Enable SSL certificate verification
				`
			}
		},

		{
			key:'drv_ssl_protocols',
			group:{en:'Data',ru:'Данные'},
			caption:{en:'SSL-protocols URL',ru:'SSL-протоколы'},
			control:'input',
			help:{
				ru:`
				OpenSSL protocol list - список протоколов для подключения к серверам, не принимающим стандартные настройки. См. https://docs.openssl.org/1.1.1/man1/ciphers
				`,
				en:`
				OpenSSL protocol list - list of protocols for connecting to servers that do not accept the default settings. See https://docs.openssl.org/1.1.1/man1/ciphers
				`
			}
		},

		{
			key:'drv_ssl_ciphers',
			group:{en:'Data',ru:'Данные'},
			caption:{en:'SSL-ciphers URL',ru:'SSL-шифры'},
			control:'input',
			help:{
				ru:`
				OpenSSL cipher list - список шифров для подключения к серверам, не принимающим стандартные настройки. См. https://docs.openssl.org/1.1.1/man1/ciphers
				`,
				en:`
				OpenSSL cipher list - list of ciphers for connecting to servers that do not accept the default settings. See https://docs.openssl.org/1.1.1/man1/ciphers
				`
			}
		},
		 

		{
			key:'drv_interval',
			group:{en:'Data',ru:'Данные'},
			caption:{en:'Interval',ru:'Интервал'},
			control:'input',
			help:{
				ru:`
					- MJPEG: Интервал обновления переменной, мс.
					Если параметр задан, переменная будет обновляться не чаще указанного интервала. Видеоданные, при этом, продолжат поступать от сервера без ограничения (особенности формата MJPEG). Таким образом, параметр Интервал не влияет на трафик между драйвером и видеосервером, но может сократить трафик между шлюзом и сервером.
					Если параметр не задан или равен нулю, перемнная обновляются с каждым новым кадром, т.е. частота обновления кадров в интерфейсе будет максимальной.

					- JPEG: Интервал опроса URL, мс.
					Если параметр не задан или равен нулю, новый запрос на получение кадра выполняется сразу после обработки предыдущего запроса. В данном формате параметр Интервал позволяет ограничить трафик между драйвером и видеосервером и, как следствие, между шлюзом и сервером.
				`,
				en:`
				- MJPEG: Variable update interval, ms.
					 If the parameter is specified, the variable will be updated no more frequently than the specified interval. Video data, at the same time, will continue to arrive from the server without restrictions (features of the MJPEG format). Therefore, the Interval setting does not affect traffic between the driver and the video server, but may reduce traffic between the gateway and the server.
					 If the parameter is not specified or equal to zero, the variable is updated with each new frame, i.e. The frame refresh rate in the interface will be maximum.

					 - JPEG: URL polling interval, ms.
					 If the parameter is not specified or equal to zero, a new frame request is executed immediately after the previous request is processed. In this format, the Interval parameter allows you to limit traffic between the driver and the video server and, as a result, between the gateway and the server.
				`
			}
		},

		{
			key:'drv_timeout',
			group:{en:'Data',ru:'Данные'},
			caption:{en:'Timeout',ru:'Таймаут'},
			edit:'text',
			control:'input',
			def:10000,
			drvdef:10000,
			help:{
				ru:`
					Максимальное время ожидания ответа видеосервера
				`,
				en:`
					Maximum waiting time for video server response
				`
			}
		},

		commonDriverParams.extra_service_local_exec__enabled,
commonDriverParams.service_local_exec__init,
commonDriverParams.service_local_exec__timeout,
commonDriverParams.service_local_exec__command,

		commonDriverParams.extra_server_comment,
	]
};
	let drv_cmd={
	typename: {
		ru: "Переменная CMD",
		en: "CMD var",
	},
	drvdesc: {
		ru: "Переменные драйвера командной строки",
		en: "Comand line driver vars",
	},
	dontApplyCommonProps: true,
	properties:[
		
		commonDriverParams.name,
		commonDriverParams.extra_server_active,
		commonDriverParams.extra_server_unit,
		commonDriverParams.trend,
		commonDriverParams.buffer,
		commonDriverParams.gateway,
		commonDriverParams.readonly,
		commonDriverParams.extra_server_tags,
		commonDriverParams.extra_server_description,
		
		change(commonDriverParams.extra_polling,{def:'period'}),
		change(commonDriverParams.drv_period,{def:1000}),
		commonDriverParams.drv_cron,

		{
			key:'drv_timeout',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Timeout',ru:'Таймаут'},
			control:'input',
			def:100,
			drvdef:100,
			help:{
				ru:"Таймаут команды, мс",
				en:"Command timeout, ms"
			}
		},

		{
			key:'drv_cache',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Cache',ru:'Кеширование'},
			control:'input',
			def:50,
			drvdef:50,
			help:{
				ru:"Время кеширование ответа, мс",
				en:"Response cache time, ms"
			}
		},

		{
			key:'drv_dcmsep',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Decimal separator',ru:'Десятичный разделитель'},
			control:'select',
			def:'.',
			drvdef:'.',
			list:[
				'.',','
			]
		},

		{
			key:'drv_wrrnd',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Write rounding',ru:'Округление записи'},
			control:'input',
			help:{
				ru:`
				Округление значения при выполнении команды записи до указанного количества знаков после запятой
				`,
				en:`
				Rounds a value when executing a write command to the specified number of decimal places
				`
			}
		},

		{
			key:'extra_format',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'Format',ru:'Формат'},
			control:'select',
			def:'num',
			drvdef:'num',
			list:[
				['num',{en:'Number',ru:'Число'}],
				['json','JSON'],
				['text',{en:'Text',ru:'Текст'}],
				['regx','Regex'],
			]
		},

		{
			key:'drv_json',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'JSON-path',ru:'JSON-путь'},
			control:'input',
			visibleIf:'extra_format=json',
			help:{
				ru:`
				Путь к параметру в JSON-объекте. Для обращения свойству объекта используйте имя свойства, а для обращения к элементу массива используйте индекс.

				Например: 
				params/temperature
				sensors/102/params/temperature

				Для получения данных из элемента массива с неизвестным заранее индексом, используйте комбинацию парамтеров JSON-путь и Условие с подстановкой символа * (звездочка):

				JSON-путь: sensors/*/params/temperature
				Условие:   sensors/*/params/id:102
				`,
				en:`
				Path to the parameter in the JSON object. To access a property of an object, use the property name, and to access an array element, use the index.

				For example:
				params/temperature
				sensors/102/params/temperature

				To obtain data from an array element with an index unknown in advance, use a combination of the JSON path and Condition parameters with the * (asterisk) symbol substitution:

				JSON path: sensors/*/params/temperature
				Condition: sensors/*/params/id:102
				`
			},
		},

		{
			key:'drv_text',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'Text pattern',ru:'Текстовый паттерн'},
			control:'input',
			visibleIf:'extra_format=text',
			help:{
				ru:"Текстовый паттерн с подстановкой ключевого слова %value% там, где требуется получить числовое значение. Например: param=pressure;value=%value%",
				en:"A text pattern with the %value% keyword substituted wherever a numeric value is required. For example: param=pressure;value=%value%"
			}
		},

		{
			key:'drv_regx',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'Regex pattern',ru:'Regex паттерн'},
			control:'input',
			visibleIf:'extra_format=regx',
			help:{
				ru:"Regex паттерн. Например: param=humidity;value=([-,0-9,.]+)",
				en:"Regex pattern. For example: param=humidity;value=([-,0-9,.]+)"
			}
		},

		{
			key:'drv_match',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'Match',ru:'Условие'},
			control:'input',
			visibleIf:'extra_format=regx,text,json',
			help:{
				ru:`Строка или json-объект (в зависимости от формата), условие для считывания значения.
				
				Если свойство задано, значение будет считываться только при соответствии условия считанным данным. Данную опцию можно использовать для фильтрации сообщений, приходящих в один топик.

				- для работы с text: строка, текстовая маска с использованием символов подстановки * и ?
				Например: *sensor_id=S75*

				- для работы с regx: строка, регулярное выражение
				Например: .*sensor_id=S75.*
				
				- для работы с json: набор условий, разделенных переносом строк.
				Например:
				путь/к/значению1:значение1
				путь/к/значению2:значение2
				sensor_id:S75
				model/vendor:Alfamatika
				`,
			}
		},

		{
			key:'drv_read',
			group:{en:'Read command',ru:'Команда чтения'},
			groupHelp:{
				ru:`
					Команда командной строки для считывания значения в переменную.
					Например: cat /dev/gpio1
					Считанный текст будет интерпретирован в соответствии с значением параметра Формат.
				`,
				en:`
				A command line command to read a value into a variable.
				For example: cat /dev/gpio1
				The read text will be interpreted in accordance with the Format parameter value.
				`
			},
			control:'edit',
		},

		{
			key:'drv_write',
			group:{en:'Write command',ru:'Команда записи'},
			groupHelp:{
				ru:`
					Команда командной строки, выполняемая при установке нового значения переменной.
					Например: echo %value% > /dev/gpio1
					Вместо %value% будет подставлено устанавливаемое значение переменной.
				`,
				en:`
				A command line command executed when a variable is set to a new value.
					 For example: echo %value% > /dev/gpio1
					 Instead of %value% the value of the variable being set will be substituted.
				`
			},
			control:'edit',
		},

		{
			key:'drv_list',
			group:{en:'Substitution list',ru:'Список подстановки'},
			control:'edit',
			multiline:1,
			groupHelp:{
				ru:`
				Список из пар Ключ:Значение разделенных переносом строки.
				При считывании текстового значения, равного Ключу, в переменную будет записано соответствующее Ключу Значение. При выполнении команды записи перобразование будет выполнено в обратную сторону (Значение -> Ключ). В именах ключей допускается использование символов * и ?. 
				
				Например:
				
				stop:1
				start:2
				*mode*:3
				*:0

				`,
				en:`
				A list of Key:Value pairs separated by line breaks.
				When reading a text value equal to the Key, the Value corresponding to the Key will be written to the variable. When executing a write command, the conversion will be performed in the opposite direction (Value -> Key).
					
					 For example:
					
					 stop:0
					 start:1
					 auto:2
				`
			},
			height:30,
		},

		commonDriverParams.round,
commonDriverParams.threshold,
commonDriverParams.threstime,
commonDriverParams.min,
commonDriverParams.max,
commonDriverParams.coef,
commonDriverParams.offset,
commonDriverParams.scl_hi,
commonDriverParams.scl_lo,            
commonDriverParams.raw_hi,
commonDriverParams.raw_lo,
commonDriverParams.reset_interval,
commonDriverParams.reset_value,
commonDriverParams.group_vars,
commonDriverParams.group_inv,
commonDriverParams.group_ignore,
commonDriverParams.group_block,
commonDriverParams.group_sync,
commonDriverParams.group_timeout,
commonDriverParams.group_tolerance,
		commonDriverParams.extra_service_remote_map_mbtcp__enabled,
commonDriverParams.service_remote_map_mbtcp__port,
commonDriverParams.service_remote_map_mbtcp__slave,
commonDriverParams.service_remote_map_mbtcp__reg_type,
commonDriverParams.service_remote_map_mbtcp__reg_addr,
commonDriverParams.service_remote_map_mbtcp__reg_off,
commonDriverParams.service_remote_map_mbtcp__format,

commonDriverParams.extra_service_local_map_mbtcp__enabled,
commonDriverParams.service_local_map_mbtcp__port,
commonDriverParams.service_local_map_mbtcp__slave,
commonDriverParams.service_local_map_mbtcp__reg_type,
commonDriverParams.service_local_map_mbtcp__reg_addr,
commonDriverParams.service_local_map_mbtcp__reg_off,
commonDriverParams.service_local_map_mbtcp__format,
		commonDriverParams.extra_service_local_exec__enabled,
commonDriverParams.service_local_exec__init,
commonDriverParams.service_local_exec__timeout,
commonDriverParams.service_local_exec__command,
		commonDriverParams.sim_random,
commonDriverParams.sim_random_period,
commonDriverParams.sim_random_delta,
commonDriverParams.sim_random_init,
commonDriverParams.sim_random_min,
commonDriverParams.sim_random_max,

		commonDriverParams.extra_server_comment,
	]
};
	let drv_gpsd={
	typename: {
		ru: "Переменная GPSD",
		en: "GPSD var",
	},
	drvdesc: {
		ru: "Переменные драйвера GPSD",
		en: "GPSD driver vars",
	},
	dontApplyCommonProps: true,
	properties:[
		
		commonDriverParams.name,
		commonDriverParams.extra_server_active,
		commonDriverParams.extra_server_unit,
		commonDriverParams.trend,
		commonDriverParams.buffer,
		commonDriverParams.gateway,
		commonDriverParams.extra_server_tags,
		commonDriverParams.extra_server_description,

		{
			key:'drv_parameter',
			group:{en:'Data',ru:'Данные'},
			caption:{en:'parameter',ru:'Параметр'},
			control:'edit',
			undef:true,
			list:[
				'lat',
				'lon',
				'alt',
				'speed',
				'climb',
				'track',
				'magtrack',
				'epy',
				'epx',
				'epv',
				'eps',
				'epc',
				'ept',
			],
			help:{
				ru:`
					Имя параметра в сервисе GPSd, значение которого будет считываться в переменную.

					Доступные параметры:

					lat         - Широта, градусы
					lon         - Долгота, градусы
					alt         - Высота над уровнем моря, метры
					speed       - Скорость горизонтальная, метры в секунду
					climb       - Скорость вертикальная, метры в секунду
					track       - Курс от истинного севера, градусы
					magtrack    - Курс от магнитного севера, градусы
					epy         - Время в формате UNIX TIMESTAMP (UTC), секунды
					epx         - Погрешность измерения широты, в метрах
					epv         - Погрешность измерения долготы, в метрах
					eps         - Погрешность измерения высоты, в метрах
					epc         - Погрешность измерения горизонтальной скорости, в метрах
					ept         - Погрешность измерения вертикальной скорости, в метрах

					Набор доступных параметров зависит от версии сервиса GPSd,
					Дополнительная информация:
					<a href='https://gpsd.gitlab.io/gpsd/client-howto.html'>https://gpsd.gitlab.io/gpsd/client-howto.html</a>
					<a href='https://gpsd.gitlab.io/gpsd/gpsd_json.html'>https://gpsd.gitlab.io/gpsd/gpsd_json.html</a>
				`,
				en:`
					The name of the parameter in the GPSd service, the value of which will be read into the variable.

					Available options:

					lat - Latitude, degrees
					lon - Longitude, degrees
					alt - Altitude above sea level, meters
					speed - Horizontal speed, meters per second
					climb - Vertical speed, meters per second
					track - Course from true north, degrees
					magtrack - Course from magnetic north, degrees
					epy - Time in UNIX TIMESTAMP (UTC) format, seconds
					epx - Latitude measurement error, in meters
					epv - Longitude measurement error, in meters
					eps - Height measurement error, in meters
					epc - Horizontal speed measurement error, in meters
					ept - Vertical velocity measurement error, in meters

					The set of available parameters depends on the version of the GPSd service,
					Additional Information:
					<a href='https://gpsd.gitlab.io/gpsd/client-howto.html'>https://gpsd.gitlab.io/gpsd/client-howto.html</a>
					<a href='https://gpsd.gitlab.io/gpsd/gpsd_json.html'>https://gpsd.gitlab.io/gpsd/gpsd_json.html</a>
				`
			}
		}, 

		{
			key:'extra_data',
			group:{en:'Data',ru:'Данные'},
			caption:{en:'Adv. options',ru:'Доп. опции'},
			control:'edit',
			undef:true,
			def:0,
			checkbox:true,
			listholder:true,
			list:[
				[0,{ru:'0 - откл',en:'0 - off'}],
				[1,{ru:'1 - вкл',en:'1 - on'}],
			],
		},

		{
			key:'drv_class',
			group:{en:'Data',ru:'Данные'},
			caption:{en:'class',ru:'Класс'},
			control:'input',
			def:'TPV',
			drvdef:'TPV',
			visibleIf:'extra_data'
		},

		{
			key:'drv_node_host',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Host',ru:'Хост'},
			control:'input',
			def:'127.0.0.1',
			drvdef:'127.0.0.1',
			visibleIf:'extra_data'
		},

		{
			key:'drv_node_port',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Port',ru:'Порт'},
			control:'input',
			def:'2947',
			drvdef:'2947',
			visibleIf:'extra_data'
		},

		{
			key:'drv_node_timeout',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Timeout',ru:'Таймаут'},
			control:'input',
			def:'10000',
			drvdef:'10000',
			visibleIf:'extra_data'
		},

		{
			key:'drv_node_watch_request',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Request',ru:'Запрос'},
			control:'input',
			def:'?WATCH={"enable":true,"json":true}',
			drvdef:'?WATCH={"enable":true,"json":true}',
			visibleIf:'extra_data'
		},

		
		commonDriverParams.round,
commonDriverParams.threshold,
commonDriverParams.threstime,
commonDriverParams.min,
commonDriverParams.max,
commonDriverParams.coef,
commonDriverParams.offset,
commonDriverParams.scl_hi,
commonDriverParams.scl_lo,            
commonDriverParams.raw_hi,
commonDriverParams.raw_lo,
commonDriverParams.reset_interval,
commonDriverParams.reset_value,
commonDriverParams.group_vars,
commonDriverParams.group_inv,
commonDriverParams.group_ignore,
commonDriverParams.group_block,
commonDriverParams.group_sync,
commonDriverParams.group_timeout,
commonDriverParams.group_tolerance,
		commonDriverParams.extra_service_remote_map_mbtcp__enabled,
commonDriverParams.service_remote_map_mbtcp__port,
commonDriverParams.service_remote_map_mbtcp__slave,
commonDriverParams.service_remote_map_mbtcp__reg_type,
commonDriverParams.service_remote_map_mbtcp__reg_addr,
commonDriverParams.service_remote_map_mbtcp__reg_off,
commonDriverParams.service_remote_map_mbtcp__format,

commonDriverParams.extra_service_local_map_mbtcp__enabled,
commonDriverParams.service_local_map_mbtcp__port,
commonDriverParams.service_local_map_mbtcp__slave,
commonDriverParams.service_local_map_mbtcp__reg_type,
commonDriverParams.service_local_map_mbtcp__reg_addr,
commonDriverParams.service_local_map_mbtcp__reg_off,
commonDriverParams.service_local_map_mbtcp__format,
		commonDriverParams.extra_service_local_exec__enabled,
commonDriverParams.service_local_exec__init,
commonDriverParams.service_local_exec__timeout,
commonDriverParams.service_local_exec__command,
		commonDriverParams.sim_random,
commonDriverParams.sim_random_period,
commonDriverParams.sim_random_delta,
commonDriverParams.sim_random_init,
commonDriverParams.sim_random_min,
commonDriverParams.sim_random_max,

		commonDriverParams.extra_server_comment,
	]
};
	let drv_gateway={
	typename: {
		ru: "Переменная драйвера шлюза",
		en: "Gateway driver var",
	},
	drvdesc: {
		ru: "Переменные драйвера шлюза",
		en: "Gateway driver vars",
	},
	drvtitle: 'GW',
	dontApplyCommonProps: true,
	properties:[
		
		commonDriverParams.name,
		commonDriverParams.extra_server_active,
		commonDriverParams.extra_server_unit,
		commonDriverParams.trend,
		commonDriverParams.buffer,
		change(commonDriverParams.gateway,{def:'[LOCAL]'}),
		commonDriverParams.readonly,
		commonDriverParams.extra_server_tags,
		commonDriverParams.extra_server_description,

		{
			key:'extra_data',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Data',ru:'Данные'},
			control:'select',
			def:'metric',
			list:[
				['varid',{en:'Remote var',ru:'Удаленная переменная'}],
				['metric',{en:'Metric',ru:'Метрика'}],
			],
			help:{
				ru:`<b>Удаленная переменная</b>: в переменную будет считываться значение удаленной переменной
				
				<b>Метрика</b>: в переменную будет считываться статистика работы шлюза
				`,
				en:`<b>Remote var</b>: the variable will read the value of the remote variable
				<b>Metric</b>: the variable will read the statistics of the gateway
				`
			}
		},

		{
			key:'drv_metric',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Metric',ru:'Метрика'},
			control:'select',
			def:'online',
			visibleIf:"extra_data=metric",
			list:[
				['online',{en:'Online',ru:'Онлайн'}],
				['offline',{en:'Offline',ru:'Офлайн'}],
				['ping',{en:'Ping',ru:'Пинг'}],
			]
		},

		{
			key:'drv_varid',
			group:{en:'Remote variable',ru:'Удаленная переменная'},
			caption:{en:'Index',ru:'Индекс'},
			control:'input',
			visibleIf:"extra_data!=metric",
			help:{
				ru:"Индекс переменной в пространстве имен шлюза",
				en:"Variable index in gateway namespace"
			}
		},
		
		{
			key:'drv_node_name',
			hide:true,
			def:'%gateway%',
		},
		
		
		
		commonDriverParams.round,
commonDriverParams.threshold,
commonDriverParams.threstime,
commonDriverParams.min,
commonDriverParams.max,
commonDriverParams.coef,
commonDriverParams.offset,
commonDriverParams.scl_hi,
commonDriverParams.scl_lo,            
commonDriverParams.raw_hi,
commonDriverParams.raw_lo,
commonDriverParams.reset_interval,
commonDriverParams.reset_value,
commonDriverParams.group_vars,
commonDriverParams.group_inv,
commonDriverParams.group_ignore,
commonDriverParams.group_block,
commonDriverParams.group_sync,
commonDriverParams.group_timeout,
commonDriverParams.group_tolerance,
		commonDriverParams.extra_service_remote_map_mbtcp__enabled,
commonDriverParams.service_remote_map_mbtcp__port,
commonDriverParams.service_remote_map_mbtcp__slave,
commonDriverParams.service_remote_map_mbtcp__reg_type,
commonDriverParams.service_remote_map_mbtcp__reg_addr,
commonDriverParams.service_remote_map_mbtcp__reg_off,
commonDriverParams.service_remote_map_mbtcp__format,

commonDriverParams.extra_service_local_map_mbtcp__enabled,
commonDriverParams.service_local_map_mbtcp__port,
commonDriverParams.service_local_map_mbtcp__slave,
commonDriverParams.service_local_map_mbtcp__reg_type,
commonDriverParams.service_local_map_mbtcp__reg_addr,
commonDriverParams.service_local_map_mbtcp__reg_off,
commonDriverParams.service_local_map_mbtcp__format,
		commonDriverParams.extra_service_local_exec__enabled,
commonDriverParams.service_local_exec__init,
commonDriverParams.service_local_exec__timeout,
commonDriverParams.service_local_exec__command,
		commonDriverParams.sim_random,
commonDriverParams.sim_random_period,
commonDriverParams.sim_random_delta,
commonDriverParams.sim_random_init,
commonDriverParams.sim_random_min,
commonDriverParams.sim_random_max,

		commonDriverParams.extra_server_comment,
	]
};
	let drv_http={
	typename: {
		ru: "Переменная HTTP",
		en: "HTTP var",
	},
	drvdesc: {
		ru: "Переменные драйвера HTTP",
		en: "HTTP driver vars",
	},
	dontApplyCommonProps: true,
	properties:[
		
		commonDriverParams.name,
		commonDriverParams.extra_server_active,
		commonDriverParams.extra_server_unit,
		commonDriverParams.trend,
		commonDriverParams.buffer,
		commonDriverParams.gateway,
		commonDriverParams.readonly,
		commonDriverParams.extra_server_tags,
		commonDriverParams.extra_server_description,

		{
			key:'drv_incoming',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Direction',ru:'Направление'},
			control:'select',
			def:0,
			drvdef:0,
			list:[
				[0,{en:'Outgoing request',ru:'Исходящий запрос'}],
				[1,{en:'Incoming request',ru:'Входящий запрос'}],
			],
			help:{
				ru:`
					В режиме исходящих запросов на удаленный хост будут выполняться HTTP-запросы чтения с параметрами URL чтения, Заголовки чтения и Тело чтения. Ответы узла будут интерпретированы согласно параметрам Источник и Формат. 
					
					При записи нового значения переменной будут выполняться HTTP-запросы с параметрами URL записи, Заголовки записи и Тело записи (если параметры заданы).

					В режиме входящих запросов драйвер будет принимать входящие HTTP-запросы на порты, указанные в настройках драйвера HTTP (сервис drv_http). Входящий запрос будет проверяться на соответствие URL параметру URL чтения. Данные запроса будут интерпретированы согласно параметрам Источник и Формат.
					
					Запись значений в таком режиме не предусмотрена.
				`,
				en:`
					In outbound request mode, HTTP read requests will be made to the remote host with the Read URL, Read Headers, and Read Body parameters. Node responses will be interpreted according to the Source and Format parameters.
						
					When writing a new variable value, HTTP requests will be made with the parameters Post URL, Post Headers, and Post Body (if parameters are specified).

					In incoming request mode, the driver will accept incoming HTTP requests on the ports specified in the HTTP driver settings (drv_http service). The incoming request will be checked to see if the URL matches the read URL parameter. The request data will be interpreted according to the Source and Format parameters.
				
					There is no provision for recording values in this mode.
				`
			}
		},
		
		change(commonDriverParams.extra_polling,{def:'period',visibleIf:'drv_incoming!=1'}),
		change(commonDriverParams.drv_period,{def:1000,visibleIf:'extra_polling!=cron;drv_incoming!=1'}),
		change(commonDriverParams.drv_cron,{visibleIf:'extra_polling=cron;drv_incoming!=1'}),

		{
			key:'drv_checkcert',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Check certificate',ru:'Проверка сертификата'},
			control:'edit',
			undef:true,
			def:0,
			drvdef:0,
			listholder:true,
			checkbox:true,
			visibleIf:'drv_incoming!=1',
			list:[
				[0,{ru:'0 - откл',en:'0 - off'}],
				[1,{ru:'1 - вкл',en:'1 - on'}],
			],
			help:{
				ru:`
					Включить проверку SSL-сертификата
				`,
				en:`
					Enable SSL certificate verification
				`
			}
		},

		{
			key:'drv_timeout',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Timeout',ru:'Таймаут'},
			control:'input',
			def:10000,
			drvdef:10000,
			help:{
				ru:"Таймаут команды, мс. Для входящих запросов: если timeout > 0, отсутствие запросов от клиента в течение времени таймаута будет приводить к генерации ошибки DRV_HTTP_ERROR_OFFLINE",
				en:"Command timeout, ms. For incoming requests: if timeout > 0, the absence of requests from the client during the timeout time will result in the generation of the DRV_HTTP_ERROR_OFFLINE error"
			}
		},

		{
			key:'drv_cache',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Cache',ru:'Кеширование'},
			control:'input',
			visibleIf:'drv_incoming!=1',
			def:50,
			drvdef:50,
			help:{
				ru:"Время кеширование ответа, мс",
				en:"Response cache time, ms"
			}
		},

		{
			key:'drv_dcmsep',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Decimal separator',ru:'Десятичный разделитель'},
			control:'select',
			def:'.',
			drvdef:'.',
			list:[
				'.',','
			]
		},

		{
			key:'drv_wrrnd',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Write rounding',ru:'Округление записи'},
			control:'input',
			help:{
				ru:`
				Округление значения при выполнении команды записи до указанного количества знаков после запятой
				`,
				en:`
				Rounds a value when executing a write command to the specified number of decimal places
				`
			}
		},

		{
			key:'drv_source',
			group:{en:'Reading',ru:'Чтение'},
			caption:{en:'Source',ru:'Источник'},
			control:'select',
			undef:true,
			list:[
				['body',{en:'Body',ru:'Тело'}],
				['header',{en:'Headers',ru:'Заголовки'}],
				['get_param',{en:'GET-param',ru:'GET-параметр'}],
				['post_param',{en:'POST-param',ru:'POST-параметр'}],
				['http_code',{en:'Response Code',ru:'Код ответа'}],
			],
			help:{
				ru:`
					Источник данных для парсинга значения переменной.
					Текст из выбранного источника будет интерпретирован согласно параметру Формат.
				`,
				en:`
					Data source for parsing the variable value.
					Text from the selected source will be interpreted according to the Format parameter.
				`
			},
		},

		{
			key:'drv_method',
			group:{en:'Reading',ru:'Чтение'},
			caption:{en:'Read method',ru:'Метод чтения'},
			control:'select',
			undef:true,
			placeholder: {en:'Auto',ru:'Авто'},
			list:[
				'GET',
				'POST',
				'PUT',
				'PATCH',
				'DELETE',
			],
			visibleIf:'drv_incoming!=1',
		},

		{
			key:'drv_url',
			group:{en:'Read URL',ru:'URL чтения'},
			control:'input',
			groupHelp:{
				ru:`
					Regex паттерн, которому должен соответствовать URL входящего запроса.
					В простейшем случае - часть URL входящего запроса, например:
					
					/data/sensor125
				`,
				en:`
				Regex pattern that the URL of the incoming request must match.
				In the simplest case, it is part of the URL of the incoming request, for example:
				
				/data/sensor125
				`
			},
		},

		{
			key:'drv_headers',
			group:{en:'Read headers',ru:'Заголовки чтения'},
			control:'edit',
			groupHelp:{
				ru:`
					HTTP-заголовки запроса чтения. 
					Список из пар Ключ:Значение разделенных переносом строки.
					Используйте, если сервер требует наличие определенных заголовков в HTTP-запросе.
					Например:
					
					x-username:admin
					x-password:12345

				`,
				en:`
					Read request HTTP-headers.
					A list of Key:Value pairs separated by line breaks.
					Use if the server requires certain headers to be present in the HTTP request.
					For example:
					
					x-username:admin
					x-password:12345
				`
			},
			visibleIf:'drv_incoming!=1',
		},

		{
			key:'drv_post',
			group:{en:'Read body',ru:'Тело чтения'},
			control:'edit',
			groupHelp:{
				ru:`
					Тело HTTP-запроса чтения.
					Актуально для запросов методом POST и тп.
				`,
				en:`
				Body of HTTP-request for read var.
				Relevant for requests using the POST method, etc.
				`
			},
			visibleIf:'drv_incoming!=1',
		},

		{
			key:'drv_write_method',
			group:{en:'Writing',ru:'Запись'},
			caption:{en:'Write method',ru:'Метод записи'},
			control:'select',
			undef:true,
			placeholder: {en:'Auto',ru:'Авто'},
			list:[
				'GET',
				'POST',
				'PUT',
				'PATCH',
				'DELETE',
			],
			visibleIf:'drv_incoming!=1',
		},
		
		{
			key:'drv_write_url',
			group:{en:'Write URL',ru:'URL записи'},
			visibleIf:'drv_incoming!=1',
			control:'input',
		},

		{
			key:'drv_write_headers',
			group:{en:'Write headers',ru:'Заголовки записи'},
			control:'edit',
			groupHelp:{
				ru:`
					HTTP-заголовки запроса записи. 
					Список из пар Ключ:Значение разделенных переносом строки.
					Используйте, если сервер требует наличие определенных заголовков в HTTP-запросе.
					Например:
					
					x-username:admin
					x-password:12345

				`,
				en:`
					Write request HTTP-headers.
					A list of Key:Value pairs separated by line breaks.
					Use if the server requires certain headers to be present in the HTTP request.
					For example:
					
					x-username:admin
					x-password:12345
				`
			},
			visibleIf:'drv_incoming!=1',
		},

		{
			key:'drv_write_post',
			group:{en:'Write body',ru:'Тело записи'},
			control:'edit',
			groupHelp:{
				ru:`
					Тело HTTP-запроса записи.
					Актуально для запросов методом POST и тп.
				`,
				en:`
				Body of HTTP-request for write var.
				Relevant for requests using the POST method, etc.
				`
			},
			visibleIf:'drv_incoming!=1',
		},

		{
			key:'extra_format',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'Format',ru:'Формат'},
			control:'select',
			def:'num',
			drvdef:'num',
			list:[
				['num',{en:'Number',ru:'Число'}],
				['json','JSON'],
				['text',{en:'Text',ru:'Текст'}],
				['regx','Regex'],
			]
		},

		{
			key:'drv_json',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'JSON-path',ru:'JSON-путь'},
			control:'input',
			visibleIf:'extra_format=json',
			help:{
				ru:`
				Путь к параметру в JSON-объекте. Для обращения свойству объекта используйте имя свойства, а для обращения к элементу массива используйте индекс.

				Например: 
				params/temperature
				sensors/102/params/temperature

				Для получения данных из элемента массива с неизвестным заранее индексом, используйте комбинацию парамтеров JSON-путь и Условие с подстановкой символа * (звездочка):

				JSON-путь: sensors/*/params/temperature
				Условие:   sensors/*/params/id:102
				`,
				en:`
				Path to the parameter in the JSON object. To access a property of an object, use the property name, and to access an array element, use the index.

				For example:
				params/temperature
				sensors/102/params/temperature

				To obtain data from an array element with an index unknown in advance, use a combination of the JSON path and Condition parameters with the * (asterisk) symbol substitution:

				JSON path: sensors/*/params/temperature
				Condition: sensors/*/params/id:102
				`
			}
		},

		{
			key:'drv_text',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'Text pattern',ru:'Текстовый паттерн'},
			control:'input',
			visibleIf:'extra_format=text',
			help:{
				ru:"Текстовый паттерн с подстановкой ключевого слова %value% там, где требуется получить числовое значение. Например: param=pressure;value=%value%",
				en:"A text pattern with the %value% keyword substituted wherever a numeric value is required. For example: param=pressure;value=%value%"
			}
		},

		{
			key:'drv_regx',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'Regex pattern',ru:'Regex паттерн'},
			control:'input',
			visibleIf:'extra_format=regx',
			help:{
				ru:"Regex паттерн. Например: param=humidity;value=([-,0-9,.]+)",
				en:"Regex pattern. For example: param=humidity;value=([-,0-9,.]+)"
			}
		},

		{
			key:'drv_match',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'Match',ru:'Условие'},
			control:'input',
			visibleIf:'extra_format=regx,text,json',
			help:{
				ru:`Строка или json-объект (в зависимости от формата), условие для считывания значения.
				
				Если свойство задано, значение будет считываться только при соответствии условия считанным данным. Данную опцию можно использовать для фильтрации сообщений, приходящих в один топик.

				- для работы с text: строка, текстовая маска с использованием символов подстановки * и ?
				Например: *sensor_id=S75*

				- для работы с regx: строка, регулярное выражение
				Например: .*sensor_id=S75.*
				
				- для работы с json: набор условий, разделенных переносом строк.
				Например:
				путь/к/значению1:значение1
				путь/к/значению2:значение2
				sensor_id:S75
				model/vendor:Alfamatika
				`,
				en:`A string or json object (depending on the format), a condition for reading the value.
				
				If the property is set, the value will only be read if the condition matches the incoming MQTT message. This option can be used to filter messages coming to one topic.

				- for working with text: string, text mask using wildcard characters * and ?
				For example: *sensor_id=S75*

				- for working with regx: string, regular expression
				For example: .*sensor_id=S75.*
			   
				- for working with json: json object, set of conditions
				{"path/to/value1":"value1","path/to/value2":"value2",...}
				For example: {"sensor_id":"S75","model/vendor":"Alfamatika"}
				`,
			}
		},

		{
			key:'drv_list',
			group:{en:'Substitution list',ru:'Список подстановки'},
			control:'edit',
			multiline:1,
			groupHelp:{
				ru:`
					Список из пар Ключ:Значение разделенных переносом строки.
					При считывании текстового значения, равного Ключу, в переменную будет записано соответствующее Ключу Значение. При выполнении команды записи перобразование будет выполнено в обратную сторону (Значение -> Ключ). В именах ключей допускается использование символов * и ?. 
					
					Например:
					
					stop:1
					start:2
					*mode*:3
					*:0
				`,
				en:`
				A list of Key:Value pairs separated by line breaks.
				When reading a text value equal to the Key, the Value corresponding to the Key will be written to the variable. When executing a write command, the conversion will be performed in the opposite direction (Value -> Key).
					
					 For example:
					
					 stop:0
					 start:1
					 auto:2
				`
			},
		},
		

		commonDriverParams.round,
commonDriverParams.threshold,
commonDriverParams.threstime,
commonDriverParams.min,
commonDriverParams.max,
commonDriverParams.coef,
commonDriverParams.offset,
commonDriverParams.scl_hi,
commonDriverParams.scl_lo,            
commonDriverParams.raw_hi,
commonDriverParams.raw_lo,
commonDriverParams.reset_interval,
commonDriverParams.reset_value,
commonDriverParams.group_vars,
commonDriverParams.group_inv,
commonDriverParams.group_ignore,
commonDriverParams.group_block,
commonDriverParams.group_sync,
commonDriverParams.group_timeout,
commonDriverParams.group_tolerance,
		commonDriverParams.extra_service_remote_map_mbtcp__enabled,
commonDriverParams.service_remote_map_mbtcp__port,
commonDriverParams.service_remote_map_mbtcp__slave,
commonDriverParams.service_remote_map_mbtcp__reg_type,
commonDriverParams.service_remote_map_mbtcp__reg_addr,
commonDriverParams.service_remote_map_mbtcp__reg_off,
commonDriverParams.service_remote_map_mbtcp__format,

commonDriverParams.extra_service_local_map_mbtcp__enabled,
commonDriverParams.service_local_map_mbtcp__port,
commonDriverParams.service_local_map_mbtcp__slave,
commonDriverParams.service_local_map_mbtcp__reg_type,
commonDriverParams.service_local_map_mbtcp__reg_addr,
commonDriverParams.service_local_map_mbtcp__reg_off,
commonDriverParams.service_local_map_mbtcp__format,
		commonDriverParams.extra_service_local_exec__enabled,
commonDriverParams.service_local_exec__init,
commonDriverParams.service_local_exec__timeout,
commonDriverParams.service_local_exec__command,
		commonDriverParams.sim_random,
commonDriverParams.sim_random_period,
commonDriverParams.sim_random_delta,
commonDriverParams.sim_random_init,
commonDriverParams.sim_random_min,
commonDriverParams.sim_random_max,

		commonDriverParams.extra_server_comment,
	]
};
	let drv_mbrtu={
	typename: {
		ru: "Переменная Modbus-RTU",
		en: "Modbus-RTU var",
	},
	drvdesc: {
		ru: "Переменные драйвера Modbus-RTU",
		en: "Modbus-RTU driver vars",
	},
	dontApplyCommonProps: true,
	clipboardType:'prototype_mb',
	properties:[
		
		commonDriverParams.name,
		commonDriverParams.extra_server_active,
		commonDriverParams.extra_server_unit,
		commonDriverParams.trend,
		commonDriverParams.buffer,
		commonDriverParams.gateway,
		commonDriverParams.readonly,
		commonDriverParams.extra_server_tags,
		commonDriverParams.extra_server_description,
		commonDriverParams.extra_polling,
		change(commonDriverParams.drv_period,{def:1000}),
		commonDriverParams.drv_cron,

		{
			key:'extra_data',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Data',ru:'Данные'},
			control:'select',
			def:'register',
			undef:true,
			list:[
				['register',{en:'Register',ru:'Регистр'}],
				['metric',{en:'Metric',ru:'Метрика'}],
			],
			help:{
				ru:`<b>Регистр</b>: в переменную будет считываться значение указанного регистра
				
				<b>Метрика</b>: в переменную будет считываться статистика работы узла
				`,
				en:`<b>Register</b>: the variable will read the value of the specified register
				<b>Metric</b>: the variable will read the statistics of the node
				`
			}
		},
		
		{
			key:'drv_metric',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Metric',ru:'Метрика'},
			control:'select',
			visibleIf:"extra_data=metric",
			list:[
				['requests',{en:'Requests',ru:'Запросы'}],
				['errors',{en:'Errors',ru:'Ошибки'}],
				['vars',{en:'Vars',ru:'Переменные'}],
				['registers',{en:'Registers',ru:'Регистры'}],
				['quality',{en:'Quality',ru:'Качество'}],
				['art','ART'],
				['rps','RPS'],
				['eps','EPS'],
				['varps','VARPS'],
				['regps','REGPS'],
				['up','UP'],
				['down','DOWN'],
				['total_requests',{en:'Total Requests',ru:'Все запросы'}],
				['total_errors',{en:'Total Errors',ru:'Все ошибки'}],
				['total_vars',{en:'Total Vars',ru:'Все переменные'}],
				['total_registers',{en:'Total Registers',ru:'Все регистры'}],
				['total_quality',{en:'Total Quality',ru:'Все качество'}],
				['total_art','Total ART'],
				['total_rps','Total RPS'],
				['total_eps','Total EPS'],
				['total_varps','Total VARPS'],
				['total_regps','Total REGPS'],
				['total_queue','Total Queue'],
			],
			help:{
				ru:`Работа переменной в режиме "Метрика" позволяет получать статистику о работе соединения. Все данные считаются за период опроса и для одного TCP-соединения, определенного параметрами Хост:Порт.

					<b>Запросы</b>: количество отправленных запросов за период опроса
					
					<b>Ошибки</b>: количество полученных ответов с ошибками  за период опроса
					
					<b>Переменные</b>: количество опрошенных переменных за период опроса
					
					<b>Регистры</b>: количество опрошенных регистров за период опроса
					
					<b>Качество</b>: отношение успешных запросов к общему количеству запросов; 0 - ни одного успешного ответа, 1 - ни одной ошибки

					<b>ART</b>: Среднее время ответа, мс

					<b>RPS</b>: Количество запросов в секунду

					<b>EPS</b>: Количество ошибок в секунду

					<b>VARPS</b>: Количество опрашиваемых переменных в секунду

					<b>REGPS</b>: Количество опрашиваемых регистров в секунду

					<b>UP</b>: Узел отвечает (как мин. один ответ без ошибки)

					<b>DOWN</b>: Узел "лежит" (все ответы с ошибками)

				`,
				en:`The operation of a variable in the "Metric" mode allows you to get statistics about the the connection. All data is counted for the polling period and for one TCP connection defined by the Host:Port parameters.

				<b>Requests</b>: number of requests sent during the polling period
			   
				<b>Errors</b>: number of responses received with errors during the polling period
			   
				<b>Variables</b>: number of polled variables during the polling period
			   
				<b>Registers</b>: number of polled registers during the polling period
			   
				<b>Quality</b>: the ratio of successful requests to the total number of requests; 0 - no successful response, 1 - no errors

				<b>ART</b>: Average response time, ms

				<b>RPS</b>: Number of requests per second

				<b>EPS</b>: Number of errors per second

				<b>VARPS</b>: Number of variables polled per second

				<b>REGPS</b>: Number of registers polled per second

				<b>UP</b>: Node is responding (at least one response without error)
				
				<b>DOWN</b>: Node is not responding (all response are errors)
				`
			}
		},

		{
			key:'drv_reg_type',
			group:{en:'Register',ru:'Регистр'},
			caption:{en:'Type',ru:'Тип'},
			control:'edit',
			undef:true,
			list:['CL','DI','HR','IR'],
			def:'HR',
			drvdef:'HR',
			visibleIf:"extra_data=register",
			help:`
					CL: Coil
					DI: Digital Input
					HR: Holding Register
					IR: Input Register
			`
		},

		{
			key:'drv_reg_addr',
			group:{en:'Register',ru:'Регистр'},
			caption:{en:'Address',ru:'Адрес'},
			def:1,
			drvdef:1,
			visibleIf:"extra_data=register",
			control:'input',
			help:{
				ru:`
					Адрес регистра, число 0 - 65535.
					Физический адрес регистра = Адрес + Смещение
				`,
				en:`
					Register Address, number 0 - 65535.
					Register Physical Address = Address + Offset
				`
			}
		},

		{
			key:'drv_format',
			group:{en:'Register',ru:'Регистр'},
			caption:{en:'Format',ru:'Формат'},
			visibleIf:"extra_data=register;drv_reg_type=IR,HR",
			control:'edit',
			undef:true,
			def:'INT16',
			drvdef:'INT16',
			list:[
				'INT16',
				'UINT16',
				'FLOAT_ABCD',
				'FLOAT_CDAB',
				'FLOAT_BADC',
				'FLOAT_DCBA',
				'INT32_ABCD',
				'INT32_CDAB',
				'INT32_BADC',
				'INT32_DCBA',
				'UINT32_ABCD',
				'UINT32_CDAB',
				'UINT32_BADC',
				'UINT32_DCBA',
				'BIT_0',
				'BIT_1',
				'BIT_2',
				'BIT_3',
				'BIT_4',
				'BIT_5',
				'BIT_6',
				'BIT_7',
				'BIT_8',
				'BIT_9',
				'BIT_10',
				'BIT_11',
				'BIT_12',
				'BIT_13',
				'BIT_14',
				'BIT_15',
			],		
		},

		{
			key:'drv_hipri',
			group:{en:'Register',ru:'Регистр'},
			caption:'HI-PRI',
			visibleIf:"extra_data=register",
			control:'edit',
			listholder:true,
			checkbox:true,
			undef:true,
			def:0,
			drvdef:0,
			list:[
				[0,{ru:'0 - откл',en:'0 - off'}],
				[1,{ru:'1 - вкл',en:'1 - on'}],
			],
			help:{
				ru:`Высокий приоритет опроса. Запрос добавляется в начало очереди, а не в конец, как у обычных переменных. 
				
				При большом количестве HI-PRI переменных может возникнуть ситуация, когда обычные переменные не будут успевать опрашиваться, т.к. очередь будет всегда заполняться HI-PRI переменными.`,
				en:`High polling priority. The request is added to the beginning of the queue, not to the end, as with ordinary variables. 
				
				If there are a large number of HI-PRI variables, a situation may arise when ordinary variables will not be polled in time, because the queue will always be filled with HI-PRI variables.`
			}
		}, 

		{
			key:'drv_group_id',
			group:{en:'Register',ru:'Регистр'},
			caption:'GroupId',
			visibleIf:"extra_data=register",
			control:'input',
			help:{
				ru:"Число 0 - 65535, см. Группировка в разделе Узел",
				en:"Number 0 - 65535, see the Grouping option in the Node section"
			}
		},

		{
			key:'drv_node_slave',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Slave',ru:'Slave'},
			control:'input',
			def:1,
			drvdef:1,
			help:{
				ru:"Modbus Slave ID, 1 - 247",
				en:"Modbus Slave ID, 1 - 247"
			}
		},

		{
			key:'drv_node_port',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Port',ru:'Порт'},
			control:'input',
			def:'/dev/ttyUSB0',
			help:{
				ru:`
					Последовательный порт, например: /dev/ttyUSB0
					Список доступных портов на устройстве можно посмотреть командой	<b>ls /dev</b>
				`,
				en:`
					Serial port, for example: /dev/ttyUSB0
					The list of available ports on the device can be viewed with the command <b>ls /dev</b>
				`
			}
		},

		{
			key:'drv_node_speed',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Speed',ru:'Скорость'},
			control:'edit',
			undef:true,
			def:115200,
			drvdef:115200,
			list:[
				110, 300, 600, 1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200, 230400, 460800, 500000, 576000, 921600, 1000000, 1152000, 1500000, 2500000, 3000000, 3500000, 4000000
			],
			help:{
				ru:"Скорость передачи, бод", 
				en:"Transfer speed, baud"
			}
		},

		{
			key:'drv_node_parity',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Parity',ru:'Четность'},
			control:'edit',
			undef:true,
			listholder:true,
			def:0,
			drvdef:0,
			list:[[0,{ru:'0 - None / Нет',en:'0 - None'}],[1,{ru:'1 - Odd / Нечетное',en:'1 - Odd'}],[2,{ru:'2 - Even / Четное',en:'2 - Even'}]],
		},

		{
			key:'drv_node_stopbits',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Stopbits',ru:'Стоп-биты'},
			control:'edit',
			undef:true,
			listholder:true,
			def:1,
			drvdef:2,
			list:[1,2],
		},

		{
			key:'drv_node_flow_control',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Flow control',ru:'Управление потоком'},
			control:'edit',
			undef:true,
			listholder:true,
			checkbox:true,
			def:0,
			drvdef:0,
			list:[
				[0,{ru:'0 - Нет',en:'0 - None'}],
				[1,'1 - RTS'],
				[2,'2 - XON_XOFF'],
			],
			help:{
				ru:`
					Аппаратное управление потоком RTS/CTS
				`,
				en:`
					Hardware RTS/CTS flow control
				`
			}
		},

		{
			key:'drv_node_offset',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Addr. offset',ru:'Смещение адресов'},
			control:'input',
			def:0,
			drvdef:0,
			help:{
				ru:`
					Смещение адресов регистров.
					Физический адрес регистра = Адрес + Смещение
				`,
				en:`
					Offset of register addresses.
					Register Physical Address = Address + Offset
				`
			}
		},

		{
			key:'drv_node_timeout',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Timeout',ru:'Таймаут'},
			control:'input',
			def:100,
			drvdef:100,
			help:{
				ru:"Таймаут запроса, мс",
				en:"Request timeout, ms"
			}
		},
		
		{
			key:'drv_node_delay',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Delay',ru:'Задержка'},
			control:'input',
			def:0,
			drvdef:0,
			help:{
				ru:"Задержка между запросами, мс (параметр async должен быть отключен)",
				en:"Delay between requests, ms (async parameter must be disabled)"
			}
		},

		{
			key:'drv_node_group',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Grouping',ru:'Группировка'},
			control:'edit',
			undef:true,
			listholder:true,
			checkbox:true,
			def:1,
			drvdef:1,
			list:[
				[0,{ru:'0 - не группировать',en:'do not group'}],
				[1,{ru:'1 - группировать без пробелов',en:'group w/o spaces'}],
				[2,{ru:'2 - группировать с пробелами',en:'group with spaces'}],
				[3,{ru:'3 - группировать по переменным',en:'group by variables'}],
				[4,{ru:'4 - группировать по GroupId',en:'group by GroupId'}],
			],
			help:{
				ru:`
					Группировка регистров в запросе:
					0 - не группировать (отдельный запрос на каждый регистр)
					1 - группировать без пробелов (отдельный запрос на группу регистров с последовательными адресами, без пробелов)
					2 - группировать с пробелами (запрос составляется из максимального количества регистров)
					3 - группировать по переменным (регистры каждой переменной запрашиваются отдельным запросом)
					4 - группировать по GroupId (регистры переменных с одинаковым GroupId запрашиваются отдельным запросом)
				`,
				en:`
					Grouping registers in a request:
					0 - do not group (one request for each register)
					1 - group without spaces (one request for a group of registers with sequential addresses, without spaces)
					2 - group with spaces (the request is composed of the maximum number of registers)
					3 - group by variables (registers of each variable are requested in a separate request)
					4 - group by GroupId (variable registers with the same GroupId are requested in a separate request)
				`
			}
		},

		{
			key:'drv_node_ignore_errors',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Ignore Errors',ru:'Игнорировать ошибки'},
			control:'input',
			def:0,
			drvdef:0,
			help:{
				ru:`Количество идущих подряд ошибок чтения, которые будут проигнорированы, 0 - 15. 0 - не игнорировать ошибки`,
				en:`Number of consecutive read errors that will be ignored, 0 - 15. 0 - do not ignore errors`
			}
		},

		{
			key:'drv_node_writecmd',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Write HR function',ru:'Функция записи HR'},
			control:'edit',
			list:[16,6],
			def:16,
			drvdef:16,
			help:{
				ru:`Код функции записи для Holding Register: 16 или 6`,
				en:`Write function code for Holding Register: 16 or 6`
			}
		},

		commonDriverParams.retry_count,
commonDriverParams.retry_interval,
		commonDriverParams.round,
commonDriverParams.threshold,
commonDriverParams.threstime,
commonDriverParams.min,
commonDriverParams.max,
commonDriverParams.coef,
commonDriverParams.offset,
commonDriverParams.scl_hi,
commonDriverParams.scl_lo,            
commonDriverParams.raw_hi,
commonDriverParams.raw_lo,
commonDriverParams.reset_interval,
commonDriverParams.reset_value,
commonDriverParams.group_vars,
commonDriverParams.group_inv,
commonDriverParams.group_ignore,
commonDriverParams.group_block,
commonDriverParams.group_sync,
commonDriverParams.group_timeout,
commonDriverParams.group_tolerance,
		commonDriverParams.extra_service_remote_map_mbtcp__enabled,
commonDriverParams.service_remote_map_mbtcp__port,
commonDriverParams.service_remote_map_mbtcp__slave,
commonDriverParams.service_remote_map_mbtcp__reg_type,
commonDriverParams.service_remote_map_mbtcp__reg_addr,
commonDriverParams.service_remote_map_mbtcp__reg_off,
commonDriverParams.service_remote_map_mbtcp__format,

commonDriverParams.extra_service_local_map_mbtcp__enabled,
commonDriverParams.service_local_map_mbtcp__port,
commonDriverParams.service_local_map_mbtcp__slave,
commonDriverParams.service_local_map_mbtcp__reg_type,
commonDriverParams.service_local_map_mbtcp__reg_addr,
commonDriverParams.service_local_map_mbtcp__reg_off,
commonDriverParams.service_local_map_mbtcp__format,
		commonDriverParams.extra_service_local_exec__enabled,
commonDriverParams.service_local_exec__init,
commonDriverParams.service_local_exec__timeout,
commonDriverParams.service_local_exec__command,
		commonDriverParams.sim_random,
commonDriverParams.sim_random_period,
commonDriverParams.sim_random_delta,
commonDriverParams.sim_random_init,
commonDriverParams.sim_random_min,
commonDriverParams.sim_random_max,

		commonDriverParams.extra_server_comment,
	]
};
	let drv_mbtcp={
	typename: {
		ru: "Переменная Modbus-TCP",
		en: "Modbus-TCP var",
	},
	drvdesc: {
		ru: "Переменные драйвера Modbus-TCP",
		en: "Modbus-TCP driver vars",
	},
	dontApplyCommonProps: true,
	clipboardType:'prototype_mb',
	properties:[
		
		commonDriverParams.name,
		commonDriverParams.extra_server_active,
		commonDriverParams.extra_server_unit,
		commonDriverParams.trend,
		commonDriverParams.buffer,
		commonDriverParams.gateway,
		commonDriverParams.readonly,
		commonDriverParams.extra_server_tags,
		commonDriverParams.extra_server_description,
		commonDriverParams.extra_polling,
		change(commonDriverParams.drv_period,{def:1000}),
		commonDriverParams.drv_cron,

		{
			key:'extra_data',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Data',ru:'Данные'},
			control:'select',
			def:'register',
			undef:true,
			list:[
				['register',{en:'Register',ru:'Регистр'}],
				['metric',{en:'Metric',ru:'Метрика'}],
			],
			help:{
				ru:`<b>Регистр</b>: в переменную будет считываться значение указанного регистра
				
				<b>Метрика</b>: в переменную будет считываться статистика работы узла
				`,
				en:`<b>Register</b>: the variable will read the value of the specified register
				<b>Metric</b>: the variable will read the statistics of the node
				`
			}
		},
		
		{
			key:'drv_metric',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Metric',ru:'Метрика'},
			control:'select',
			visibleIf:"extra_data=metric",
			list:[
				['requests',{en:'Requests',ru:'Запросы'}],
				['errors',{en:'Errors',ru:'Ошибки'}],
				['vars',{en:'Vars',ru:'Переменные'}],
				['registers',{en:'Registers',ru:'Регистры'}],
				['quality',{en:'Quality',ru:'Качество'}],
				['art','ART'],
				['rps','RPS'],
				['eps','EPS'],
				['varps','VARPS'],
				['regps','REGPS'],
				['up','UP'],
				['down','DOWN'],
				['total_requests',{en:'Total Requests',ru:'Все запросы'}],
				['total_errors',{en:'Total Errors',ru:'Все ошибки'}],
				['total_vars',{en:'Total Vars',ru:'Все переменные'}],
				['total_registers',{en:'Total Registers',ru:'Все регистры'}],
				['total_quality',{en:'Total Quality',ru:'Все качество'}],
				['total_art','Total ART'],
				['total_rps','Total RPS'],
				['total_eps','Total EPS'],
				['total_varps','Total VARPS'],
				['total_regps','Total REGPS'],
				['total_queue','Total Queue'],
			],
			help:{
				ru:`Работа переменной в режиме "Метрика" позволяет получать статистику о работе соединения. Все данные считаются за период опроса и для одного TCP-соединения, определенного параметрами Хост:Порт.

					<b>Запросы</b>: количество отправленных запросов за период опроса
					
					<b>Ошибки</b>: количество полученных ответов с ошибками  за период опроса
					
					<b>Переменные</b>: количество опрошенных переменных за период опроса
					
					<b>Регистры</b>: количество опрошенных регистров за период опроса
					
					<b>Качество</b>: отношение успешных запросов к общему количеству запросов; 0 - ни одного успешного ответа, 1 - ни одной ошибки

					<b>ART</b>: Среднее время ответа, мс

					<b>RPS</b>: Количество запросов в секунду

					<b>EPS</b>: Количество ошибок в секунду

					<b>VARPS</b>: Количество опрашиваемых переменных в секунду

					<b>REGPS</b>: Количество опрашиваемых регистров в секунду

					<b>UP</b>: Узел отвечает (как мин. один ответ без ошибки)

					<b>DOWN</b>: Узел "лежит" (все ответы с ошибками)

				`,
				en:`The operation of a variable in the "Metric" mode allows you to get statistics about the the connection. All data is counted for the polling period and for one TCP connection defined by the Host:Port parameters.

				<b>Requests</b>: number of requests sent during the polling period
			   
				<b>Errors</b>: number of responses received with errors during the polling period
			   
				<b>Variables</b>: number of polled variables during the polling period
			   
				<b>Registers</b>: number of polled registers during the polling period
			   
				<b>Quality</b>: the ratio of successful requests to the total number of requests; 0 - no successful response, 1 - no errors

				<b>ART</b>: Average response time, ms

				<b>RPS</b>: Number of requests per second

				<b>EPS</b>: Number of errors per second

				<b>VARPS</b>: Number of variables polled per second

				<b>REGPS</b>: Number of registers polled per second

				<b>UP</b>: Node is responding (at least one response without error)
				
				<b>DOWN</b>: Node is not responding (all response are errors)
				`
			}
		},

		{
			key:'drv_reg_type',
			group:{en:'Register',ru:'Регистр'},
			caption:{en:'Type',ru:'Тип'},
			control:'edit',
			undef:true,
			list:['CL','DI','HR','IR'],
			def:'HR',
			drvdef:'HR',
			visibleIf:"extra_data=register",
			help:`
					CL: Coil
					DI: Digital Input
					HR: Holding Register
					IR: Input Register
			`
		},

		{
			key:'drv_reg_addr',
			group:{en:'Register',ru:'Регистр'},
			caption:{en:'Address',ru:'Адрес'},
			def:1,
			drvdef:1,
			visibleIf:"extra_data=register",
			control:'input',
			help:{
				ru:`
					Адрес регистра, число 0 - 65535.
					Физический адрес регистра = Адрес + Смещение
				`,
				en:`
					Register Address, number 0 - 65535.
					Register Physical Address = Address + Offset
				`
			}
		},

		{
			key:'drv_format',
			group:{en:'Register',ru:'Регистр'},
			caption:{en:'Format',ru:'Формат'},
			visibleIf:"extra_data=register;drv_reg_type=IR,HR",
			control:'edit',
			undef:true,
			def:'INT16',
			drvdef:'INT16',
			list:[
				'INT16',
				'UINT16',
				'FLOAT_ABCD',
				'FLOAT_CDAB',
				'FLOAT_BADC',
				'FLOAT_DCBA',
				'INT32_ABCD',
				'INT32_CDAB',
				'INT32_BADC',
				'INT32_DCBA',
				'UINT32_ABCD',
				'UINT32_CDAB',
				'UINT32_BADC',
				'UINT32_DCBA',
				'BIT_0',
				'BIT_1',
				'BIT_2',
				'BIT_3',
				'BIT_4',
				'BIT_5',
				'BIT_6',
				'BIT_7',
				'BIT_8',
				'BIT_9',
				'BIT_10',
				'BIT_11',
				'BIT_12',
				'BIT_13',
				'BIT_14',
				'BIT_15',
			],		
		},

		{
			key:'drv_hipri',
			group:{en:'Register',ru:'Регистр'},
			caption:'HI-PRI',
			visibleIf:"extra_data=register",
			control:'edit',
			listholder:true,
			checkbox:true,
			undef:true,
			def:0,
			drvdef:0,
			list:[
				[0,{ru:'0 - откл',en:'0 - off'}],
				[1,{ru:'1 - вкл',en:'1 - on'}],
			],
			help:{
				ru:`Высокий приоритет опроса. Запрос добавляется в начало очереди, а не в конец, как у обычных переменных. 
				
				При большом количестве HI-PRI переменных может возникнуть ситуация, когда обычные переменные не будут успевать опрашиваться, т.к. очередь будет всегда заполняться HI-PRI переменными.`,
				en:`High polling priority. The request is added to the beginning of the queue, not to the end, as with ordinary variables. 
				
				If there are a large number of HI-PRI variables, a situation may arise when ordinary variables will not be polled in time, because the queue will always be filled with HI-PRI variables.`
			}
		}, 

		{
			key:'drv_group_id',
			group:{en:'Register',ru:'Регистр'},
			caption:'GroupId',
			visibleIf:"extra_data=register",
			control:'input',
			help:{
				ru:"Число 0 - 65535, см. Группировка в разделе Узел",
				en:"Number 0 - 65535, see the Grouping option in the Node section"
			}
		},

		{
			key:'drv_node_host',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Host',ru:'Хост'},
			control:'input',
			def:'127.0.0.1',
			drvdef:'127.0.0.1',
			help:{
				ru:"IP-адрес или доменное имя",
				en:"IP-address or domain name"
			}
		},
		
		{
			key:'drv_node_port',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Port',ru:'Порт'},
			control:'input',
			def:502,
			drvdef:502,
			help:{
				ru:"Номер Modbus-порта",
				en:"Modbus-port number"
			}
		},

		{
			key:'drv_node_slave',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Slave',ru:'Slave'},
			control:'input',
			def:1,
			drvdef:1,
			help:{
				ru:"Modbus Slave ID, 1 - 247",
				en:"Modbus Slave ID, 1 - 247"
			}
		},

		{
			key:'drv_node_offset',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Addr. offset',ru:'Смещение адресов'},
			control:'input',
			def:0,
			drvdef:0,
			help:{
				ru:`
					Смещение адресов регистров.
					Физический адрес регистра = Адрес + Смещение
				`,
				en:`
					Offset of register addresses.
					Register Physical Address = Address + Offset
				`
			}
		},

		{
			key:'drv_node_timeout',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Timeout',ru:'Таймаут'},
			control:'input',
			def:100,
			drvdef:100,
			help:{
				ru:"Таймаут запроса, мс",
				en:"Request timeout, ms"
			}
		},

		{
			key:'drv_node_delay',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Delay',ru:'Задержка'},
			control:'input',
			def:0,
			drvdef:0,
			help:{
				ru:"Задержка между запросами, мс (параметр async должен быть отключен)",
				en:"Delay between requests, ms (async parameter must be disabled)"
			}
		},

		{
			key:'drv_node_keepalive',
			group:{en:'Node',ru:'Узел'},
			caption:'Keep-Alive',
			control:'edit',
			undef:true,
			listholder:true,
			checkbox:true,
			def:1,
			drvdef:1,
			list:[
				[0,{ru:'0 - откл',en:'0 - off'}],
				[1,{ru:'1 - вкл',en:'1 - on'}],
			],
			help:{
				ru:"Держать соединение открытым (не закрывать соединение каждого запроса)",
				en:"Don't close the connection after each request"
			}
		}, 

		{
			key:'drv_node_sharecon',
			group:{en:'Node',ru:'Узел'},
			caption:'Share-Con',
			control:'edit',
			undef:true,
			listholder:true,
			checkbox:true,
			def:1,
			drvdef:1,
			list:[
				[0,{ru:'0 - откл',en:'0 - off'}],
				[1,{ru:'1 - вкл',en:'1 - on'}],
			],
			help:{
				ru:"Использовать по возможности одно TCP-соединение для нескольких узлов",
				en:"Use one TCP-connection for several nodes if possible"
			}
		},

		{
			key:'drv_node_async',
			group:{en:'Node',ru:'Узел'},
			caption:'Async',
			control:'edit',
			undef:true,
			listholder:true,
			checkbox:true,
			def:0,
			drvdef:0,
			list:[
				[0,{ru:'0 - откл',en:'0 - off'}],
				[1,{ru:'1 - вкл',en:'1 - on'}],
			],
			help:{
				ru:"Разрешить параллельные асинхронные запросы (система будет отправлять новые запросы не дожидаясь ответов на предыдущие)",
				en:"Allow concurrent asynchronous requests (the system will send new requests without waiting for responses to previous ones)"
			}
		},

		{
			key:'drv_node_group',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Grouping',ru:'Группировка'},
			control:'edit',
			undef:true,
			listholder:true,
			checkbox:true,
			def:1,
			drvdef:1,
			list:[
				[0,{ru:'0 - не группировать',en:'do not group'}],
				[1,{ru:'1 - группировать без пробелов',en:'group w/o spaces'}],
				[2,{ru:'2 - группировать с пробелами',en:'group with spaces'}],
				[3,{ru:'3 - группировать по переменным',en:'group by variables'}],
				[4,{ru:'4 - группировать по GroupId',en:'group by GroupId'}],
			],
			help:{
				ru:`
					Группировка регистров в запросе:
					0 - не группировать (отдельный запрос на каждый регистр)
					1 - группировать без пробелов (отдельный запрос на группу регистров с последовательными адресами, без пробелов)
					2 - группировать с пробелами (запрос составляется из максимального количества регистров)
					3 - группировать по переменным (регистры каждой переменной запрашиваются отдельным запросом)
					4 - группировать по GroupId (регистры переменных с одинаковым GroupId запрашиваются отдельным запросом)
				`,
				en:`
					Grouping registers in a request:
					0 - do not group (one request for each register)
					1 - group without spaces (one request for a group of registers with sequential addresses, without spaces)
					2 - group with spaces (the request is composed of the maximum number of registers)
					3 - group by variables (registers of each variable are requested in a separate request)
					4 - group by GroupId (variable registers with the same GroupId are requested in a separate request)
				`
			}
		},

		{
			key:'drv_node_ignore_errors',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Ignore Errors',ru:'Игнорировать ошибки'},
			control:'input',
			def:0,
			drvdef:0,
			help:{
				ru:`Количество идущих подряд ошибок чтения, которые будут проигнорированы, 0 - 15. 0 - не игнорировать ошибки`,
				en:`Number of consecutive read errors that will be ignored, 0 - 15. 0 - do not ignore errors`
			}
		},

		{
			key:'drv_node_writecmd',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Write HR function',ru:'Функция записи HR'},
			control:'edit',
			list:[16,6],
			def:16,
			drvdef:16,
			help:{
				ru:`Код функции записи для Holding Register: 16 или 6`,
				en:`Write function code for Holding Register: 16 or 6`
			}
		},



		commonDriverParams.retry_count,
commonDriverParams.retry_interval,
		commonDriverParams.round,
commonDriverParams.threshold,
commonDriverParams.threstime,
commonDriverParams.min,
commonDriverParams.max,
commonDriverParams.coef,
commonDriverParams.offset,
commonDriverParams.scl_hi,
commonDriverParams.scl_lo,            
commonDriverParams.raw_hi,
commonDriverParams.raw_lo,
commonDriverParams.reset_interval,
commonDriverParams.reset_value,
commonDriverParams.group_vars,
commonDriverParams.group_inv,
commonDriverParams.group_ignore,
commonDriverParams.group_block,
commonDriverParams.group_sync,
commonDriverParams.group_timeout,
commonDriverParams.group_tolerance,
		commonDriverParams.extra_service_remote_map_mbtcp__enabled,
commonDriverParams.service_remote_map_mbtcp__port,
commonDriverParams.service_remote_map_mbtcp__slave,
commonDriverParams.service_remote_map_mbtcp__reg_type,
commonDriverParams.service_remote_map_mbtcp__reg_addr,
commonDriverParams.service_remote_map_mbtcp__reg_off,
commonDriverParams.service_remote_map_mbtcp__format,

commonDriverParams.extra_service_local_map_mbtcp__enabled,
commonDriverParams.service_local_map_mbtcp__port,
commonDriverParams.service_local_map_mbtcp__slave,
commonDriverParams.service_local_map_mbtcp__reg_type,
commonDriverParams.service_local_map_mbtcp__reg_addr,
commonDriverParams.service_local_map_mbtcp__reg_off,
commonDriverParams.service_local_map_mbtcp__format,
		commonDriverParams.extra_service_local_exec__enabled,
commonDriverParams.service_local_exec__init,
commonDriverParams.service_local_exec__timeout,
commonDriverParams.service_local_exec__command,
		commonDriverParams.sim_random,
commonDriverParams.sim_random_period,
commonDriverParams.sim_random_delta,
commonDriverParams.sim_random_init,
commonDriverParams.sim_random_min,
commonDriverParams.sim_random_max,

		commonDriverParams.extra_server_comment,
	]
};
	let drv_mbrot={
	typename: {
		ru: "Переменная Modbus-RTU-Over-TCP",
		en: "Modbus-RTU-Over-TCP var",
	},
	drvdesc: {
		ru: "Переменные драйвера Modbus-RTU-Over-TCP",
		en: "Modbus-RTU-Over-TCP driver vars",
	},
	dontApplyCommonProps: true,
	clipboardType:'prototype_mb',
	properties:[
		
		commonDriverParams.name,
		commonDriverParams.extra_server_active,
		commonDriverParams.extra_server_unit,
		commonDriverParams.trend,
		commonDriverParams.buffer,
		commonDriverParams.gateway,
		commonDriverParams.readonly,
		commonDriverParams.extra_server_tags,
		commonDriverParams.extra_server_description,
		commonDriverParams.extra_polling,
		change(commonDriverParams.drv_period,{def:1000}),
		commonDriverParams.drv_cron,

		{
			key:'extra_data',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Data',ru:'Данные'},
			control:'select',
			def:'register',
			undef:true,
			list:[
				['register',{en:'Register',ru:'Регистр'}],
				['metric',{en:'Metric',ru:'Метрика'}],
			],
			help:{
				ru:`<b>Регистр</b>: в переменную будет считываться значение указанного регистра
				
				<b>Метрика</b>: в переменную будет считываться статистика работы узла
				`,
				en:`<b>Register</b>: the variable will read the value of the specified register
				<b>Metric</b>: the variable will read the statistics of the node
				`
			}
		},
		
		{
			key:'drv_metric',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Metric',ru:'Метрика'},
			control:'select',
			visibleIf:"extra_data=metric",
			list:[
				['requests',{en:'Requests',ru:'Запросы'}],
				['errors',{en:'Errors',ru:'Ошибки'}],
				['vars',{en:'Vars',ru:'Переменные'}],
				['registers',{en:'Registers',ru:'Регистры'}],
				['quality',{en:'Quality',ru:'Качество'}],
				['art','ART'],
				['rps','RPS'],
				['eps','EPS'],
				['varps','VARPS'],
				['regps','REGPS'],
				['up','UP'],
				['down','DOWN'],
				['total_requests',{en:'Total Requests',ru:'Все запросы'}],
				['total_errors',{en:'Total Errors',ru:'Все ошибки'}],
				['total_vars',{en:'Total Vars',ru:'Все переменные'}],
				['total_registers',{en:'Total Registers',ru:'Все регистры'}],
				['total_quality',{en:'Total Quality',ru:'Все качество'}],
				['total_art','Total ART'],
				['total_rps','Total RPS'],
				['total_eps','Total EPS'],
				['total_varps','Total VARPS'],
				['total_regps','Total REGPS'],
				['total_queue','Total Queue'],
			],
			help:{
				ru:`Работа переменной в режиме "Метрика" позволяет получать статистику о работе соединения. Все данные считаются за период опроса и для одного TCP-соединения, определенного параметрами Хост:Порт.

					<b>Запросы</b>: количество отправленных запросов за период опроса
					
					<b>Ошибки</b>: количество полученных ответов с ошибками  за период опроса
					
					<b>Переменные</b>: количество опрошенных переменных за период опроса
					
					<b>Регистры</b>: количество опрошенных регистров за период опроса
					
					<b>Качество</b>: отношение успешных запросов к общему количеству запросов; 0 - ни одного успешного ответа, 1 - ни одной ошибки

					<b>ART</b>: Среднее время ответа, мс

					<b>RPS</b>: Количество запросов в секунду

					<b>EPS</b>: Количество ошибок в секунду

					<b>VARPS</b>: Количество опрашиваемых переменных в секунду

					<b>REGPS</b>: Количество опрашиваемых регистров в секунду

					<b>UP</b>: Узел отвечает (как мин. один ответ без ошибки)

					<b>DOWN</b>: Узел "лежит" (все ответы с ошибками)

				`,
				en:`The operation of a variable in the "Metric" mode allows you to get statistics about the the connection. All data is counted for the polling period and for one TCP connection defined by the Host:Port parameters.

				<b>Requests</b>: number of requests sent during the polling period
			   
				<b>Errors</b>: number of responses received with errors during the polling period
			   
				<b>Variables</b>: number of polled variables during the polling period
			   
				<b>Registers</b>: number of polled registers during the polling period
			   
				<b>Quality</b>: the ratio of successful requests to the total number of requests; 0 - no successful response, 1 - no errors

				<b>ART</b>: Average response time, ms

				<b>RPS</b>: Number of requests per second

				<b>EPS</b>: Number of errors per second

				<b>VARPS</b>: Number of variables polled per second

				<b>REGPS</b>: Number of registers polled per second

				<b>UP</b>: Node is responding (at least one response without error)
				
				<b>DOWN</b>: Node is not responding (all response are errors)
				`
			}
		},

		{
			key:'drv_reg_type',
			group:{en:'Register',ru:'Регистр'},
			caption:{en:'Type',ru:'Тип'},
			control:'edit',
			undef:true,
			list:['CL','DI','HR','IR'],
			def:'HR',
			drvdef:'HR',
			visibleIf:"extra_data=register",
			help:`
					CL: Coil
					DI: Digital Input
					HR: Holding Register
					IR: Input Register
			`
		},

		{
			key:'drv_reg_addr',
			group:{en:'Register',ru:'Регистр'},
			caption:{en:'Address',ru:'Адрес'},
			def:1,
			drvdef:1,
			visibleIf:"extra_data=register",
			control:'input',
			help:{
				ru:`
					Адрес регистра, число 0 - 65535.
					Физический адрес регистра = Адрес + Смещение
				`,
				en:`
					Register Address, number 0 - 65535.
					Register Physical Address = Address + Offset
				`
			}
		},

		{
			key:'drv_format',
			group:{en:'Register',ru:'Регистр'},
			caption:{en:'Format',ru:'Формат'},
			visibleIf:"extra_data=register;drv_reg_type=IR,HR",
			control:'edit',
			undef:true,
			def:'INT16',
			drvdef:'INT16',
			list:[
				'INT16',
				'UINT16',
				'FLOAT_ABCD',
				'FLOAT_CDAB',
				'FLOAT_BADC',
				'FLOAT_DCBA',
				'INT32_ABCD',
				'INT32_CDAB',
				'INT32_BADC',
				'INT32_DCBA',
				'UINT32_ABCD',
				'UINT32_CDAB',
				'UINT32_BADC',
				'UINT32_DCBA',
				'BIT_0',
				'BIT_1',
				'BIT_2',
				'BIT_3',
				'BIT_4',
				'BIT_5',
				'BIT_6',
				'BIT_7',
				'BIT_8',
				'BIT_9',
				'BIT_10',
				'BIT_11',
				'BIT_12',
				'BIT_13',
				'BIT_14',
				'BIT_15',
			],		
		},

		{
			key:'drv_hipri',
			group:{en:'Register',ru:'Регистр'},
			caption:'HI-PRI',
			visibleIf:"extra_data=register",
			control:'edit',
			listholder:true,
			checkbox:true,
			undef:true,
			def:0,
			drvdef:0,
			list:[
				[0,{ru:'0 - откл',en:'0 - off'}],
				[1,{ru:'1 - вкл',en:'1 - on'}],
			],
			help:{
				ru:`Высокий приоритет опроса. Запрос добавляется в начало очереди, а не в конец, как у обычных переменных. 
				
				При большом количестве HI-PRI переменных может возникнуть ситуация, когда обычные переменные не будут успевать опрашиваться, т.к. очередь будет всегда заполняться HI-PRI переменными.`,
				en:`High polling priority. The request is added to the beginning of the queue, not to the end, as with ordinary variables. 
				
				If there are a large number of HI-PRI variables, a situation may arise when ordinary variables will not be polled in time, because the queue will always be filled with HI-PRI variables.`
			}
		}, 

		{
			key:'drv_group_id',
			group:{en:'Register',ru:'Регистр'},
			caption:'GroupId',
			visibleIf:"extra_data=register",
			control:'input',
			help:{
				ru:"Число 0 - 65535, см. Группировка в разделе Узел",
				en:"Number 0 - 65535, see the Grouping option in the Node section"
			}
		},

		{
			key:'drv_node_host',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Host',ru:'Хост'},
			control:'input',
			def:'127.0.0.1',
			drvdef:'127.0.0.1',
			help:{
				ru:"IP-адрес или доменное имя",
				en:"IP-address or domain name"
			}
		},
		
		{
			key:'drv_node_port',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Port',ru:'Порт'},
			control:'input',
			def:502,
			drvdef:502,
			help:{
				ru:"Номер Modbus-порта",
				en:"Modbus-port number"
			}
		},

		{
			key:'drv_node_slave',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Slave',ru:'Slave'},
			control:'input',
			def:1,
			drvdef:1,
			help:{
				ru:"Modbus Slave ID, 1 - 247",
				en:"Modbus Slave ID, 1 - 247"
			}
		},

		{
			key:'drv_node_offset',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Addr. offset',ru:'Смещение адресов'},
			control:'input',
			def:0,
			drvdef:0,
			help:{
				ru:`
					Смещение адресов регистров.
					Физический адрес регистра = Адрес + Смещение
				`,
				en:`
					Offset of register addresses.
					Register Physical Address = Address + Offset
				`
			}
		},

		{
			key:'drv_node_timeout',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Timeout',ru:'Таймаут'},
			control:'input',
			def:100,
			drvdef:100,
			help:{
				ru:"Таймаут запроса, мс",
				en:"Request timeout, ms"
			}
		},

		{
			key:'drv_node_delay',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Delay',ru:'Задержка'},
			control:'input',
			def:0,
			drvdef:0,
			help:{
				ru:"Задержка между запросами, мс (параметр async должен быть отключен)",
				en:"Delay between requests, ms (async parameter must be disabled)"
			}
		},

		{
			key:'drv_node_keepalive',
			group:{en:'Node',ru:'Узел'},
			caption:'Keep-Alive',
			control:'edit',
			undef:true,
			listholder:true,
			checkbox:true,
			def:1,
			drvdef:1,
			list:[
				[0,{ru:'0 - откл',en:'0 - off'}],
				[1,{ru:'1 - вкл',en:'1 - on'}],
			],
			help:{
				ru:"Держать соединение открытым (не закрывать соединение каждого запроса)",
				en:"Don't close the connection after each request"
			}
		}, 

		{
			key:'drv_node_sharecon',
			group:{en:'Node',ru:'Узел'},
			caption:'Share-Con',
			control:'edit',
			undef:true,
			listholder:true,
			checkbox:true,
			def:1,
			drvdef:1,
			list:[
				[0,{ru:'0 - откл',en:'0 - off'}],
				[1,{ru:'1 - вкл',en:'1 - on'}],
			],
			help:{
				ru:"Использовать по возможности одно TCP-соединение для нескольких узлов",
				en:"Use one TCP-connection for several nodes if possible"
			}
		},

		{
			key:'drv_node_group',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Grouping',ru:'Группировка'},
			control:'edit',
			undef:true,
			listholder:true,
			checkbox:true,
			def:1,
			drvdef:1,
			list:[
				[0,{ru:'0 - не группировать',en:'do not group'}],
				[1,{ru:'1 - группировать без пробелов',en:'group w/o spaces'}],
				[2,{ru:'2 - группировать с пробелами',en:'group with spaces'}],
				[3,{ru:'3 - группировать по переменным',en:'group by variables'}],
				[4,{ru:'4 - группировать по GroupId',en:'group by GroupId'}],
			],
			help:{
				ru:`
					Группировка регистров в запросе:
					0 - не группировать (отдельный запрос на каждый регистр)
					1 - группировать без пробелов (отдельный запрос на группу регистров с последовательными адресами, без пробелов)
					2 - группировать с пробелами (запрос составляется из максимального количества регистров)
					3 - группировать по переменным (регистры каждой переменной запрашиваются отдельным запросом)
					4 - группировать по GroupId (регистры переменных с одинаковым GroupId запрашиваются отдельным запросом)
				`,
				en:`
					Grouping registers in a request:
					0 - do not group (one request for each register)
					1 - group without spaces (one request for a group of registers with sequential addresses, without spaces)
					2 - group with spaces (the request is composed of the maximum number of registers)
					3 - group by variables (registers of each variable are requested in a separate request)
					4 - group by GroupId (variable registers with the same GroupId are requested in a separate request)
				`
			}
		},

		{
			key:'drv_node_ignore_errors',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Ignore Errors',ru:'Игнорировать ошибки'},
			control:'input',
			def:0,
			drvdef:0,
			help:{
				ru:`Количество идущих подряд ошибок чтения, которые будут проигнорированы, 0 - 15. 0 - не игнорировать ошибки`,
				en:`Number of consecutive read errors that will be ignored, 0 - 15. 0 - do not ignore errors`
			}
		},

		{
			key:'drv_node_writecmd',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Write HR function',ru:'Функция записи HR'},
			control:'edit',
			list:[16,6],
			def:16,
			drvdef:16,
			help:{
				ru:`Код функции записи для Holding Register: 16 или 6`,
				en:`Write function code for Holding Register: 16 or 6`
			}
		},



		commonDriverParams.retry_count,
commonDriverParams.retry_interval,
		commonDriverParams.round,
commonDriverParams.threshold,
commonDriverParams.threstime,
commonDriverParams.min,
commonDriverParams.max,
commonDriverParams.coef,
commonDriverParams.offset,
commonDriverParams.scl_hi,
commonDriverParams.scl_lo,            
commonDriverParams.raw_hi,
commonDriverParams.raw_lo,
commonDriverParams.reset_interval,
commonDriverParams.reset_value,
commonDriverParams.group_vars,
commonDriverParams.group_inv,
commonDriverParams.group_ignore,
commonDriverParams.group_block,
commonDriverParams.group_sync,
commonDriverParams.group_timeout,
commonDriverParams.group_tolerance,
		commonDriverParams.extra_service_remote_map_mbtcp__enabled,
commonDriverParams.service_remote_map_mbtcp__port,
commonDriverParams.service_remote_map_mbtcp__slave,
commonDriverParams.service_remote_map_mbtcp__reg_type,
commonDriverParams.service_remote_map_mbtcp__reg_addr,
commonDriverParams.service_remote_map_mbtcp__reg_off,
commonDriverParams.service_remote_map_mbtcp__format,

commonDriverParams.extra_service_local_map_mbtcp__enabled,
commonDriverParams.service_local_map_mbtcp__port,
commonDriverParams.service_local_map_mbtcp__slave,
commonDriverParams.service_local_map_mbtcp__reg_type,
commonDriverParams.service_local_map_mbtcp__reg_addr,
commonDriverParams.service_local_map_mbtcp__reg_off,
commonDriverParams.service_local_map_mbtcp__format,
		commonDriverParams.extra_service_local_exec__enabled,
commonDriverParams.service_local_exec__init,
commonDriverParams.service_local_exec__timeout,
commonDriverParams.service_local_exec__command,
		commonDriverParams.sim_random,
commonDriverParams.sim_random_period,
commonDriverParams.sim_random_delta,
commonDriverParams.sim_random_init,
commonDriverParams.sim_random_min,
commonDriverParams.sim_random_max,

		commonDriverParams.extra_server_comment,
	]
};
	let drv_mqtt={
	typename: {
		ru: "Переменная MQTT",
		en: "MQTT var",
	},
	drvdesc: {
		ru: "Переменные драйвера MQTT",
		en: "MQTT driver vars",
	},
	dontApplyCommonProps: true,
	properties:[
		
		commonDriverParams.name,
		commonDriverParams.extra_server_active,
		commonDriverParams.extra_server_unit,
		commonDriverParams.trend,
		commonDriverParams.buffer,
		commonDriverParams.gateway,
		commonDriverParams.readonly,
		commonDriverParams.extra_server_tags,
		commonDriverParams.extra_server_description,

		{
			key:'drv_dcmsep',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Decimal separator',ru:'Десятичный разделитель'},
			control:'select',
			def:'.',
			drvdef:'.',
			list:[
				'.',','
			]
		},

		{
			key:'drv_wrrnd',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Write rounding',ru:'Округление записи'},
			control:'input',
			help:{
				ru:`
				Округление значения при выполнении команды записи до указанного количества знаков после запятой
				`,
				en:`
				Rounds a value when executing a write command to the specified number of decimal places
				`
			}
		},

		{
			key:'drv_timeout',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Timeout',ru:'Таймаут'},
			control:'input',
			help:{
				ru:`
				Таймаут ожидания сообщения, мс. Если таймаут > 0 и в течение данного периода времени не будет получено ни одного MQTT-сообщения в топик переменной, переменная будет установлена в ошибку DRV_MQTT_ERROR_PUBLISHER_OFFLINE. 
				`,
				en:`
				Message waiting timeout, ms. If timeout > 0 and no MQTT messages are received in the variable topic during this period of time, the variable will be set to the DRV_MQTT_ERROR_PUBLISHER_OFFLINE error.
				`
			}
		},

		{
			key:'drv_topic',
			group:{en:'Topic',ru:'Топик'},
			caption:'',
			hideCaption:true,
			control:'input',
		},
		


		{
			key:'extra_format',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'Format',ru:'Формат'},
			control:'select',
			def:'num',
			drvdef:'num',
			undef:true,
			list:[
				['num',{en:'Number',ru:'Число'}],
				['json','JSON'],
				['text',{en:'Text',ru:'Текст'}],
				['regx','Regex'],
			]
		},

		{
			key:'drv_json',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'JSON-path',ru:'JSON-путь'},
			control:'input',
			visibleIf:'extra_format=json',
			help:{
				ru:`
				Путь к параметру в JSON-объекте. Для обращения свойству объекта используйте имя свойства, а для обращения к элементу массива используйте индекс.

				Например: 
				params/temperature
				sensors/102/params/temperature

				Для получения данных из элемента массива с неизвестным заранее индексом, используйте комбинацию парамтеров JSON-путь и Условие с подстановкой символа * (звездочка):

				JSON-путь: sensors/*/params/temperature
				Условие:   sensors/*/params/id:102
				`,
				en:`
				Path to the parameter in the JSON object. To access a property of an object, use the property name, and to access an array element, use the index.

				For example:
				parameters/temperature
				sensors/102/params/temperature

				To obtain data from an array element with an index unknown in advance, use a combination of the JSON path and Condition parameters with the * (asterisk) symbol substitution:

				JSON path: sensors/*/params/temperature
				Condition: sensors/*/params/id:102
				`
			}
		},

		{
			key:'drv_text',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'Text pattern',ru:'Текстовый паттерн'},
			control:'input',
			visibleIf:'extra_format=text',
			help:{
				ru:"Текстовый паттерн с подстановкой ключевого слова %value% там, где требуется получить числовое значение. Например: param=pressure;value=%value%",
				en:"A text pattern with the %value% keyword substituted wherever a numeric value is required. For example: param=pressure;value=%value%"
			}
		},
		
		{
			key:'drv_regx',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'Regex pattern',ru:'Regex паттерн'},
			control:'input',
			visibleIf:'extra_format=regx',
			help:{
				ru:"Regex паттерн. Например: param=humidity;value=([-,0-9,.]+)",
				en:"Regex pattern. For example: param=humidity;value=([-,0-9,.]+)"
			}
		},
		
		{
			key:'drv_num',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'Encoding',ru:'Кодировка'},
			visibleIf:'extra_format=num',
			control:'select',
			def:'dec',
			drvdef:'dec',
			undef:true,
			list:[
				['dec', 'Десятичное'],
				'HEX_BE_INT8',
				'HEX_BE_INT16',
				'HEX_BE_INT32',
				'HEX_BE_INT64',
				'HEX_BE_UINT8',
				'HEX_BE_UINT16',
				'HEX_BE_UINT32',
				'HEX_BE_UINT64',
				'HEX_BE_FLOAT',
				'HEX_BE_DOUBLE',
				'HEX_LE_INT8',
				'HEX_LE_INT16',
				'HEX_LE_INT32',
				'HEX_LE_INT64',
				'HEX_LE_UINT8',
				'HEX_LE_UINT16',
				'HEX_LE_UINT32',
				'HEX_LE_UINT64',
				'HEX_LE_FLOAT',
				'HEX_LE_DOUBLE',
			]
		},

		// {
		// 	key:'drv_offset',
		// 	group:{en:'Format',ru:'Формат'},
		// 	caption:{en:'Offset',ru:'Смещение'},
		// 	control:'input',
		// 	def:0,
		// 	drvdef:0,
		// 	visibleIf:'extra_format=num',
		// 	help:{
		// 		ru:`
		// 			Смещение от начала пакета данных, байты
		// 		`,
		// 		en:`
		// 			Offset from the start of the data packet, bytes
		// 		`,
		// 	}
		// },

		{
			key:'drv_match',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'Match',ru:'Условие'},
			control:'input',
			visibleIf:'extra_format=regx,text,json',
			help:{
				ru:`Строка или json-объект (в зависимости от формата), условие для считывания значения.
				
				Если свойство задано, значение будет считываться только при соответствии условия считанным данным. Данную опцию можно использовать для фильтрации сообщений, приходящих в один топик.

				- для работы с text: строка, текстовая маска с использованием символов подстановки * и ?
				Например: *sensor_id=S75*

				- для работы с regx: строка, регулярное выражение
				Например: .*sensor_id=S75.*
				
				- для работы с json: набор условий, разделенных переносом строк.
				Например:
				путь/к/значению1:значение1
				путь/к/значению2:значение2
				sensor_id:S75
				model/vendor:Alfamatika
				`,
				en:`String or json object (depending on format), condition for reading the value.

				If the property is set, the value will be read only if the condition matches the read data. This option can be used to filter messages coming to one topic.

				- for working with text: string, text mask using wildcard characters * and ?
				For example: *sensor_id=S75*

				- for working with regx: string, regular expression
				For example: .*sensor_id=S75.*

				- for working with json: a set of conditions separated by line breaks.
				For example:
				path/to/value1:value1
				path/to/value2:value2
				sensor_id:S75
				model/vendor:Alfamatika
				`,
			}
		},

		{
			key:'drv_list',
			group:{en:'Substitution list',ru:'Список подстановки'},
			control:'edit',
			visibleIf:'extra_format=regx,text,json',
			multiline:1,
			groupHelp:{
				ru:`
					Список из пар Ключ:Значение разделенных переносом строки.
					При считывании текстового значения, равного Ключу, в переменную будет записано соответствующее Ключу Значение. При выполнении команды записи перобразование будет выполнено в обратную сторону (Значение -> Ключ). В именах ключей допускается использование символов * и ?. 
					
					Например:
					
					stop:1
					start:2
					*mode*:3
					*:0
				`,
				en:`
					A list of Key:Value pairs separated by line breaks.
					When reading a text value equal to the Key, the Value corresponding to the Key will be written to the variable. When executing a write command, the conversion will be performed in the opposite direction (Value -> Key).
						
					For example:
				
					stop:0
					start:1
					auto:2
				`
			},
		},

		{
			key:'drv_write',
			group:{en:'Write command',ru:'Команда записи'},
			visibleIf:'extra_format=regx,text,json',
			groupHelp:{
				ru:`
					Текст, который будет отправлен в топик при выполнении записи значения переменной.
					Например:

					{
						"sensor":123,
						"setpoint":%value%
					}

					Вместо %value% будет подставлено устанавливаемое значение переменной.

					Если данный параметр не задан, в топик будет отправленно:
					- при наличии списка подстановки: строка, соответствующая устанавливаемому значению
					- иначе: устанавливаемое значение в виде десятичного числа, где в качестве разделителя будет указан символ, заданный в параметре "Десятичный разделитель".
				`,
				en:`
				The text that will be sent to the topic when writing the variable value.
					 For example:

					 {
						 "sensor":123,
						 "setpoint":%value%
					 }

					 Instead of %value% the value of the variable being set will be substituted.

					 If this parameter is not specified, the following will be sent to the topic:
					 - if there is a substitution list: a string corresponding to the value being set
					 - otherwise: the value to be set in the form of a decimal number, where the character specified in the "Decimal separator" parameter will be specified as the separator.
				`
			},
			caption:'',
			hideCaption:true,
			control:'textarea',
		},

		{
			key:'drv_node_host',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Host',ru:'Хост'},
			control:'input',
			def:'localhost',
			drvdef:'localhost',
			help:{
				ru:"IP-адрес или доменное имя MQTT-брокера",
				en:"IP-address or domain name of MQTT-broker"
			}
		},

		{
			key:'drv_node_port',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Port',ru:'Порт'},
			control:'input',
			def:1883,
			drvdef:1883,
			help:{
				ru:`
					Номер порта MQTT-брокера. 
					1883 - стандартный порт брокера без SSL-шифрования.
					8883 - стандартный порт брокера с SSL-шифрованием.
				`,
				en:`
					MQTT broker port number.
					1883 is the standard broker port without SSL encryption.
					8883 is a standard broker port with SSL encryption.
				`
			}
		},

		{
			key:'drv_node_ssl',
			group:{en:'Node',ru:'Узел'},
			caption:'SSL',
			control:'edit',
			undef:true,
			def:0,
			drvdef:0,
			listholder:true,
			checkbox:true,
			list:[
				[0,{ru:'0 - откл',en:'0 - off'}],
				[1,{ru:'1 - вкл',en:'1 - on'}],
			],
			help:{
				ru:`
					Включить SSL-шифрование. Данная опция должна соответствовать настройкам MQTT-брокера.
				`,
				en:`
					Enable SSL encryption. This option must match the settings of the MQTT broker.
				`
			}
		},

		{
			key:'drv_node_checkcert',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Check certificate',ru:'Проверка сертификата'},
			control:'edit',
			undef:true,
			def:0,
			drvdef:0,
			listholder:true,
			checkbox:true,
			list:[
				[0,{ru:'0 - откл',en:'0 - off'}],
				[1,{ru:'1 - вкл',en:'1 - on'}],
			],
			help:{
				ru:`
					Включить проверку SSL-сертификата
				`,
				en:`
					Enable SSL certificate verification
				`
			}
		},

		{
			key:'drv_node_timeout',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Connect timeout',ru:'Таймаут подключения'},
			control:'input',
			def:10000,
			drvdef:10000,
			help:{
				ru:`
				Таймаут подключения к MQTT-брокеру, мс
				`,
				en:`
				Connection timeout to the MQTT-broker, ms
				`
			}
		},

		{
			key:'drv_node_username',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Username',ru:'Пользователь'},
			control:'input',
			help:{
				ru:`
					Имя пользователя на MQTT-брокере
				`,
				en:`
					MQTT broker username
				`
			}
		},

		{
			key:'drv_node_password',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Password',ru:'Пароль'},
			control:'input',
			help:{
				ru:`
					Пароль пользователя на MQTT-брокере
				`,
				en:`
					MQTT broker password
				`
			}
		},

		{
			key:'drv_node_qos',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'QoS',ru:'QoS'},
			control:'input',
			def:1,
			drvdef:1,
			help:{
				ru:`
					MQTT Quality of Service
				`,
				en:`
					MQTT Quality of Service
				`
			}
		},

		{
			key:'drv_node_retain',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Retain',ru:'Retain'},
			control:'edit',
			undef:true,
			def:0,
			drvdef:0,
			listholder:true,
			checkbox:true,
			list:[
				[0,{ru:'0 - откл',en:'0 - off'}],
				[1,{ru:'1 - вкл',en:'1 - on'}],
			],
			help:{
				ru:`
				Включить флаг Retain при публикации сообщений для данного узла. Данный флаг указывает MQTT-брокеру сохранять публикуемые сообщения. 
				`,
				en:`
				Включить флажок Сохранять при публикации сообщений для данного узла. Данный флаг указывает на сохранение публикуемых сообщений MQTT-брокером.
				`
			}
		},

		commonDriverParams.round,
commonDriverParams.threshold,
commonDriverParams.threstime,
commonDriverParams.min,
commonDriverParams.max,
commonDriverParams.coef,
commonDriverParams.offset,
commonDriverParams.scl_hi,
commonDriverParams.scl_lo,            
commonDriverParams.raw_hi,
commonDriverParams.raw_lo,
commonDriverParams.reset_interval,
commonDriverParams.reset_value,
commonDriverParams.group_vars,
commonDriverParams.group_inv,
commonDriverParams.group_ignore,
commonDriverParams.group_block,
commonDriverParams.group_sync,
commonDriverParams.group_timeout,
commonDriverParams.group_tolerance,
		commonDriverParams.extra_service_remote_map_mbtcp__enabled,
commonDriverParams.service_remote_map_mbtcp__port,
commonDriverParams.service_remote_map_mbtcp__slave,
commonDriverParams.service_remote_map_mbtcp__reg_type,
commonDriverParams.service_remote_map_mbtcp__reg_addr,
commonDriverParams.service_remote_map_mbtcp__reg_off,
commonDriverParams.service_remote_map_mbtcp__format,

commonDriverParams.extra_service_local_map_mbtcp__enabled,
commonDriverParams.service_local_map_mbtcp__port,
commonDriverParams.service_local_map_mbtcp__slave,
commonDriverParams.service_local_map_mbtcp__reg_type,
commonDriverParams.service_local_map_mbtcp__reg_addr,
commonDriverParams.service_local_map_mbtcp__reg_off,
commonDriverParams.service_local_map_mbtcp__format,
		commonDriverParams.extra_service_local_exec__enabled,
commonDriverParams.service_local_exec__init,
commonDriverParams.service_local_exec__timeout,
commonDriverParams.service_local_exec__command,
		commonDriverParams.sim_random,
commonDriverParams.sim_random_period,
commonDriverParams.sim_random_delta,
commonDriverParams.sim_random_init,
commonDriverParams.sim_random_min,
commonDriverParams.sim_random_max,

		commonDriverParams.extra_server_comment,
	]
};
	let drv_opcua={
	typename: {
		ru: "Переменная OPC UA",
		en: "OPC UA var",
	},
	drvdesc: {
		ru: "Переменные драйвера OPC UA",
		en: "OPC UA driver vars",
	},
	dontApplyCommonProps: true,
	properties:[
		
		commonDriverParams.name,
		commonDriverParams.extra_server_active,
		commonDriverParams.extra_server_unit,
		commonDriverParams.trend,
		commonDriverParams.buffer,
		commonDriverParams.gateway,
		commonDriverParams.readonly,
		commonDriverParams.extra_server_tags,
		commonDriverParams.extra_server_description,

		{
			key:'extra_data',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Data',ru:'Данные'},
			control:'select',
			def:'var',
			list:[
				['var',{en:'Variable',ru:'Переменная'}],
				['metric',{en:'Metric',ru:'Метрика'}],
			],
			help:{
				ru:`<b>Переменная</b>: в переменную будет считываться значение переменной с OPCUA-сервера
				
				<b>Метрика</b>: в переменную будет считываться значение соответствующей метрики
				`,
				en:`<b>Variable</b>: the variable value will be read into the variable from the OPCUA server
				
				<b>Metric</b>: the value of the corresponding metric will be read into the variable
				`
			}
		},

		{
			key:'drv_metric',
			group:{en:'Mode',ru:'Режим'},
			caption:{en:'Metric',ru:'Метрика'},
			control:'select',
			visibleIf:"extra_data=metric",
			list:[
				['online',{en:'Online',ru:'Онлайн'}],
			]
		},
		
		{
			key:'drv_ua_nodeid',
			group:{en:'Variable',ru:'Переменная'},
			caption:'Node ID',
			control:'input',
			visibleIf:"extra_data!=metric",
			help:{
				ru:`
				OPCUA Node ID в формате 
				ns=<namespaceindex>;<type>=<varname>

				Примеры:

				ns=1;i=1466
				ns=8;s=StringVarName1
				ns=5;g=BAEAF004-1E43-4A06-9EF0-E52010D5CD10
				ns=5;b=AP8=
				`,
				en:`
				OPCUA Node ID in the format
				ns=<namespaceindex>;<type>=<varname>

				Examples:

				ns=1;i=1466
				ns=8;s=StringVarName1
				ns=5;g=BAEAF004-1E43-4A06-9EF0-E52010D5CD10
				ns=5;b=AP8=
				`
			}
		}, 

		{
			key:'drv_node_url',
			group:{en:'Server',ru:'Сервер'},
			caption:{en:'URL',ru:'URL'},
			control:'input',
			def:'opc.tcp://127.0.0.1:4840',
			drvdef:'opc.tcp://127.0.0.1:4840',
			help:{
				ru:`
				URL OPCUA-сервера. Порт по умолчанию: 4840.

				Например:

				opc.tcp://localhost
				opc.tcp://192.168.0.148:12345
				opc.tcp://domain.name:12345
				`,
				en:`
				URL of OPCUA-server. Default port is 4840.

				Examples:

				opc.tcp://localhost
				opc.tcp://192.168.0.148:12345
				opc.tcp://domain.name:12345
				`
			}
		},

		{
			key:'drv_node_timeout',
			group:{en:'Server',ru:'Сервер'},
			caption:{en:'Timeout',ru:'Таймаут'},
			control:'input',
			def:10000,
		},

		{
			key:'drv_node_subscription_attempts',
			group:{en:'Subscription',ru:'Подписка'},
			caption:{en:'Attempts',ru:'Попытки'},
			control:'input',
			def:10,
			help:{
				ru:`
				Количество попыток подписаться на переменные.
				Если подписаться не удалось - переменные устанавливаются в ошибку DRV_OPCUA_ERROR_SUBSCRIBE.
				`,
				en:`
				Number of attempts to subscribe to variables.
				If the subscription fails - the variables are set to DRV_OPCUA_ERROR_SUBSCRIBE error.
				`
			}
		},

		{
			key:'drv_node_subscription_interval',
			group:{en:'Subscription',ru:'Подписка'},
			caption:{en:'Interval',ru:'Интервал'},
			control:'input',
			def:1000,
			help:{
				ru:`
				Интервал между попытками подписки, мс
				`,
				en:`
				Interval between subscription attempts, ms
				`
			}
		},

		{
			key:'drv_node_subscription_per_request',
			group:{en:'Subscription',ru:'Подписка'},
			caption:{en:'Vars/request',ru:'Переменных/запрос'},
			control:'input',
			def:1000,
			help:{
				ru:`
				Количество переменных в одном запросе на подписку.
				Если общее количество переменных сервера, на которые подписывается драйвер, больше данного количества, подписка будет разбита на несколько запросов.

				Некоторые серверы слишком долго обрабатывают запросы на подписку с большим количеством переменных и не успевают ответить до таймаута. В таких случаях рекомендуется уменьшить значение данного параметра, и таким образом разбить подписку на более мелкие запросы.
				`,
				en:`
				Number of variables in one subscription request.
				If the total number of server variables to which the driver subscribes is greater than this number, the subscription will be split into multiple requests.

				Some servers take too long to process subscription requests with a large number of variables and fail to respond before timeout. In such cases, it is recommended to reduce the value of this parameter and thus split the subscription into smaller requests.
				`
			}
		},

		commonDriverParams.round,
commonDriverParams.threshold,
commonDriverParams.threstime,
commonDriverParams.min,
commonDriverParams.max,
commonDriverParams.coef,
commonDriverParams.offset,
commonDriverParams.scl_hi,
commonDriverParams.scl_lo,            
commonDriverParams.raw_hi,
commonDriverParams.raw_lo,
commonDriverParams.reset_interval,
commonDriverParams.reset_value,
commonDriverParams.group_vars,
commonDriverParams.group_inv,
commonDriverParams.group_ignore,
commonDriverParams.group_block,
commonDriverParams.group_sync,
commonDriverParams.group_timeout,
commonDriverParams.group_tolerance,
		commonDriverParams.extra_service_remote_map_mbtcp__enabled,
commonDriverParams.service_remote_map_mbtcp__port,
commonDriverParams.service_remote_map_mbtcp__slave,
commonDriverParams.service_remote_map_mbtcp__reg_type,
commonDriverParams.service_remote_map_mbtcp__reg_addr,
commonDriverParams.service_remote_map_mbtcp__reg_off,
commonDriverParams.service_remote_map_mbtcp__format,

commonDriverParams.extra_service_local_map_mbtcp__enabled,
commonDriverParams.service_local_map_mbtcp__port,
commonDriverParams.service_local_map_mbtcp__slave,
commonDriverParams.service_local_map_mbtcp__reg_type,
commonDriverParams.service_local_map_mbtcp__reg_addr,
commonDriverParams.service_local_map_mbtcp__reg_off,
commonDriverParams.service_local_map_mbtcp__format,
		commonDriverParams.extra_service_local_exec__enabled,
commonDriverParams.service_local_exec__init,
commonDriverParams.service_local_exec__timeout,
commonDriverParams.service_local_exec__command,
		commonDriverParams.sim_random,
commonDriverParams.sim_random_period,
commonDriverParams.sim_random_delta,
commonDriverParams.sim_random_init,
commonDriverParams.sim_random_min,
commonDriverParams.sim_random_max,

		commonDriverParams.extra_server_comment,
	]
};
	let drv_snmp={
	typename: {
		ru: "Переменная SNMP",
		en: "SNMP var",
	},
	drvdesc: {
		ru: "Переменные драйвера SNMP",
		en: "SNMP driver vars",
	},
	dontApplyCommonProps: true,
	clipboardType:'prototype_mb',
	properties:[
		
		commonDriverParams.name,
		commonDriverParams.extra_server_active,
		commonDriverParams.extra_server_unit,
		commonDriverParams.trend,
		commonDriverParams.buffer,
		commonDriverParams.gateway,
		commonDriverParams.readonly,
		commonDriverParams.extra_server_tags,
		commonDriverParams.extra_server_description,
		commonDriverParams.extra_polling,
		change(commonDriverParams.drv_period,{def:1000}),
		commonDriverParams.drv_cron,

		
		{
			key:'drv_oid',
			group:{en:'SNMP',ru:'SNMP'},
			caption:{en:'OID',ru:'OID'},
			control:'input',
			help:{
				ru:`
				Числовой OID (идентификатор объекта): иерархический адрес, используемый в SNMP для идентификации и доступа к определенным точкам данных или ресурсам в сетевом устройстве.
				
				Например:
				1.3.6.1.4.1.2021.10.1.5.1
				1.3.6.1.2.1.25.3.3.1.2.1
				1.3.6.1.2.1.4.3.0
				1.3.6.1.2.1.1.1.0

				Получить список доступных OID устройства можно утилитой <em>snmpwalk</em>:

				snmpwalk -v 2c -c public DEVICE_IP

				Утилита вернет список OID в строковом представлении, например <em>UCD-SNMP-MIB::laLoadInt.1</em>. Получить числовое представление OID можно утилитой <em>snmptranslate</em>:

				snmptranslate -On STRING_OID

				Например:

				> snmptranslate -On UCD-SNMP-MIB::laLoadInt.1
				.1.3.6.1.4.1.2021.10.1.5.1

				`,
				en:`
				OID (Object Identifier): A hierarchical address used in SNMP to identify and access specific data points or resources in a networked device.

				Example:
				1.3.6.1.4.1.2021.10.1.5.1
				1.3.6.1.2.1.25.3.3.1.2.1
				1.3.6.1.2.1.4.3.0
				1.3.6.1.2.1.1.1.0

				You can get a list of available device OIDs using the <em>snmpwalk</em> utility:

				snmpwalk -v 2c -c public DEVICE_IP

				The utility will return a list of OIDs in string representation, for example <em>UCD-SNMP-MIB::laLoadInt.1</em>. You can get a numeric representation of the OID using the <em>snmptranslate</em> utility:

				snmptranslate -On STRING_OID

				For example:

				> snmptranslate -On UCD-SNMP-MIB::laLoadInt.1
				.1.3.6.1.4.1.2021.10.1.5.1

				`
			}
		},

		{
			key:'drv_node_community',
			group:{en:'SNMP',ru:'SNMP'},
			caption:{en:'Community',ru:'Community'},
			control:'input',
			def:'public',
			drvdef:'public',
			help:{
				ru:`Идентификатор уровня доступа к данным`,
				en:`Data access level identifier`,
			}
		},


		{
			key:'extra_format',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'Format',ru:'Формат'},
			control:'select',
			def:'num',
			drvdef:'num',
			undef:true,
			list:[
				['num',{en:'Number',ru:'Число'}],
				['json','JSON'],
				['text',{en:'Text',ru:'Текст'}],
				['regx','Regex'],
			]
		},

		{
			key:'drv_json',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'JSON-path',ru:'JSON-путь'},
			control:'input',
			visibleIf:'extra_format=json',
			help:{
				ru:`
				Путь к параметру в JSON-объекте. Для обращения свойству объекта используйте имя свойства, а для обращения к элементу массива используйте индекс.

				Например: 
				params/temperature
				sensors/102/params/temperature

				Для получения данных из элемента массива с неизвестным заранее индексом, используйте комбинацию парамтеров JSON-путь и Условие с подстановкой символа * (звездочка):

				JSON-путь: sensors/*/params/temperature
				Условие:   sensors/*/params/id:102
				`,
				en:`
				Path to the parameter in the JSON object. To access a property of an object, use the property name, and to access an array element, use the index.

				For example:
				parameters/temperature
				sensors/102/params/temperature

				To obtain data from an array element with an index unknown in advance, use a combination of the JSON path and Condition parameters with the * (asterisk) symbol substitution:

				JSON path: sensors/*/params/temperature
				Condition: sensors/*/params/id:102
				`
			}
		},

		{
			key:'drv_text',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'Text pattern',ru:'Текстовый паттерн'},
			control:'input',
			visibleIf:'extra_format=text',
			help:{
				ru:"Текстовый паттерн с подстановкой ключевого слова %value% там, где требуется получить числовое значение. Например: param=pressure;value=%value%",
				en:"A text pattern with the %value% keyword substituted wherever a numeric value is required. For example: param=pressure;value=%value%"
			}
		},
		
		{
			key:'drv_regx',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'Regex pattern',ru:'Regex паттерн'},
			control:'input',
			visibleIf:'extra_format=regx',
			help:{
				ru:"Regex паттерн. Например: param=humidity;value=([-,0-9,.]+)",
				en:"Regex pattern. For example: param=humidity;value=([-,0-9,.]+)"
			}
		},
		
		{
			key:'drv_num',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'Encoding',ru:'Кодировка'},
			visibleIf:'extra_format=num',
			control:'select',
			def:'auto',
			drvdef:'auto',
			undef:true,
			list:[
				'auto',
				'HEX_BE_INT8',
				'HEX_BE_INT16',
				'HEX_BE_INT32',
				'HEX_BE_INT64',
				'HEX_BE_UINT8',
				'HEX_BE_UINT16',
				'HEX_BE_UINT32',
				'HEX_BE_UINT64',
				'HEX_BE_FLOAT',
				'HEX_BE_DOUBLE',
				'HEX_LE_INT8',
				'HEX_LE_INT16',
				'HEX_LE_INT32',
				'HEX_LE_INT64',
				'HEX_LE_UINT8',
				'HEX_LE_UINT16',
				'HEX_LE_UINT32',
				'HEX_LE_UINT64',
				'HEX_LE_FLOAT',
				'HEX_LE_DOUBLE',
			]
		},

		// {
		// 	key:'drv_offset',
		// 	group:{en:'Format',ru:'Формат'},
		// 	caption:{en:'Offset',ru:'Смещение'},
		// 	control:'input',
		// 	def:0,
		// 	drvdef:0,
		// 	visibleIf:'extra_format=num',
		// 	help:{
		// 		ru:`
		// 			Смещение от начала пакета данных, байты
		// 		`,
		// 		en:`
		// 			Offset from the start of the data packet, bytes
		// 		`,
		// 	}
		// },

		{
			key:'drv_match',
			group:{en:'Format',ru:'Формат'},
			caption:{en:'Match',ru:'Условие'},
			control:'input',
			visibleIf:'extra_format=regx,text,json',
			help:{
				ru:`Строка или json-объект (в зависимости от формата), условие для считывания значения.
				
				Если свойство задано, значение будет считываться только при соответствии условия считанным данным. Данную опцию можно использовать для фильтрации сообщений, приходящих в один топик.

				- для работы с text: строка, текстовая маска с использованием символов подстановки * и ?
				Например: *sensor_id=S75*

				- для работы с regx: строка, регулярное выражение
				Например: .*sensor_id=S75.*
				
				- для работы с json: набор условий, разделенных переносом строк.
				Например:
				путь/к/значению1:значение1
				путь/к/значению2:значение2
				sensor_id:S75
				model/vendor:Alfamatika
				`,
				en:`String or json object (depending on format), condition for reading the value.

				If the property is set, the value will be read only if the condition matches the read data. This option can be used to filter messages coming to one topic.

				- for working with text: string, text mask using wildcard characters * and ?
				For example: *sensor_id=S75*

				- for working with regx: string, regular expression
				For example: .*sensor_id=S75.*

				- for working with json: a set of conditions separated by line breaks.
				For example:
				path/to/value1:value1
				path/to/value2:value2
				sensor_id:S75
				model/vendor:Alfamatika
				`,
			}
		},

		{
			key:'drv_list',
			group:{en:'Substitution list',ru:'Список подстановки'},
			control:'edit',
			multiline:1,
			visibleIf:'extra_format=regx,text,json',
			groupHelp:{
				ru:`
					Список из пар Ключ:Значение разделенных переносом строки.
					При считывании текстового значения, равного Ключу, в переменную будет записано соответствующее Ключу Значение. При выполнении команды записи перобразование будет выполнено в обратную сторону (Значение -> Ключ). В именах ключей допускается использование символов * и ?. 
					
					Например:
					
					stop:1
					start:2
					*mode*:3
					*:0
				`,
				en:`
					A list of Key:Value pairs separated by line breaks.
					When reading a text value equal to the Key, the Value corresponding to the Key will be written to the variable. When executing a write command, the conversion will be performed in the opposite direction (Value -> Key).
						
					For example:
				
					stop:0
					start:1
					auto:2
				`
			},
		},



		{
			key:'drv_node_host',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Host',ru:'Хост'},
			control:'input',
			def:'127.0.0.1',
			drvdef:'127.0.0.1',
			help:{
				ru:"IP-адрес или доменное имя",
				en:"IP-address or domain name"
			}
		},
		
		{
			key:'drv_node_port',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Port',ru:'Порт'},
			control:'input',
			def:161,
			drvdef:161,
			help:{
				ru:"Номер SNMP-порта",
				en:"SNMP-port number"
			}
		},

		{
			key:'drv_node_timeout',
			group:{en:'Node',ru:'Узел'},
			caption:{en:'Timeout',ru:'Таймаут'},
			control:'input',
			def:100,
			drvdef:100,
			help:{
				ru:"Таймаут запроса, мс",
				en:"Request timeout, ms"
			}
		},

		commonDriverParams.round,
commonDriverParams.threshold,
commonDriverParams.threstime,
commonDriverParams.min,
commonDriverParams.max,
commonDriverParams.coef,
commonDriverParams.offset,
commonDriverParams.scl_hi,
commonDriverParams.scl_lo,            
commonDriverParams.raw_hi,
commonDriverParams.raw_lo,
commonDriverParams.reset_interval,
commonDriverParams.reset_value,
commonDriverParams.group_vars,
commonDriverParams.group_inv,
commonDriverParams.group_ignore,
commonDriverParams.group_block,
commonDriverParams.group_sync,
commonDriverParams.group_timeout,
commonDriverParams.group_tolerance,
		commonDriverParams.extra_service_remote_map_mbtcp__enabled,
commonDriverParams.service_remote_map_mbtcp__port,
commonDriverParams.service_remote_map_mbtcp__slave,
commonDriverParams.service_remote_map_mbtcp__reg_type,
commonDriverParams.service_remote_map_mbtcp__reg_addr,
commonDriverParams.service_remote_map_mbtcp__reg_off,
commonDriverParams.service_remote_map_mbtcp__format,

commonDriverParams.extra_service_local_map_mbtcp__enabled,
commonDriverParams.service_local_map_mbtcp__port,
commonDriverParams.service_local_map_mbtcp__slave,
commonDriverParams.service_local_map_mbtcp__reg_type,
commonDriverParams.service_local_map_mbtcp__reg_addr,
commonDriverParams.service_local_map_mbtcp__reg_off,
commonDriverParams.service_local_map_mbtcp__format,
		commonDriverParams.extra_service_local_exec__enabled,
commonDriverParams.service_local_exec__init,
commonDriverParams.service_local_exec__timeout,
commonDriverParams.service_local_exec__command,
		commonDriverParams.sim_random,
commonDriverParams.sim_random_period,
commonDriverParams.sim_random_delta,
commonDriverParams.sim_random_init,
commonDriverParams.sim_random_min,
commonDriverParams.sim_random_max,

		commonDriverParams.extra_server_comment,
	]
};
	let mimic_controls={
	name:{en:'Mimic Controls',ru:'Контролы Мнемосхемы'},
	params:[
		{
			key:'class',
			control:'textarea',
			group:'class',
			height:30,
		},
		{
			key:'css',
			control:'textarea',
			group:'css',
			height:100,
		},
		{
			key:'style_text',
			control:'textarea',
			group:'style',
			height:100,
		},
	]
};

	schema={
		mimic_controls:mimic_controls,
		types: {
			common: {
				allowed_docs: [
					"@prop.json",
					"@access.json",
				],
				objects_title: {
					ru: "Объекты",
					en: "Objects",
				},
				properties: [
					{
						key: "_type",
						disabled:true,
						caption: {
							ru: "Объект",
							en: "Object",
						},
						props: {
							hide: true,
						},
					},
					{
						key: "_title",
						caption: {
							ru: "Имя",
							en: "Name",
						},
						unique: true,
						trim: true,
						reload: true,
						validation: [
							"notEmpty",
							"path",
						],
					},
					{
						key: "id",
						hide: true,
						caption: {
							ru: "ID",
							en: "ID",
						},
					},
					{
						key: "_prefix",
						hide: true,
						caption: {
							ru: "Префикс",
							en: "Prefix",
						},
						help: {
							ru: `
							Префикс узла. Генерируется автоматически по ID проекта, типу и ID узла.
							
							Все переменные узла получают глобальное имя с данным префиксом. Например, <em>P12_N43_var1</em> или <em>P5_G87_frost_alarm</em>. 
							
							Зная префикс узла и имя переменной в прототипе можно получить глобальное имя переменной, которое может быть использовано, например, в скриптах, для доступа к переменным других узлов.
							`,
							en: `
							Node prefix. Generated automatically by project ID, type and node ID.

							All node variables receive a global name with this prefix. For example, <em>P12_N43_var1</em> or <em>P5_G87_frost_alarm</em>.

							Knowing the node prefix and the variable name in the prototype, you can get the global variable name, which can be used, for example, in scripts to access variables of other nodes.
							`,
						},
					},
					{
						key: "alias",
						caption: {
							ru: "Псевдоним",
							en: "Alias",
						},
						reload: true,
						hide: true,
					},
					commonParams.icon,
					{
						key: "sort",
						caption: {
							ru: "Сортировка",
							en: "Sorting",
						},
						bottom:true,
						control: "select",
						reload: true,
						def: "name_asc",
						hide: true,
						table: {
							hide: true,
						},
						list: [
							[
								"name_asc",
								{
									ru: "По имени ↓",
									en: "By name ↓",
								},
							],
							[
								"name_desc",
								{
									ru: "По имени ↑",
									en: "By name ↑",
								},
							],
							[
								"index_asc",
								{
									ru: "По индексу ↓",
									en: "By index ↓",
								},
							],
							[
								"index_desc",
								{
									ru: "По индексу ↑",
									en: "By index ↑",
								},
							],
							[
								"created_asc",
								{
									ru: "По дате создания ↓",
									en: "By created ↓",
								},
							],
							[
								"created_desc",
								{
									ru: "По дате создания ↑",
									en: "By created ↑",
								},
							],
						],
						help: {
							ru: "Сортировка вложенных объектов",
							en: "Sorting children objects",
						},
					},
					{
						key: "index",
						caption: {
							ru: "Индекс",
							en: "Index",
						},
						bottom:true,
						control: "number",
						unique: true,
						reload: true,
						help: {
							ru: "Индекс сортировки, число -2^31 - 2^31. Позволяет вручную задать позицию объекта в списке и в меню. Для включения сортировки по индексу у объекта-контейнера необходимо установить параметр Сортировка = Индекс.",
							en: "Sorting index, number -2^31 - 2^31. Allows you to manually set the position of an object in the list and menu. To enable sorting by index on a container object, you must set parameter Sorting = Index.",
						},
					},
					commonParams.comment,
				],
			},
			users: {
				typename: {
					ru: "Пользователи",
					en: "Users",
				},
				typedef: {
					name: "users",
					parents: [
						"root",
					],
				},
			},
			user: {
				typename: {
					ru: "Пользователь",
					en: "User",
				},
				typedef: {
					ext: "user",
					parents: [
						"users",
					],
				},
				fill: [
					"sessions",
				],
			},
			sessions: {
				typename: {
					ru: "Сессии",
					en: "Sessions",
				},
				file_format: "json",
				typedef: {
					name: "sessions",
					parents: [
						"user",
					],
				},
			},
			session: {
				typename: {
					ru: "Сессия",
					en: "Session",
				},
				typedef: {
					ext: "session",
					container: "file",
				},
			},
			projects: {
				typename: {
					ru: "Проекты",
					en: "Projects",
				},
				icon: "/images/projects.svg",
				editor: "#editor/home",
				interface: "#interface/projects",
				allowed_docs: [
					"@prop.json",
				],
				typedef: {
					name: "projects",
					parents: [
						"root",
					],
				},
				properties: [
					{
						key: "_title",
						hide: true,
					},
					{
						key: "inactive",
						hide: true,
					},
					{
						key: "alias",
						hide: false,
					},
					{
						key: "sort",
						hide: false,
					},
					{
						key: "icon",
						caption: {
							ru: "Иконка",
							en: "Icon",
						},
						control: "edit",
					},
				],
			},
			project: {
				typename: {
					ru: "Проект",
					en: "Project",
				},
				typedef: {
					ext: "project",
					parents: [
						"projects",
					],
				},
				objects_title: {
					ru: "Разделы",
					en: "Parts",
				},
				icon: "/images/project.svg",
				disable_add_children: true,
				//hide_table_selector:true,
				templates: "projects",
				exportable: true,
				need_build:true,
				create_and_open:false,
				fill: [
					"alarms",
					"journal",
					"schedules",
					"vars",
					"groups",
					"prototypes",
					"nodes",
					"mimics",
					"screens",
					"files",
					"dictionaries",
				],
				properties: [
					{
						key: "description",
						caption: {
							ru: "Описание",
							en: "Description",
						},

						control: "edit",
						multiline: true,
						spellcheck: true,
					},
					commonParams.tags,
					commonParams.inactive,
					//commonParams.invisible,
					commonParams.show_header,
					change(commonParams.hide_alarm_panel,{
						group:{ru:"Разное",en:"Miscellaneous"},
						def:true,
					}),
					{
						key: "sort",
						hide: false,
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
					},
					{
						key: "index",
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
					},
					{
						key: "icon",
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
					},
					{
						key: "comment",
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
					},

					{
						group: {ru: "Звук тревог",en: "Alarm sound",},
						key: "alarm_sound_alarm",
						caption: {ru: "Авария",	en: "Alarm",},
						control: "checkbox",
						def:1,
						table: {hide: true},
					},
					{
						group: {ru: "Звук тревог",en: "Alarm sound",},
						key: "alarm_sound_fault",
						caption: {ru: "Неисправность",	en: "Fault",},
						control: "checkbox",
						def:1,
						table: {hide: true},
					},
					{
						group: {ru: "Звук тревог",en: "Alarm sound",},
						key: "alarm_sound_warning",
						caption: {ru: "Предупреждение",	en: "Warning",},
						control: "checkbox",
						def:1,
						table: {hide: true},
					},
					{
						group: {ru: "Звук тревог",en: "Alarm sound",},
						key: "alarm_sound_notice",
						caption: {ru: "Оповещение",	en: "Notice",},
						control: "checkbox",
						def:1,
						table: {hide: true},
					},


					{
						key: "layout",
						control: "edit",
						table: {
							hide: true,
						},
						group: {
							ru: "Главный экран",
							en: "Home screen",
						},
						caption: {
							ru: "Раскладка",
							en: "Layout",
						},
						def: "V-12-3",
						list: [
							[
								"S-1",
								"S-1",
								"/images/layout-s-1.svg",
							],
							[
								"H-1-2",
								"H-1-2",
								"/images/layout-h-1-2.svg",
							],
							[
								"V-1-2",
								"V-1-2",
								"/images/layout-v-1-2.svg",
							],
							[
								"H-12-3",
								"H-12-3",
								"/images/layout-h-12-3.svg",
							],
							[
								"V-12-3",
								"V-12-3",
								"/images/layout-v-12-3.svg",
							],
							[
								"H-3-12",
								"H-3-12",
								"/images/layout-h-3-12.svg",
							],
							[
								"V-3-12",
								"V-3-12",
								"/images/layout-v-3-12.svg",
							],
							[
								"H-12-34",
								"H-12-34",
								"/images/layout-h-12-34.svg",
							],
							[
								"V-12-34",
								"V-12-34",
								"/images/layout-v-12-34.svg",
							],
						],
					},
					{
						key: "size12",
						control: "input",
						table: {
							hide: true,
						},
						group: {
							ru: "Главный экран",
							en: "Home screen",
						},
						caption: {
							ru: "Размер 12",
							en: "Size 12",
						},
						help:{
							ru:`Начальный размер блока с панелями 1 и 2. 0 - размер будет подстраиваться под Размер 34.`,
							en:`The initial size of the block with panels 1 and 2. 0 - the size will be adjusted to Size 34.`,
						},
						def: 400,
						visibleIf: "layout=H-12-3,H-12-34,V-12-3,V-12-34,H-3-12,V-3-12",
					},
					{
						key: "size34",
						control: "input",
						table: {
							hide: true,
						},
						group: {
							ru: "Главный экран",
							en: "Home screen",
						},
						caption: {
							ru: "Размер 34",
							en: "Size 34",
						},
						help:{
							ru:`Начальный размер блока с панелями 3 и 4. 0 - размер будет подстраиваться под Размер 12.`,
							en:`The initial size of the block with panels 3 and 4. 0 - the size will be adjusted to Size 12.`,
						},
						def: 0,
						visibleIf: "layout=H-12-3,H-12-34,V-12-3,V-12-34,H-3-12,V-3-12",
					},
					{
						key: "size1",
						control: "number",
						table: {
							hide: true,
						},
						group: {
							ru: "Главный экран",
							en: "Home screen",
						},
						caption: {
							ru: "Размер 1",
							en: "Size 1",
						},
						help:{
							ru:`Начальный размер панели 1. 0 - размер будет подстраиваться под Размер 2.`,
							en:`The initial size of panel 1. 0 - the size will be adjusted to Size 2.`,
						},
						def: 500,
						visibleIf: "layout!=S-1",
					},
					{
						key: "size2",
						control: "number",
						table: {
							hide: true,
						},
						group: {
							ru: "Главный экран",
							en: "Home screen",
						},
						caption: {
							ru: "Размер 2",
							en: "Size 2",
						},
						help:{
							ru:`Начальный размер панели 2. 0 - размер будет подстраиваться под Размер 1.`,
							en:`The initial size of panel 2. 0 - the size will be adjusted to Size 1.`,
						},
						def: 500,
						visibleIf: "layout!=S-1",
					},
					{
						key: "size3",
						control: "number",
						table: {
							hide: true,
						},
						group: {
							ru: "Главный экран",
							en: "Home screen",
						},
						caption: {
							ru: "Размер 3",
							en: "Size 3",
						},
						def: 500,
						visibleIf: "layout=H-12-34,V-12-34",
					},
					{
						key: "size4",
						control: "number",
						table: {
							hide: true,
						},
						group: {
							ru: "Главный экран",
							en: "Home screen",
						},
						caption: {
							ru: "Размер 4",
							en: "Size 4",
						},
						def: 500,
						visibleIf: "layout=H-12-34,V-12-34",
					},
					{
						key: "gap",
						control: "number",
						table: {
							hide: true,
						},
						group: {
							ru: "Главный экран",
							en: "Home screen",
						},
						caption: {
							ru: "Отступы",
							en: "Gap",
						},
						def: 10,
					},
					{
						key: "content1",
						control: "select",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 1",
							en: "Content 1",
						},
						caption: {
							ru: "Содержание",
							en: "Layout",
						},
						def: "info",
						list: [
							[
								"info",
								{
									ru: "Информация",
									en: "Info",
								},
							],
							[
								"part_list",
								{
									ru: "Список разделов",
									en: "Part list",
								},
							],
							[
								"project_part",
								{
									ru: "Раздел проекта",
									en: "Project part",
								},
							],
							[
								"first_screen",
								{
									ru: "Первый экран",
									en: "First screen",
								},
							],
							[
								"custom_screen",
								{
									ru: "Произвольный экран",
									en: "Custom screen",
								},
							],
							[
								"custom_widget",
								{
									ru: "Произвольный виджет",
									en: "Custom widget",
								},
							],
						],
					},
					{
						key: "project_part1",
						control: "select",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 1",
							en: "Content 1",
						},
						caption: {
							ru: "Раздел проекта",
							en: "Project part",
						},
						visibleIf: "content1=project_part",
						list: [
							[
								"screens",
								{
									ru: "Экраны",
									en: "Screens",
								},
							],
							[
								"nodes",
								{
									ru: "Узлы",
									en: "Nodes",
								},
							],
							[
								"journal",
								{
									ru: "Журнал",
									en: "Journal",
								},
							],
							[
								"alarms",
								{
									ru: "Тревоги",
									en: "Alarms",
								},
							],
							[
								"vars",
								{
									ru: "Переменные",
									en: "Variables",
								},
							],
						],
					},
					{
						key: "custom_screen1",
						control: "select",
						visibleIf: "content1=custom_screen",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 1",
							en: "Content 1",
						},
						caption: {
							ru: "Произвольный экран",
							en: "Custom screen",
						},
						dictionary: "screens",
					},
					{
						key: "custom_widget1",
						control: "input",
						visibleIf: "content1=custom_widget",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 1",
							en: "Content 1",
						},
						caption: {
							ru: "Произвольный виджет",
							en: "Custom widget",
						},
					},
					{
						key: "hide_header1",
						control: "checkbox",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 1",
							en: "Content 1",
						},
						caption: {
							ru: "Скрыть блок заголовка",
							en: "Hide header block",
						},
					},
					{
						key: "hide_name1",
						control: "checkbox",
						visibleIf: "!hide_header1",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 1",
							en: "Content 1",
						},
						caption: {
							ru: "Скрыть заголовок",
							en: "Hide title",
						},
					},
					{
						key: "hide_caption1",
						visibleIf: "!hide_header1",
						control: "checkbox",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 1",
							en: "Content 1",
						},
						caption: {
							ru: "Скрыть подпись",
							en: "Hide caption",
						},
					},
					{
						key: "name_text1",
						control: "input",
						visibleIf: "!hide_header1;!hide_name1",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 1",
							en: "Content 1",
						},
						caption: {
							ru: "Текст заголовка",
							en: "Header text",
						},
					},
					{
						key: "caption_text1",
						control: "input",
						visibleIf: "!hide_header1;!hide_caption1",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 1",
							en: "Content 1",
						},
						caption: {
							ru: "Текст подписи",
							en: "Caption text",
						},
					},
					{
						key: "content2",
						control: "select",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 2",
							en: "Content 2",
						},
						caption: {
							ru: "Содержание",
							en: "Layout",
						},
						visibleIf: "layout!=S-1",
						def: "part_list",
						list: [
							[
								"info",
								{
									ru: "Информация",
									en: "Info",
								},
							],
							[
								"part_list",
								{
									ru: "Список разделов",
									en: "Part list",
								},
							],
							[
								"project_part",
								{
									ru: "Раздел проекта",
									en: "Project part",
								},
							],
							[
								"first_screen",
								{
									ru: "Первый экран",
									en: "First screen",
								},
							],
							[
								"custom_screen",
								{
									ru: "Произвольный экран",
									en: "Custom screen",
								},
							],
							[
								"custom_widget",
								{
									ru: "Произвольный виджет",
									en: "Custom widget",
								},
							],
						],
					},
					{
						key: "project_part2",
						control: "select",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 2",
							en: "Content 2",
						},
						caption: {
							ru: "Раздел проекта",
							en: "Project part",
						},
						visibleIf: "content2=project_part;layout!=S-1",
						list: [
							[
								"screens",
								{
									ru: "Экраны",
									en: "Screens",
								},
							],
							[
								"nodes",
								{
									ru: "Узлы",
									en: "Nodes",
								},
							],
							[
								"journal",
								{
									ru: "Журнал",
									en: "Journal",
								},
							],
							[
								"alarms",
								{
									ru: "Тревоги",
									en: "Alarms",
								},
							],
							[
								"vars",
								{
									ru: "Переменные",
									en: "Variables",
								},
							],
						],
					},
					{
						key: "custom_screen2",
						control: "select",
						visibleIf: "layout!=S-1;content2=custom_screen",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 2",
							en: "Content 2",
						},
						caption: {
							ru: "Произвольный экран",
							en: "Custom screen",
						},
						dictionary: "screens",
					},
					{
						key: "custom_widget2",
						control: "input",
						visibleIf: "layout!=S-1;content2=custom_widget",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 2",
							en: "Content 2",
						},
						caption: {
							ru: "Произвольный виджет",
							en: "Custom widget",
						},
					},
					{
						key: "hide_header2",
						control: "checkbox",
						visibleIf: "layout!=S-1",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 2",
							en: "Content 2",
						},
						caption: {
							ru: "Скрыть блок заголовка",
							en: "Hide header block",
						},
					},
					{
						key: "hide_name2",
						control: "checkbox",
						visibleIf: "layout!=S-1;!hide_header2",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 2",
							en: "Content 2",
						},
						caption: {
							ru: "Скрыть заголовок",
							en: "Hide name",
						},
					},
					{
						key: "hide_caption2",
						control: "checkbox",
						visibleIf: "layout!=S-1;!hide_header2",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 2",
							en: "Content 2",
						},
						caption: {
							ru: "Скрыть подпись",
							en: "Hide caption",
						},
					},
					{
						key: "name_text2",
						control: "input",
						visibleIf: "layout!=S-1;!hide_header2;!hide_name2",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 2",
							en: "Content 2",
						},
						caption: {
							ru: "Текст заголовка",
							en: "Header text",
						},
					},
					{
						key: "caption_text2",
						control: "input",
						visibleIf: "layout!=S-1;!hide_header2;!hide_caption2",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 2",
							en: "Content 2",
						},
						caption: {
							ru: "Текст подписи",
							en: "Caption text",
						},
					},
					{
						key: "content3",
						control: "select",
						table: {
							hide: true,
						},
						visibleIf: "layout=H-12-3,H-12-34,V-12-3,V-12-34,H-3-12,V-3-12",
						group: {
							ru: "Содержание 3",
							en: "Content 3",
						},
						caption: {
							ru: "Содержание",
							en: "Layout",
						},
						def: "first_screen",
						list: [
							[
								"info",
								{
									ru: "Информация",
									en: "Info",
								},
							],
							[
								"part_list",
								{
									ru: "Список разделов",
									en: "Part list",
								},
							],
							[
								"project_part",
								{
									ru: "Раздел проекта",
									en: "Project part",
								},
							],
							[
								"first_screen",
								{
									ru: "Первый экран",
									en: "First screen",
								},
							],
							[
								"custom_screen",
								{
									ru: "Произвольный экран",
									en: "Custom screen",
								},
							],
							[
								"custom_widget",
								{
									ru: "Произвольный виджет",
									en: "Custom widget",
								},
							],
						],
					},
					{
						key: "project_part3",
						control: "select",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 3",
							en: "Content 3",
						},
						caption: {
							ru: "Раздел проекта",
							en: "Project part",
						},
						visibleIf: "layout=H-12-3,H-12-34,V-12-3,V-12-34,H-3-12,V-3-12;content3=project_part",
						list: [
							[
								"screens",
								{
									ru: "Экраны",
									en: "Screens",
								},
							],
							[
								"nodes",
								{
									ru: "Узлы",
									en: "Nodes",
								},
							],
							[
								"journal",
								{
									ru: "Журнал",
									en: "Journal",
								},
							],
							[
								"alarms",
								{
									ru: "Тревоги",
									en: "Alarms",
								},
							],
							[
								"vars",
								{
									ru: "Переменные",
									en: "Variables",
								},
							],
						],
					},
					{
						key: "custom_screen3",
						control: "select",
						visibleIf: "layout=H-12-3,H-12-34,V-12-3,V-12-34,H-3-12,V-3-12;content3=custom_screen",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 3",
							en: "Content 3",
						},
						caption: {
							ru: "Произвольный экран",
							en: "Custom screen",
						},
						dictionary: "screens",
					},
					{
						key: "custom_widget3",
						control: "input",
						visibleIf: "layout=H-12-3,H-12-34,V-12-3,V-12-34,H-3-12,V-3-12;content3=custom_widget",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 3",
							en: "Content 3",
						},
						caption: {
							ru: "Произвольный виджет",
							en: "Custom widget",
						},
					},
					{
						key: "hide_header3",
						control: "checkbox",
						visibleIf: "layout=H-12-3,H-12-34,V-12-3,V-12-34,H-3-12,V-3-12",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 3",
							en: "Content 3",
						},
						caption: {
							ru: "Скрыть блок заголовка",
							en: "Hide header block",
						},
					},
					{
						key: "hide_name3",
						control: "checkbox",
						visibleIf: "layout=H-12-3,H-12-34,V-12-3,V-12-34,H-3-12,V-3-12;!hide_header3",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 3",
							en: "Content 3",
						},
						caption: {
							ru: "Скрыть заголовок",
							en: "Hide name",
						},
					},
					{
						key: "hide_caption3",
						control: "checkbox",
						visibleIf: "layout=H-12-3,H-12-34,V-12-3,V-12-34,H-3-12,V-3-12;!hide_header3",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 3",
							en: "Content 3",
						},
						caption: {
							ru: "Скрыть подпись",
							en: "Hide caption",
						},
					},
					{
						key: "name_text3",
						control: "input",
						visibleIf: "layout=H-12-3,H-12-34,V-12-3,V-12-34,H-3-12,V-3-12;!hide_header3;!hide_name3",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 3",
							en: "Content 3",
						},
						caption: {
							ru: "Текст заголовка",
							en: "Header text",
						},
					},
					{
						key: "caption_text3",
						control: "input",
						visibleIf: "layout=H-12-3,H-12-34,V-12-3,V-12-34,H-3-12,V-3-12;!hide_header3;!hide_caption3",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 3",
							en: "Content 3",
						},
						caption: {
							ru: "Текст подписи",
							en: "Caption text",
						},
					},
					{
						key: "content4",
						control: "select",
						table: {
							hide: true,
						},
						visibleIf: "layout=H-12-34,V-12-34",
						group: {
							ru: "Содержание 4",
							en: "Content 4",
						},
						caption: {
							ru: "Содержание",
							en: "Layout",
						},
						def: "part_list",
						list: [
							[
								"info",
								{
									ru: "Информация",
									en: "Info",
								},
							],
							[
								"part_list",
								{
									ru: "Список разделов",
									en: "Part list",
								},
							],
							[
								"project_part",
								{
									ru: "Раздел проекта",
									en: "Project part",
								},
							],
							[
								"first_screen",
								{
									ru: "Первый экран",
									en: "First screen",
								},
							],
							[
								"custom_screen",
								{
									ru: "Произвольный экран",
									en: "Custom screen",
								},
							],
							[
								"custom_widget",
								{
									ru: "Произвольный виджет",
									en: "Custom widget",
								},
							],
						],
					},
					{
						key: "project_part4",
						control: "select",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 4",
							en: "Content 4",
						},
						caption: {
							ru: "Раздел проекта",
							en: "Project part",
						},
						visibleIf: "layout=H-12-34,V-12-34;content4=project_part",
						list: [
							[
								"screens",
								{
									ru: "Экраны",
									en: "Screens",
								},
							],
							[
								"nodes",
								{
									ru: "Узлы",
									en: "Nodes",
								},
							],
							[
								"journal",
								{
									ru: "Журнал",
									en: "Journal",
								},
							],
							[
								"alarms",
								{
									ru: "Тревоги",
									en: "Alarms",
								},
							],
							[
								"vars",
								{
									ru: "Переменные",
									en: "Variables",
								},
							],
						],
					},
					{
						key: "custom_screen4",
						control: "select",
						visibleIf: "layout=H-12-34,V-12-34;content4=custom_screen",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 4",
							en: "Content 4",
						},
						caption: {
							ru: "Произвольный экран",
							en: "Custom screen",
						},
						dictionary: "screens",
					},
					{
						key: "custom_widget4",
						control: "input",
						visibleIf: "layout=H-12-34,V-12-34;content4=custom_widget",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 4",
							en: "Content 4",
						},
						caption: {
							ru: "Произвольный виджет",
							en: "Custom widget",
						},
					},
					{
						key: "hide_header4",
						control: "checkbox",
						visibleIf: "layout=H-12-34,V-12-34",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 4",
							en: "Content 4",
						},
						caption: {
							ru: "Скрыть блок заголовка",
							en: "Hide header block",
						},
					},
					{
						key: "hide_name4",
						control: "checkbox",
						visibleIf: "layout=H-12-34,V-12-34;!hide_header4",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 4",
							en: "Content 4",
						},
						caption: {
							ru: "Скрыть заголовок",
							en: "Hide name",
						},
					},
					{
						key: "hide_caption4",
						control: "checkbox",
						visibleIf: "layout=H-12-34,V-12-34;!hide_header4",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 4",
							en: "Content 4",
						},
						caption: {
							ru: "Скрыть подпись",
							en: "Hide caption",
						},
					},
					{
						key: "name_text4",
						control: "input",
						visibleIf: "layout=H-12-34,V-12-34;!hide_header4;!hide_name4",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 4",
							en: "Content 4",
						},
						caption: {
							ru: "Текст заголовка",
							en: "Header text",
						},
					},
					{
						key: "caption_text4",
						control: "input",
						visibleIf: "layout=H-12-34,V-12-34;!hide_header4;!hide_caption4",
						table: {
							hide: true,
						},
						group: {
							ru: "Содержание 4",
							en: "Content 4",
						},
						caption: {
							ru: "Текст подписи",
							en: "Caption text",
						},
					},
				],
				allow_edit_children_list: false,
				editor: "#editor/project",
				interface: "#interface/project",
			},
			groups: {
				typename: {
					ru: "Группы",
					en: "Groups",
				},
				title: {
					ru: "Группы",
					en: "Groups",
				},
				typedef: {
					name: "groups",
					parents: [
						"project",
					],
				},
				list_description: {
					ru: "Группы пользователей проекта",
					en: "Project user groups",
				},
				help: {
					ru: "Группы пользователей проекта. Каждая группа содержит свой список пользователей. Пользователи получают доступ к элементам проекта в соответствии с разрешениями, установленными для групп, в которые они включены.",
					en: "Project user groups. Each group contains its own list of users. Users have access to project items based on the permissions set for the groups they are assigned to.",
				},
				icon: "/images/group.svg",
				access:["view","edit"],
				properties: [
					
					{
						key: "_type",
						caption: {
							ru: "Раздел",
							en: "Section",
						},
					},
					{
						key: "_title",
						hide: true,
					},
					{
						key: "alias",
						hide: false,
						
					},
					{
						key: "sort",
						hide: false,
					},
				],
			},

			schedules: {
				typename: {
					ru: "Расписания",
					en: "Schedules",
				},
				title: {
					ru: "Расписания",
					en: "Schedules",
				},
				typedef: {
					name: "schedules",
					parents: [
						"project",
					],
				},
				hideInEditorProjectWelcome: true,
				icon: "/images/simple_schedule.svg",
				interface: "#interface/schedules",
				interface_tree: false,
				access:["view","edit"],
				properties: [
				]
			},

			screens: {
				typename: {
					ru: "Экраны",
					en: "Screens",
				},
				title: {
					ru: "Экраны",
					en: "Screens",
				},
				typedef: {
					name: "screens",
					parents: [
						"project",
					],
				},
				help: {
					ru: "Дерево экранов отображается в интерфейсе пользователя. Экран связывает мнемосхему с одним или несколькими узлами, визуализирует значения переменных и предоставляет оператору интерфейс для управления узлом или инженерной системой.",
					en: "The screen tree is displayed in the user interface. The screen connects the mnemonic diagram with one or more nodes, visualizes the values of variables and provides the operator with an interface for controlling the node or engineering system.",
				},
				icon: "/images/screen.svg",
				interface: "#interface/browser",
				access:["view","edit"],
				properties: [
					commonParams.invisible,
					{
						key: "_type",
						caption: {
							ru: "Раздел",
							en: "Section",
						},
					},
					{
						key: "_title",
						hide: true,
					},
					{
						key: "alias",
						hide: false,
						
					},
					{
						key: "sort",
						hide: false,
					},
				],
			},
			mimics: {
				typename: {
					ru: "Мнемосхемы",
					en: "Mimics",
				},
				title: {
					ru: "Мнемосхемы",
					en: "Mimics",
				},
				typedef: {
					name: "mimics",
					parents: [
						"project",
					],
				},
				help: {
					ru: "Мнемосхема представляет собой шаблон экрана и является макетом графического интерфейса какого-либо устройств или инженерной системы. Каждая мнемосхема может применяться к одному или к нескольким экранам.",
					en: "A mnemonic diagram is a screen template and is a layout of the graphical interface of any device or engineering system. Each mimic can be applied to one or more screens.",
				},
				icon: "/images/mimic.svg",
				access:["view","edit"],
				properties: [
					{
						key: "_type",
						caption: {
							ru: "Раздел",
							en: "Section",
						},
					},
					{
						key: "_title",
						hide: true,
					},
					{
						key: "alias",
						hide: false,
						
					},
					{
						key: "sort",
						hide: false,
					},
				],
			},
			nodes: {
				typename: {
					ru: "Узлы",
					en: "Nodes",
				},
				title: {
					ru: "Узлы",
					en: "Nodes",
				},
				typedef: {
					name: "nodes",
					parents: [
						"project",
					],
				},
				help: {
					ru: "Дерево узлов проекта. Узлы наследуют параметры и список переменных в соответствии со своим прототипом. Каждый узел является логической единицей, может содержать переменные разных драйверов и работать с несколькими физическими узлами.",
					en: "Project node tree. Nodes inherit parameters and a list of variables according to their prototype. Each node is a logical unit, can contain variables from different drivers and work with multiple physical nodes.",
				},
				icon: "/images/node.svg",
				interface: "#interface/browser",
				properties: [
					commonParams.invisible,
					change(commonParams.hide_alarm_panel,{
						table:{
							hide:true,
						}
					}),
					{
						key: "_type",
						caption: {
							ru: "Раздел",
							en: "Section",
						},
					},
					{
						key: "_title",
						hide: true,
					},
					{
						key: "alias",
						hide: false,
						
					},
					{
						key: "sort",
						hide: false,
					},
				],
			},
			prototypes: {
				typename: {
					ru: "Прототипы",
					en: "Prototypes",
				},
				title: {
					ru: "Прототипы",
					en: "Prototypes",
				},
				typedef: {
					name: "prototypes",
					parents: [
						"project",
					],
				},
				help: {
					ru: "Прототипы узлов, в которых определяется набор параметров и список переменных узла. Каждый прототип может применяться к одному или к нескольким узлам. Один прототип может содержать переменные разных драйверов.",
					en: "Node prototypes that define a set of parameters and a list of node variables. Each prototype can be applied to one or more nodes. One prototype can contain variables from different drivers.",
				},
				icon: "/images/prototype.svg",
				access:["view","edit"],
				properties: [
					{
						key: "_type",
						caption: {
							ru: "Раздел",
							en: "Section",
						},
					},
					{
						key: "_title",
						hide: true,
					},
					{
						key: "alias",
						hide: false,
						
					},
					{
						key: "sort",
						hide: false,
					},
				],
			},
			files: {
				typename: {
					ru: "Файлы",
					en: "Files",
				},
				typedef: {
					name: "files",
					parents: [
						"project",
					],
				},
				help: {
					ru: "Хранилище файлов. Может быть использовано для организации и хранения проектной документации, схем, чертежей, прошивок контроллеров и т.п. Изображения, сохраненные в хранилище, доступны для использования в качестве иконок и элементов мнемосхем в данном проекте.",
					en: "File storage. Can be used to organize and store project documentation, diagrams, drawings, programs and firmware for controllers, etc. Images saved in the storage are available for use as icons and elements of mnemonic diagrams in this project.",
				},
				icon: "/images/folder.svg",
				allow_dragdrop: true,
				interface: "#interface/browser",
				access:["view","edit"],
				properties: [
					commonParams.invisible,
					{
						key: "_type",
						caption: {
							ru: "Раздел",
							en: "Section",
						},
					},
					{
						key: "_title",
						hide: true,
					},
					{
						key: "alias",
						hide: false,
						
					},
					{
						key: "sort",
						hide: false,
					},
				],
			},
			alarms: {
				typename: {
					ru: "Тревоги",
					en: "Alarms",
				},
				typedef: {
					name: "alarms",
					parents: [
						"project",
					],
				},
				allowed_docs: [
					"@access.json",
				],
				icon: "/images/alarms.svg",
				interface: "#interface/alarms",
				editor: "#editor/access",
				hideInEditorProjectWelcome: true,
				access:["view","edit"],
				properties: [
					commonParams.invisible,
					{
						key: "_type",
						
						caption: {
							ru: "Раздел",
							en: "Section",
						},
					},
					{
						key: "_title",
						hide: true,
					},
					{
						key: "alias",
						hide: false,
						
					},
				],
			},
			journal: {
				typename: {
					ru: "Журнал",
					en: "Journal",
				},
				typedef: {
					name: "journal",
					parents: [
						"project",
					],
				},
				allowed_docs: [
					"@access.json",
				],
				icon: "/images/journal.svg",
				interface: "#interface/journal",
				editor: "#editor/access",
				hideInEditorProjectWelcome: true,
				access:["view","edit"],
				properties: [
					commonParams.invisible,
					{
						key: "_type",
						caption: {
							ru: "Раздел",
							en: "Section",
						},
					},
					{
						key: "_title",
						hide: true,
					},
					{
						key: "alias",
						hide: false,
						
					},
				],
			},
			vars: {
				typename: {
					ru: "Переменные",
					en: "Variables",
				},
				typedef: {
					name: "vars",
					parents: [
						"project",
					],
				},
				allowed_docs: [
					"@access.json",
				],
				icon: "/images/var.svg",
				interface: "#interface/vars",
				editor: "#editor/access",
				hideInEditorProjectWelcome: true,
				access:["view","edit"],
				properties: [
					commonParams.invisible,
					{
						key: "_type",
						caption: {
							ru: "Раздел",
							en: "Section",
						},
					},
					{
						key: "_title",
						hide: true,
					},
					{
						key: "alias",
						hide: false,
						
					},
				],
			},

			dictionaries: {
				typename: {
					ru: "Справочники",
					en: "Dictionaries",
				},
				typedef: {
					name: "dictionaries",
					parents: [
						"project",
					],
				},
				hideInEditorProjectWelcome: true,
				icon: "/images/dictionary.svg",
				editor: "#editor/browser",
				access:["view","edit"],
				properties: [
					{
						key: "_type",
						caption: {
							ru: "Раздел",
							en: "Section",
						},
					},
					{
						key: "_title",
						hide: true,
					},
					{
						key: "alias",
						hide: false,
						
					},
					{
						key: "sort",
						hide: false,
					},
				],
			},

			dictionary: {
				typename: {
					ru: "Справочник",
					en: "Dictionary",
				},
				typedef: {
					ext: "dictionary",
					parents: [
						"dictionaries",
					],
				},
				allowed_docs: [
					"@prop.json",
				],
				access:["view","edit"],
				properties: [
					{
						key: "records",
						hide: true,
					},
				],
				exportable: true,
				create_and_open:false,
				icon: "/images/dictionary.svg",
				editor: "#editor/dictionary",
				save_doc:'@prop.json',
			},

			scheduledir: {
				typename: {
					ru: "Папка",
					en: "Folder",
				},
				typedef: {
					ext: null,
					parents: [
						"schedules",
						"scheduledir",
					],
				},
				access:["view","edit"],
				properties: [
					{
						key: "_title",
						validation: [
							"notEmpty",
							"pathNoDot",
						],
					},
					commonParams.inactive,
					{
						key: "sort",
						hide: false,
					},
				],
				icon: "/images/folder.svg",
				interface: "#interface/browser",
			},

			schedule: {
				typename: {
					ru: "Расписание",
					en: "Schedule",
				},
				typedef: {
					ext: "schedule",
					parents: [
						"scheduledir",
						"schedules",
					],
				},
				allowed_docs: [
					"@prop.json",
					"@content.json",
				],
				access:["view","edit"],
				properties: [
					commonParams.inactive,
				],
				exportable: true,
				create_and_open:false,
				icon: "/images/simple_schedule.svg",
				editor: "#editor/schedule",
				interface: "#interface/universal_schedule",
				save_doc:'@prop.json',
			},

			group: {
				typename: {
					ru: "Группа",
					en: "Group",
				},
				typedef: {
					ext: "group",
					parents: [
						"groups",
					],
				},
				allowed_docs: [
					"@prop.json",
				],
				icon: "/images/group.svg",
				editor: "#editor/group",
				need_build:true,
				access:["view","edit"],
				properties: [
					commonParams.inactive,
					{
						key: "users",
						caption: {
							ru: "Пользователи",
							en: "Users",
						},
						sort_array: "asc",
						props: {
							hide: true,
						},
					},
					{
						key: "index",
						hide: true,
					},
					{
						key: "comment",
						bottom: false,
					},
					{
						key:'tag_filter',
						group:{en:'Tag filter',ru:'Фильтр по тегам'},
						caption:'',
						control:'edit',
						wordwrap: true,
						multiline:true,
						minheight:50,
						groupHelp: {
							ru:`
							Логическое выражение с именами тегов. Если выражение задано, пользователям группы будут доступны только те переменные и тревоги, которые по набору тегов удовлетворяют данному выражению.
		
							Пример 1:
							floor3 & (ventilation | heating)
		
							В данном примере группе будут доступны переменные, если в их тегах содержится тег <em>floor3</em> И один из тегов <em>ventilation</em> ИЛИ <em>heating</em>
		
							Пример 2:
							ventilation & !fire_alarm
		
							В данном примере группе будут доступны переменные, если в их тегах содержится тег <em>ventilation</em> И НЕ содержится тег <em>fire_alarm</em>
		
							Пример 3:
							ventilation | heating | cooling
		
							В данном примере группе будут доступны переменные, если в их тегах содержится хотя бы один из перечисленных тегов
							`,
							en:`
							A logical expression with tag names. If an expression is specified, only those variables and alarms that, based on the set of tags, satisfy this expression will be available to users of the group.

							Example 1:
							floor3 & (ventilation | heating)

							In this example, variables will be available to the group if their tags contain the <em>floor3</em> tag AND one of the <em>ventilation</em> OR <em>heating</em> tags

							Example 2:
							ventilation & !fire_alarm

							In this example, variables will be available to the group if their tags contain the <em>ventilation</em> tag AND DO NOT contain the <em>fire_alarm</em> tag

							Example 3:
							ventilation | heating | cooling

							In this example, variables will be available to the group if their tags contain at least one of the listed tags
							`
						}
					},
				],
			},
			nodesdir: {
				typename: {
					ru: "Папка",
					en: "Folder",
				},
				typedef: {
					ext: null,
					parents: [
						"nodes",
						"nodesdir",
					],
				},
				properties: [
					{
						key: "_title",
						validation: [
							"notEmpty",
							"pathNoDot",
						],
					},
					commonParams.tags,
					commonParams.inactive,
					commonParams.invisible,
					change(commonParams.show_header,{group:undefined}),
					commonParams.hide_alarm_panel,
					{
						key: "sort",
						hide: false,
					},
				],
				need_build:true,
				icon: "/images/folder.svg",
				interface: "#interface/browser",
			},
			nodedir: {
				typename: {
					ru: "Папка",
					en: "Folder",
				},
				typedef: {
					ext: null,
					parents: [
						"node",
						"nodedir",
					],
				},
				properties: [
					{
						key: "_title",
						validation: [
							"notEmpty",
							"pathNoDot",
						],
					},
					commonParams.tags,
					commonParams.inactive,
					commonParams.invisible,
					change(commonParams.show_header,{group:undefined}),
					commonParams.hide_alarm_panel,
					{
						key: "sort",
						hide: false,
					},
				],
				need_build:true,
				icon: "/images/folder.svg",
				interface: "#interface/browser",
			},
			node: {
				typename: {
					ru: "Узел",
					en: "Node",
				},
				typedef: {
					ext: "node",
					parents: [
						"nodes",
						"node",
						"gateway",
						"nodesdir",
						"nodedir",
						"gatewaydir",
					],
				},
				icon: "/images/node.svg",
				interface: "#interface/node",
				editor: "#editor/node",
				exportable: true,
				need_build:true,
				properties: [
					
					// {
					// 	key: "_prefix",
					// 	disabled:true,
					// 	hide:false,
					// },
					commonParams.prototype,
					commonParams.tags,
					commonParams.inactive,
					commonParams.invisible,
					commonParams.show_header,
					change(commonParams.hide_alarm_panel,{
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
						table:{
							hide:true,
						}
					}),
					{
						key: "icon",
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
					},
					{
						key: "index",
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
					},
					{
						key: "comment",
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
					},
					
				],
			},
			gateway: {
				typename: {
					ru: "Шлюз",
					en: "Gateway",
				},
				typedef: {
					ext: "gateway",
					parents: [
						"nodes",
						"nodesdir",
					],
				},
				icon: "/images/gateway.svg",
				interface: "#interface/gateway",
				editor: "#editor/gateway",
				exportable: true,
				need_build:true,
				properties: [
					
					// {
					// 	key: "_prefix",
					// 	disabled:true,
					// 	hide:false,
					// },
					{
						key: "mac",
						caption: "MAC",
						control:'edit',
						dictionary:'macs',
						validation: [
							"mac",
							"notEmpty",
						],
						help:{
							ru:`
							Уникальный идентификатор шлюза. Задается в настройках шлюза и обычно соответствует MAC-адресу одного из его ethernet-интерфейсов. Представляет собой шестнадцатиричную строку, состоящую из символов 0-9,A,B,C,D,E,F.
							`,
							en:`
							Unique gateway identifier. It is set in the gateway settings and usually corresponds to the MAC address of one of its ethernet interfaces. Represents a hexadecimal string consisting of characters 0-9,A,B,C,D,E,F.
							`
						}
					},

					commonParams.prototype,	
					commonParams.tags,
					commonParams.inactive,
					commonParams.invisible,
					commonParams.show_header,
					change(commonParams.hide_alarm_panel,{
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
						table:{
							hide:true,
						}
					}),
					{
						key: "sort",
						hide: false,
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
					},
					{
						key: "icon",
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
					},
					{
						key: "index",
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
					},
					{
						key: "comment",
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
					},
				],
			},
			gatewaydir: {
				typename: {
					ru: "Папка",
					en: "Folder",
				},
				typedef: {
					ext: null,
					parents: [
						"gateway",
						"gatewaydir",
					],
				},
				properties: [
					{
						key: "_title",
						validation: [
							"notEmpty",
							"pathNoDot",
						],
					},
					commonParams.tags,
					commonParams.inactive,	
					commonParams.invisible,
					change(commonParams.show_header,{group:undefined}),
					commonParams.hide_alarm_panel,
					{
						key: "sort",
						hide: false,
					},
				],
				icon: "/images/folder.svg",
				interface: "#interface/browser",
				exportable: true,
			},
			screen: {
				typename: {
					ru: "Экран",
					en: "Screen",
				},
				typedef: {
					ext: "screen",
					parents: [
						"screens",
						"screen",
						"screendir",
					],
				},
				icon: "/images/screen.svg",
				interface: "#interface/screen",
				editor: "#editor/screen",
				exportable: true,
				need_build:true,
				create_and_open:false,
				access:["view","edit"],
				properties: [
					commonParams.invisible,
					{
						key: "content",
						caption: {
							ru: "Содержание",
							en: "Content",
						},
						list: [
							[
								"mimics",
								{
									ru: "Мнемосхемы",
									en: "Mimics",
								},
							],
							[
								"html",
								{
									ru: "Произвольный HTML",
									en: "Custom HTML",
								},
							],
						],
						def: "mimics",
						control: "select",
						table:{
							visibleIf: "_type=screen",
						}
					},
					{
						group: {
							ru: "Мнемосхемы",
							en: "Mimics",
						},
						key: "desktop_mimic",
						caption: {
							ru: "Десктопная",
							en: "Desktop",
						},
						dictionary: "mimics",
						dictionary_link: '#editor/mimic?path=${project}/mimics/${value}',
						control: "select",
						visibleIf: "content=mimics",
						extra_properties: "mimics",
					},
					{
						group: {
							ru: "Мнемосхемы",
							en: "Mimics",
						},
						key: "mobile_mimic",
						caption: {
							ru: "Мобильная",
							en: "Mobile",
						},
						dictionary: "mimics",
						dictionary_link: '#editor/mimic?path=${project}/mimics/${value}',
						control: "select",
						visibleIf: "content=mimics",
						extra_properties: "mimics",
					},
					{
						group: {
							ru: "Произвольный HTML",
							en: "Custom HTML",
						},
						key: "html_file",
						caption: {
							ru: "HTML-файл",
							en: "HTML-file",
						},
						dictionary: "html_files",
						noCache: true,
						filesubst: true,
						control: "edit",
						visibleIf: "content=html",
						help: {
							ru: "Файл или папка из ресурсов экрана или из файлов проекта. Если указана папка - в качестве результирующего HTML-файла будет представлен код всех HTML-файлов в данной папке.",
							en: "A file or folder from screen resources or project files. If a folder is specified, the resulting HTML file will contain the code of all HTML files in this folder.",
						},
					},
					{
						group: {
							ru: "Произвольный HTML",
							en: "Custom HTML",
						},
						key: "js_file",
						caption: {
							ru: "JS-файл",
							en: "JS-file",
						},
						dictionary: "js_files",
						noCache: true,
						filesubst: true,
						control: "edit",
						visibleIf: "content=html",
						help: {
							ru: "Файл или папка из ресурсов экрана или из файлов проекта. Если указана папка - в качестве результирующего JS-файла будет представлен код всех JS-файлов в данной папке.",
							en: "A file or folder from screen resources or project files. If a folder is specified, the resulting JS file will contain the code of all JS files in this folder.",
						},
					},
					{
						group: {
							ru: "Произвольный HTML",
							en: "Custom HTML",
						},
						key: "css_file",
						caption: {
							ru: "CSS-файл",
							en: "CSS-file",
						},
						dictionary: "css_files",
						noCache: true,
						filesubst: true,
						control: "edit",
						visibleIf: "content=html",
						help: {
							ru: "Файл или папка из ресурсов экрана или из файлов проекта. Если указана папка - в качестве результирующего CSS-файла будет представлен код всех CSS-файлов в данной папке.",
							en: "A file or folder from screen resources or project files. If a folder is specified, the resulting CSS file will contain the code of all CSS files in this folder.",
						},
					},

					{
						group: {ru: "Тревоги",en: "Alarms"},
						key: "alarm_list",
						caption: {ru: "Список тревог",en: "Alarm list"},
						control: "select",
						undef:true,
						list:[
							['by_mimic',{ru:'По настройкам мнемосхемы',en:'By mimic settings'}],
							['mimic_nodes',{ru:'По узлам мнемосхемы',en:'By mimic nodes'}],
							['project_nodes',{ru:'По узлам проекта',en:'By project nodes'}],
							['all_nodes',{ru:'По всем узлам в системе',en:'By all nodes in system'}],
						],
						help:{
							ru:`
							Параметр определяет, на основе тревог каких узлов будет сформирован список тревог в панели тревог экрана, связанного с данной мнемосхемой. Сформированный список может быть ограничен фильтром по тегам и фильтром по узлам, если они заданы.

							По узлам мнемосхемы - список тревог формируется из тревог всех узлов, привязанных к экрану, который связан с данной мнемосхемой.

							По узлам проекта - список тревог формируется из тревог всех активных узлов проекта.
							
							По всем узлам в системе - список тревог формируется из тревог всех активных узлов всех активных проектов системы.
							`,
							en:`
							The parameter defines the alarms of which nodes will be used to generate the alarm list in the alarm panel of the screen associated with this mnemonic diagram. The generated list can be limited by a filter by tags and a filter by nodes, if they are specified.

							By mnemonic diagram nodes - the alarm list is generated from the alarms of all nodes linked to the screen associated with this mnemonic diagram.

							By project nodes - the alarm list is generated from the alarms of all active project nodes.

							By all nodes in the system - the alarm list is generated from the alarms of all active nodes of all active projects in the system.
							`
						},
						def:'by_mimic'
					},

					{
						group: {ru: "Тревоги",en: "Alarms"},
						key: "alarm_node_filter",
						caption: {ru: "Фильтр по узлам",en: "Node filter"},
						control: "input",
						placeholder:{ru:'По настройкам мнемосхемы',en:'By mimic settings'},
						help:{
							ru:`
							Если параметр задан, сформированный список тревог будет отфильтрован по полному пути узлов: полный путь должен соответствовать строке параметра.
							Допускается использование символов подстановки * и ?.

							Например: */Вентиляция *.project/nodes/*/Вытяжка ??.node
							`,
							en:`
							If the parameter is specified, the generated alarm list will be filtered by the full path of the nodes: the full path must match the parameter string.
							Wildcards * and ? are allowed.

							For example: */Ventilation *.project/nodes/*/Exhaust ??.node
							`
						},
					},
					
					{
						group: {ru: "Тревоги",en: "Alarms"},
						key: "alarm_tag_filter",
						caption: {ru: "Фильтр по тегам",en: "Tag filter"},
						control: "input",
						placeholder:{ru:'По настройкам мнемосхемы',en:'By mimic settings'},
						help:{
							ru:`
							Логическое выражение с именами тегов. Если выражение задано, список тревог будет ограничен только теми тревогами, которые по набору тегов удовлетворяют данному выражению.
			
							Примеры:
			
							fire_alarm
							ventilation & !fire_alarm
							ventilation | heating | cooling
			
							`,
							en:`
							A logical expression with tag names. If the expression is specified, the alarm list will be limited to only those alarms that satisfy the expression by the set of tags.

							Examples:

							fire_alarm
							ventilation & !fire_alarm
							ventilation | heating | cooling
							`
						}
					},
					
					{
						key: "sort",
						hide: false,
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
					},
					{
						key: "icon",
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
					},
					{
						key: "index",
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
					},
					{
						key: "comment",
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
					},

				],
			},
			resources: {
				typename: {
					ru: "Ресурсы",
					en: "Resources",
				},
				typedef: {
					ext: null,
					name: "resources",
					parents: [
						"screen",
						"mimic",
					],
				},
				icon: "/images/resources.svg",
				allow_dragdrop: true,
				exportable: true,
				create_and_open:false,
				access:["view","edit"],
				properties: [
					{
						key: "_title",
						hide: true,
					},
					{
						key: "inactive",
						hide: true,
					},
					{
						key: "sort",
						hide: false,
					},
				],
			},
			screendir: {
				typename: {
					ru: "Папка",
					en: "Folder",
				},
				typedef: {
					ext: null,
					parents: [
						"screens",
						"screen",
						"screendir",
					],
				},
				access:["view","edit"],
				properties: [
					{
						key: "_title",
						validation: [
							"notEmpty",
							"pathNoDot",
						],
					},
					commonParams.invisible,
					{
						key: "sort",
						hide: false,
					},
				],
				icon: "/images/folder.svg",
				interface: "#interface/browser",
				exportable: true,
			},
			resourcedir: {
				typename: {
					ru: "Папка",
					en: "Folder",
				},
				typedef: {
					container:'folder',
					parents: [
						"resources",
						"files",
						"resourcedir",
					],
				},
				access:["view","edit"],
				properties: [
					{
						key: "_title",
						validation: [
							"notEmpty",
							"pathNoDot",
						],
					},
					{
						key: "sort",
						hide: false,
					},
				],
				fullname:true,
				allow_dragdrop: true,
				icon: "/images/folder.svg",
				interface: "#interface/browser",
				exportable: true,
			},
			resourcefile: {
				typename: {
					ru: "Файл",
					en: "File",
				},
				allowed_docs: [
					"@content",
				],
				fileDialog: true,
				typedef: {
					container: "file",
					ext: "*",
					parents: [
						"resources",
						"files",
						"resourcedir",
					],
				},
				fullname:true,
				icon: "/images/file.svg",
				interface: "#interface/file",
				exportable: true,
				properties: [
					{
						key: "inactive",
						hide: true,
					},
					{
						key: "icon",
						hide: true,
					},
					{
						key: "comment",
						hide: true,
					},
					{
						key: "index",
						hide: true,
					},
				],
			},
			mimic: {
				typename: {
					ru: "Мнемосхема",
					en: "Mimic",
				},
				typedef: {
					ext: "mimic",
					parents: [
						"mimics",
						"mimicdir",
					],
				},
				allowed_docs: [
					"@prop.json",
					"@access.json",
					"@content.html",
				],
				access:["view","edit"],
				properties: [
					{
						key: "menu",
						hide: true,
					},
					{
						key: "inactive",
						hide: true,
					},
					{
						key: "debug",
						hide: true,
					},
					{
						key: "option_show_header",
						group:{ru:"Интерфейс",en:"Interface"},
						caption:{ru:"Заголовок окна",en:"Window header"},
						control:"select",
						list:[
							['show',{ru:'Показать',en:'Show'}],
							['hide',{ru:'Скрыть',en:'Hide'}],
						],
						def:'show',
						undef:true,
					},
					{
						key: "option_show_menu",
						group:{ru:"Интерфейс",en:"Interface"},
						caption:{ru:"Меню мнемосхемы",en:"Mimic menu"},
						control:"select",
						list:[
							['main',{ru:'Меню главной мнемосхемы',en:'Menu of main mimic'}],
							['sub',{ru:'Меню связанной мнемосхемы',en:'Menu of linked mimic'}],
							['hide',{ru:'Скрыть',en:'Hide'}],
						],
						def:'main',
						undef:true,
					},
					{
						key: "option_show_alarms",
						group:{ru:"Интерфейс",en:"Interface"},
						caption:{ru:"Панель тревог",en:"Alarm panel"},
						control:"select",
						list:[
							['show',{ru:'Показать',en:'Show'}],
							['hide',{ru:'Скрыть',en:'Hide'}],
						],
						def:'show',
						undef:true,
					},
					
					{
						key: "option_scale",
						group:{ru:"Интерфейс",en:"Interface"},
						caption:{ru:"Масштабирование",en:"Scaling"},
						control:"select",
						list:[
							[0,{ru:'Не масштабировать (прокрутка)',en:'No scaling (scroll)'}],
							[1,{ru:'Вписать по ширине',en:'Fit width'}],
							[2,{ru:'Вписать по высоте',en:'Fit height'}],
							[3,{ru:'Вписать по ширине и высоте',en:'Fit width and height'}],
						],
						def:0,
						undef:true,
					},

					{
						key: "option_hor_align",
						group:{ru:"Интерфейс",en:"Interface"},
						caption:{ru:"Гор. выравнивание",en:"Hor. alignment"},
						control:"select",
						list:[
							['left',{ru:'Слева',en:'Left'}],
							['center',{ru:'По центру',en:'Center'}],
							['right',{ru:'Справа',en:'Right'}],
						],
						def:'left',
						undef:true,
						help:{
							ru:`
							Выравнивание содержания мнемосхемы на экране.
							Для корректной работы данной опции необходимо на вкладке Параметры задать фиксированную ширину мнемосхемы.
							`,
							en:`
							Alignment of the mnemonic diagram content on the screen.
							For this option to work correctly, you must set a fixed width for the mnemonic diagram on the Parameters tab.
							`
						},
					},

					{
						key: "option_ver_align",
						group:{ru:"Интерфейс",en:"Interface"},
						caption:{ru:"Верт. выравнивание",en:"Vert. alignment"},
						control:"select",
						list:[
							['top',{ru:'Сверху',en:'Top'}],
							['center',{ru:'По центру',en:'Center'}],
							['bottom',{ru:'Снизу',en:'Bottom'}],
						],
						def:'top',
						undef:true,
						help:{
							ru:`
							Выравнивание содержания мнемосхемы на экране.
							Для корректной работы данной опции необходимо на вкладке Параметры задать фиксированную высоту мнемосхемы.
							`,
							en:`
							Alignment of the mnemonic diagram content on the screen.
							For this option to work correctly, you must set a fixed height for the mnemonic diagram on the Parameters tab.
							`
						},
					},

					{
						key: "option_unlinked_nodes",
						group:{ru:"Интерфейс",en:"Interface"},
						caption:{ru:"Непривязанные узлы",en:"Unlinked nodes"},
						control:"select",
						list:[
							['error',{ru:'Показывать ошибку',en:'Show error'}],
							['ignore',{ru:'Игнорировать',en:'Ignore'}],
						],
						def:'error',
						undef:true,
					},
					{
						group: {ru: "Тревоги",en: "Alarms"},
						key: "alarm_list",
						caption: {ru: "Список тревог",en: "Alarm list"},
						control: "select",
						list:[
							['mimic_nodes',{ru:'По узлам мнемосхемы',en:'By mimic nodes'}],
							['project_nodes',{ru:'По узлам проекта',en:'By project nodes'}],
							['all_nodes',{ru:'По всем узлам в системе',en:'By all nodes in system'}],
						],
						help:{
							ru:`
							Параметр определяет, на основе тревог каких узлов будет сформирован список тревог в панели тревог экрана, связанного с данной мнемосхемой. Сформированный список может быть ограничен фильтром по тегам и фильтром по узлам, если они заданы.

							По узлам мнемосхемы - список тревог формируется из тревог всех узлов, привязанных к экрану, который связан с данной мнемосхемой.

							По узлам проекта - список тревог формируется из тревог всех активных узлов проекта.
							
							По всем узлам в системе - список тревог формируется из тревог всех активных узлов всех активных проектов системы.
							`,
							en:`
							The parameter defines the alarms of which nodes will be used to generate the alarm list in the alarm panel of the screen associated with this mnemonic diagram. The generated list can be limited by a filter by tags and a filter by nodes, if they are specified.

							By mnemonic diagram nodes - the alarm list is generated from the alarms of all nodes linked to the screen associated with this mnemonic diagram.

							By project nodes - the alarm list is generated from the alarms of all active project nodes.

							By all nodes in the system - the alarm list is generated from the alarms of all active nodes of all active projects in the system.
							`
						},
						def:'mimic_nodes'
					},

					{
						group: {ru: "Тревоги",en: "Alarms"},
						key: "alarm_node_filter",
						caption: {ru: "Фильтр по узлам",en: "Node filter"},
						control: "input",
						help:{
							ru:`
							Если параметр задан, сформированный список тревог будет отфильтрован по полному пути узлов: полный путь должен соответствовать строке параметра.
							Допускается использование символов подстановки * и ?.

							Например: */Вентиляция *.project/nodes/*/Вытяжка ??.node
							`,
							en:`
							If the parameter is specified, the generated alarm list will be filtered by the full path of the nodes: the full path must match the parameter string.
							Wildcards * and ? are allowed.

							For example: */Ventilation *.project/nodes/*/Exhaust ??.node
							`
						},
					},
					
					{
						group: {ru: "Тревоги",en: "Alarms"},
						key: "alarm_tag_filter",
						caption: {ru: "Фильтр по тегам",en: "Tag filter"},
						control: "input",
						help:{
							ru:`
							Логическое выражение с именами тегов. Если выражение задано, список тревог будет ограничен только теми тревогами, которые по набору тегов удовлетворяют данному выражению.
			
							Примеры:
			
							fire_alarm
							ventilation & !fire_alarm
							ventilation | heating | cooling
			
							`,
							en:`
							A logical expression with tag names. If the expression is specified, the alarm list will be limited to only those alarms that satisfy the expression by the set of tags.

							Examples:

							fire_alarm
							ventilation & !fire_alarm
							ventilation | heating | cooling
							`
						}
					},

					{
						key: "status_node",
						group:{ru: "Статус",en: "Status"},
						caption: {ru: "Узел",en: "Node"},
						validation: [
							"latinNum",
						],
						help:{
							ru:`
							Имя узла (так, как он указан в параметрах мнемосхемы) в котором будет наблюдаться переменная статуса.
							`,
							en:`
							The name of the node (as specified in the mimic parameters) in which the status variable will be observed.
							`
						}
					},

					change(commonParams.status_var,{caption:{ru: "Переменная",en: "Var"},group:{ru: "Статус",en: "Status"}}),

				],
				icon: "/images/mimic.svg",
				editor: "#editor/mimic",
				create_and_open:false,
				exportable: true,
				need_build:true,
				save_doc:'@prop.json',
			},
			mimicdir: {
				typename: {
					ru: "Папка",
					en: "Folder",
				},
				typedef: {
					ext: null,
					parents: [
						"mimics",
						"mimicdir",
					],
				},
				access:["view","edit"],
				properties: [
					{
						key: "_title",
						validation: [
							"notEmpty",
							"pathNoDot",
						],
					},
					{
						key: "sort",
						hide: false,
					},
				],
				icon: "/images/folder.svg",
				exportable: true,
			},
			prototype: {
				typename: {
					ru: "Прототип",
					en: "Prototype",
				},
				typedef: {
					ext: "prototype",
					parents: [
						"prototypes",
						"prototypedir",
					],
				},
				allowed_docs: [
					"@prop.json",
					"@access.json",
					"@content.json",
				],
				access:["view","edit"],
				properties: [
					{
						key: "inactive",
						hide: true,
					},
					commonParams.status_var,
					commonParams.tags,				
					{
						key: "interface",
						caption:{
							ru:"Интерфейс",
							en:"Interface",
						},
						control:"select",
						list:[
							['node',{ru:'Стандартный узел',en:'Standart node'}],
							['mimic',{ru:'Мнемосхема',en:'Mimic'}],
						],
						def:'node',
						group:{ru:"Интерфейс",en:"Interface"},
						help:{
							ru:`
							Стандартный узел - клик в интерфейсе по узлу, связанному с данным прототипом, откроет стандартое окно узла (с параметрами узла и окном переменных).

							Мнемосхема - клик в интерфейсе по узлу, связанному с данным прототипом, откроет указанную мнемосхему.
							`,
							en:`
							Standard node - clicking on the node associated with this prototype in the interface will open the standard node window (with node parameters and the variables window).

							Mimic - clicking on the node associated with this prototype in the interface will open the specified mnemonic diagram.
							`
						}
					},
					{
						key:'desktop_mimic',
						group:{ru:"Интерфейс",en:"Interface"},
						caption:{en:'Desktop mimic',ru:'Десктопная мнемосхема'},
						visibleIf:'interface=mimic',
						dictionary: "mimics",
						dictionary_link: '#editor/mimic?path=${project}/mimics/${value}',
						control: "select",
					},
					{
						key:'mobile_mimic',
						group:{ru:"Интерфейс",en:"Interface"},
						caption:{en:'Mobile mimic',ru:'Мобильная мнемосхема'},
						visibleIf:'interface=mimic',
						dictionary: "mimics",
						dictionary_link: '#editor/mimic?path=${project}/mimics/${value}',
						control: "select",
					},
					{
						key: "icon",
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
					},
					{
						key: "index",
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
					},
					{
						key: "comment",
						group:{
							ru:"Разное",
							en:"Miscellaneous",
						},
					},
				],
				need_build:true,
				create_and_open:false,
				templates: "prototypes",
				save_doc:'@content.json',
				editor: "#editor/prototype",
				icon: "/images/prototype.svg",
				exportable: true,
			},
			prototypedir: {
				typename: {
					ru: "Папка",
					en: "Folder",
				},
				typedef: {
					ext: null,
					parents: [
						"prototypes",
						"prototypedir",
					],
				},
				access:["view","edit"],
				properties: [
					{
						key: "_title",
						validation: [
							"notEmpty",
							"pathNoDot",
						],
					},
					{
						key: "sort",
						hide: false,
					},
					{
						key: "inactive",
						hide: true,
					},
				],
				icon: "/images/folder.svg",
				exportable: true,
			},
			file: {
				typename: {
					ru: "Файл",
					en: "File",
				},
				typedef: {
					container: "file",
				},
				interface: "#interface/browser",
				allowed_docs: [
					"@content",
				],
				fullname:true,
				icon: "/images/file.svg",
				exportable: true,
			},
			folder: {
				typename: {
					ru: "Папка",
					en: "Folder",
				},
				access:["view","edit"],
				properties: [
					commonParams.invisible,
					{
						key: "sort",
						hide: false,
					},
				],
				fullname:true,
				icon: "/images/folder.svg",
				interface: "#interface/browser",
				exportable: true,
			},
			root: {
				fill: [
					"projects",
					"users",
				],
			},

			mimic_menu: mimic_menu,
			mimic_param: mimic_param,

			prototype_param: prototype_param,
			prototype_report: prototype_report,
			prototype_script: prototype_script,
			prototype_report_indicator: prototype_report_indicator,
			dictionary_record: dictionary_record,
			
			prototype_drv_alarm: drv_alarm,
			prototype_drv_bacnet: drv_bacnet,
			prototype_drv_cam: drv_cam,
			prototype_drv_cmd: drv_cmd,
			prototype_drv_gpsd: drv_gpsd,
			prototype_drv_gateway: drv_gateway,
			prototype_drv_http: drv_http,
			prototype_drv_mbrtu: drv_mbrtu,
			prototype_drv_mbtcp: drv_mbtcp,
			prototype_drv_mbrot: drv_mbrot,
			prototype_drv_mqtt: drv_mqtt,
			prototype_drv_opcua: drv_opcua,
			prototype_drv_snmp: drv_snmp,
			prototype_drv_virt: drv_virt,
		

		},
		sysmenu: [
			{
				text: {
					ru: "Пользователи",
					en: "Users",
				},
				href: "#interface/users",
				path: "/system/users",
				icon: "/images/users.svg",
			},
			{
				text: {
					ru: "Переменные",
					en: "Vars",
				},
				href: "#interface/vars",
				path: "/system/vars",
				icon: "/images/var.svg",
			},
			{
				text: {
					ru: "Журнал",
					en: "Journal",
				},
				href: "#interface/journal",
				path: "/system/journal",
				icon: "/images/journal.svg",
			},
			{
				text: {
					ru: "Шлюзы",
					en: "Gateways",
				},
				href: "#interface/gateways",
				path: "/system/gateways",
				icon: "/images/gateway.svg",
			},
			{
				text: {
					ru: "Настройки",
					en: "Settings",
				},
				href: "#interface/settings",
				path: "/system/settings",
				icon: "/images/settings.svg",
			},
			{
				text: {
					ru: "Логи",
					en: "Logs",
				},
				href: "#interface/logs",
				path: "/system/logs",
				icon: "/images/log.svg",
			},
			{
				text: {
					ru: "Время",
					en: "Time",
				},
				href: "#interface/time",
				path: "/system/time",
				icon: "/images/time.svg",
			},
			{
				text: {
					ru: "Перезагрузка",
					en: "Reboot",
				},
				href: "#interface/reboot",
				path: "/system/reboot",
				icon: "/images/reboot.svg",
			},
			{
				text: {
					ru: "Лицензия",
					en: "License",
				},
				href: "#interface/license",
				path: "/system/license",
				icon: "/images/license.svg",
			},
			{
				text: {
					ru: "DDoS-фильтр",
					en: "DDoS-filter",
				},
				href: "#interface/ddos",
				path: "/system/ddos",
				icon: "/images/ddos.svg",
			},
			{
				text: {
					ru: "Календарь",
					en: "Calendar",
				},
				href: "#interface/calendar",
				path: "/system/calendar",
				icon: "/images/calendar.svg",
			},
			{
				text: {
					ru: "Удаленный доступ",
					en: "Remote access",
				},
				href: "#interface/remote",
				path: "/system/remote",
				icon: "/images/remote.svg",
			},
		],
	}
})();
